/*
  This software is available to you under a choice of one of two
  licenses.  You may choose to be licensed under the terms of the GNU
  General Public License (GPL) Version 2, available at
  <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
  license, available in the LICENSE.TXT file accompanying this
  software.  These details are also available at
  <http://openib.org/license.html>.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

  Copyright (c) 2004 Mellanox Technologies Ltd.  All rights reserved.
*/

#ifndef __MOSAL_IOBUF_IMP__H
#define __MOSAL_IOBUF_IMP__H

#ifdef MT_KERNEL
#include <mosal.h>
#include <linux/list.h>
#include <mosal_iobuf.h>

#define MIN_ELEM_NUM 2
#define ELEMS_IN_MAP_ITEM (MIN_ELEM_NUM+1000)

typedef struct map_item_st {
	struct map_item_st *next;	/* link to next item */
	unsigned int elem_count;	/* number of valid elements in the list */
	struct page *arr[0];
} map_item_t;

typedef struct map_list_st {
	struct map_item_st *head;
	atomic_t ref_count;
} map_list_t;

union pages_list_u {
	map_list_t *map_list;
	struct page *page;
	MT_phys_addr_t pa;
};

struct mosal_iobuf_st {
	MT_virt_addr_t va;	/* virtual address of the buffer */
	MT_size_t size;		/* size in bytes of the buffer */
	MOSAL_prot_ctx_t prot_ctx; /* protection context */
	MOSAL_mem_perm_t req_perm; /* permission requested for the region */
	unsigned int page_shift; /* page shift for the region */
	union pages_list_u  map_u; /* pointer to list object holding page descriptors */
	int phys_ctg;		/* 1 if physically contigous i.e. kmalloced */
	int need_page_secure;   /* 1 if get/put_page() are used for the pages */

	MOSAL_iobuf_t ref_iobuf; /* iobuf object to share with */
	int   file_shared;       /* 1 if shared due to shared file */
	struct file *file_p;
	unsigned long file_pgoff;     /* Offset (within vm_file) in PAGE_SIZE */


	int iomem;               /* 1 if region is memory mapped IO */

	void *mlock_ctx;
	struct mosal_iobuf_st *prev;	/* used to link mosal_iobufs */
	struct mosal_iobuf_st *next;	/* used to link mosal_iobufs */
};

/* iterator for getting segments of tpt */
struct mosal_iobuf_iter_st {
	map_item_t *item_p;	/* the item from where to take the next translations */
	unsigned int elem_idx;	/* index from where to take the next translation */
};

#endif

call_result_t MOSAL_iobuf_init(void);
void MOSAL_iobuf_cleanup(void);

#endif				/* __MOSAL_IOBUF_IMP__H */
