/*
  This software is available to you under a choice of one of two
  licenses.  You may choose to be licensed under the terms of the GNU
  General Public License (GPL) Version 2, available at
  <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
  license, available in the LICENSE.TXT file accompanying this
  software.  These details are also available at
  <http://openib.org/license.html>.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

  Copyright (c) 2004 Mellanox Technologies Ltd.  All rights reserved.
*/

#ifndef H_MTL_SYS_DEFS_H
#define H_MTL_SYS_DEFS_H

/* common for kernel and user */
#include <mtl_log.h>
#include <mtl_types.h>

#define IN
#define OUT

/* an absent constant in Linux */
#define _UI64_MAX	0xFFFFFFFFFFFFFFFFULL
#define _UI32_MAX	0xFFFFFFFFUL

/* long long constants */
#define MAKE_LONGLONG(a)	(a##LL)
#define MAKE_ULONGLONG(a)	(a##ULL)

/* allocation on stack */
#define ALLOCA(a)			alloca(a)
#define FREEA(a)

#ifndef MT_KERNEL
	static inline u_int16_t mt_generic_16_swap(u_int16_t x)
	{
		return (x>>8) | ((x<<8)&0xff00);
	}
	
	static inline u_int32_t mt_generic_32_swap(u_int32_t x)
	{
		u_int16_t lo, hi;
		
		lo = x>>16;
		hi = x & 0xffff;
	
		return mt_generic_16_swap(lo) | (mt_generic_16_swap(hi)<<16);
	}
	
	static inline u_int64_t mt_generic_64_swap(u_int64_t x)
	{
		u_int32_t lo, hi;
		
		lo = x>>32;
		hi = x & 0xffffffff;
		
		return mt_generic_32_swap(lo) | ((u_int64_t)mt_generic_32_swap(hi)<<32);
	}
#else
	#define mt_generic_16_swap(x) swab16(x)
	#define mt_generic_32_swap(x) swab32(x)
	#define mt_generic_64_swap(x) swab64(x)
#endif

#ifdef  MT_BIG_ENDIAN
	#define MOSAL_cpu_to_be16(x)			(x)
	#define MOSAL_be16_to_cpu(x)		    (x)
	#define MOSAL_cpu_to_be32(x)			(x)
	#define MOSAL_be32_to_cpu(x)		    (x)
	#define MOSAL_cpu_to_be64(x)			(x)
	#define MOSAL_be64_to_cpu(x)		    (x)
	#define MOSAL_cpu_to_le64(x)			mt_generic_64_swap(x)
	#define MOSAL_le64_to_cpu(x)		    mt_generic_64_swap(x)
	#define MOSAL_cpu_to_le32(x)			mt_generic_32_swap(x)
	#define MOSAL_le32_to_cpu(x)		    mt_generic_32_swap(x)
	#define MOSAL_cpu_to_le16(x)			mt_generic_16_swap(x)
	#define MOSAL_le16_to_cpu(x)		    mt_generic_16_swap(x)
#else
	#define MOSAL_cpu_to_be16(x)			mt_generic_16_swap(x)
	#define MOSAL_be16_to_cpu(x)		    mt_generic_16_swap(x)
	#define MOSAL_cpu_to_be32(x)			mt_generic_32_swap(x)
	#define MOSAL_be32_to_cpu(x)		    mt_generic_32_swap(x)
	#define MOSAL_cpu_to_be64(x)			mt_generic_64_swap(x)
	#define MOSAL_be64_to_cpu(x)		    mt_generic_64_swap(x)
	#define MOSAL_cpu_to_le64(x)			(x)
	#define MOSAL_le64_to_cpu(x)		    (x)
	#define MOSAL_cpu_to_le32(x)			(x)
	#define MOSAL_le32_to_cpu(x)		    (x)
	#define MOSAL_cpu_to_le16(x)			(x)
	#define MOSAL_le16_to_cpu(x)		    (x)
	
#endif	

/* various macros */
#define EQ_TYPE_STR

#if defined(MT_KERNEL) && defined(__i386__)
    #define STACK_OK  (         \
      {                      \
      u_int32_t vsp=0, left, ret;                    \
      asm ("movl %%esp, %0;"                    \
          : "=r"(vsp)                           \
          : );                                  \
      left = vsp-((u_int32_t)current+sizeof(struct task_struct)); \
      if ( left < 0x400 ) { \
        MTL_ERROR1("you have less then 0x400 bytes of stack left\n");  \
    	ret = 0;                    \
      }                 \
      else {    \
        MTL_DEBUG1("%s: stack depth left = %d bytes\n", __FUNCTION__, left);   \
    	ret = 1; \
      }    \
      ret;  \
    }   \
    )
    
    
    #define MT_RETURN_IF_LOW_STACK(stack_watermark) { \
            long esp;  \
    					 \
            __asm__ __volatile__("andl %%esp,%0" : \
                        "=r" (esp) : "0" (THREAD_SIZE - 1)); \
            if (unlikely(esp < (sizeof(struct task_struct) + (stack_watermark)))) { \
                printk("%s: stack overflow: %ld\n", __func__, \
                    esp - sizeof(struct task_struct)); \
    			return -255; \
            } \
    }
#else
    #define STACK_OK 1
    #define MT_RETURN_IF_LOW_STACK(stack_watermark) do {} while (0)
#endif

#ifndef MT_KERNEL
    /* User space */
    #include <pthread.h>
    
    #define MTL_MUTEX_INIT(name)			static pthread_mutex_t name = PTHREAD_MUTEX_INITIALIZER
    #define MTL_MUTEX_LOCK(name)		pthread_mutex_lock(&name)
    #define MTL_MUTEX_UNLOCK(name)	pthread_mutex_unlock(&name)
#endif

/* Definitions for malloc/free porting to kernel */
#if defined(MT_KERNEL)

#include <linux/config.h>
#define __NO_VERSION__
#include <asm/atomic.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/autoconf.h>
#include <linux/types.h>
#include <linux/ctype.h>
#include <linux/sched.h>
#include <linux/linkage.h>
#include <linux/vmalloc.h>
#include <linux/mm.h>
#include <linux/pci.h>
#include <linux/smp_lock.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/securebits.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/current.h>
#include <asm/errno.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#include <linux/highmem.h>
#include <linux/wait.h>

#define LINUX_KERNEL_2_2 ((LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)) && (LINUX_VERSION_CODE <= KERNEL_VERSION(2,2,255)))
#define LINUX_KERNEL_2_4 ((LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)) && (LINUX_VERSION_CODE <= KERNEL_VERSION(2,4,255)))
#define LINUX_KERNEL_2_6 ((LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) && (LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,255)))

#if !LINUX_KERNEL_2_6
#include <linux/tqueue.h>
#endif

# ifdef MTL_LOG_MALLOC
extern void *mtl_log_vmalloc(const char *fn, int ln, int bsize);
extern void mtl_log_vfree(const char *fn, int ln, void *ptr);
extern void *mtl_log_kmalloc(const char *fn, int ln, int bsize, unsigned g);
extern void mtl_log_kfree(const char *fn, int ln, void *ptr);
#define VMALLOC(bsize) mtl_log_vmalloc(__FILE__, __LINE__, bsize)
#define VFREE(ptr)     mtl_log_vfree(__FILE__, __LINE__, ptr)
#define MALLOC(bsize)  \
                   mtl_log_kmalloc(__FILE__, __LINE__, bsize, GFP_KERNEL)
#define INTR_MALLOC(bsize) \
                   mtl_log_kmalloc(__FILE__, __LINE__, bsize, GFP_ATOMIC)
#define FREE(ptr)      mtl_log_kfree(__FILE__, __LINE__, ptr)
# else
#define VMALLOC(bsize)     QVMALLOC(bsize)
#define VFREE(ptr)         QVFREE(ptr)
#define MALLOC(bsize)      QMALLOC(bsize)
#define INTR_MALLOC(bsize) QINTR_MALLOC(bsize)
#define FREE(ptr)          QFREE(ptr)
# endif

#ifndef MTL_TRACK_ALLOC
#if LINUX_KERNEL_2_4
#define QVMALLOC(bsize)     (in_interrupt() ? __vmalloc((bsize), GFP_ATOMIC | __GFP_HIGHMEM, PAGE_KERNEL) : vmalloc(bsize))
#else				/* Unable to use VMALLOC in interrupt for kernel 2.2 */
#define QVMALLOC(bsize)     vmalloc(bsize)
#endif
#define QMALLOC(bsize)      (in_interrupt() ? kmalloc((bsize), GFP_ATOMIC) : kmalloc((bsize), GFP_KERNEL))
#define QVFREE(ptr)         vfree(ptr)
#define QINTR_MALLOC(bsize) kmalloc((bsize), GFP_ATOMIC)
#define QCMALLOC(bsize,g) 	kmalloc((bsize), g)
#define QFREE(ptr)          kfree(ptr)
#else				/* MTL_TRACK_ALLOC */
#include <memtrack.h>
#define QVMALLOC(bsize)     TRACKQVMALLOC(bsize)
#define QMALLOC(bsize)      TRACKQMALLOC(bsize)
#define QVFREE(ptr)         TRACKQVFREE(ptr)
#define QINTR_MALLOC(bsize) TRACKQINTR_MALLOC((bsize))
#define QCMALLOC(bsize,g) 	TRACKQCMALLOC((bsize), g)
#define QFREE(ptr)          TRACKQFREE(ptr)

#define TRACKQVMALLOC(bsize) ({                                                                                                    \
                                  void *__memtrack_p;                                                                                           \
                                  __memtrack_p=(in_interrupt() ? __vmalloc((bsize), GFP_ATOMIC | __GFP_HIGHMEM, PAGE_KERNEL) : vmalloc(bsize)); \
                                  if ( __memtrack_p ) {                                                                                         \
                                    memtrack_alloc(MEMTRACK_VMALLOC, (unsigned long)__memtrack_p, bsize, __FILE__, __LINE__, in_interrupt() ? GFP_ATOMIC : GFP_KERNEL); \
                                  }                                                                                                  \
                                  __memtrack_p;                                                                                                 \
                               })

#define TRACKQMALLOC(bsize)  ({                                                                                   \
                                  void *__memtrack_p;                                                                          \
                                  __memtrack_p=(in_interrupt() ? kmalloc((bsize), GFP_ATOMIC) : kmalloc((bsize), GFP_KERNEL)); \
                                  if ( __memtrack_p ) {                                                                        \
                                    memtrack_alloc(MEMTRACK_KMALLOC, (unsigned long)__memtrack_p, bsize, __FILE__, __LINE__, in_interrupt() ? GFP_ATOMIC : GFP_KERNEL);  \
                                  }                                                                                 \
                                  __memtrack_p;                                                                                \
                               })

#define TRACKQCMALLOC(bsize,g)   ({                                                                                    \
                                  void *__memtrack_p;                                                                          \
                                  __memtrack_p=kmalloc((bsize), g);                                                            \
                                  if ( __memtrack_p ) {                                                                        \
                                    memtrack_alloc(MEMTRACK_KMALLOC, (unsigned long)__memtrack_p, bsize, __FILE__, __LINE__, g);  \
                                  }                                                                                 \
                                  __memtrack_p;                                                                                \
                               })

#define TRACKQVFREE(ptr)     do {                                                    \
                                 memtrack_free(MEMTRACK_VMALLOC, (unsigned long)ptr, __FILE__, __LINE__);  \
                                 vfree(ptr);                                           \
                               }                                                       \
                               while(0)

#define TRACKQINTR_MALLOC(bsize) ({                                                                                   \
                                      void *__memtrack_p;                                                                          \
                                      __memtrack_p = kmalloc((bsize), GFP_ATOMIC);                                                 \
                                      if ( __memtrack_p ) {                                                                        \
                                        memtrack_alloc(MEMTRACK_KMALLOC, (unsigned long)__memtrack_p, bsize, __FILE__, __LINE__, GFP_ATOMIC);  \
                                      }                                                                                 \
                                      __memtrack_p;                                                                                \
                                   })

#define TRACKQFREE(ptr)      do {                                                    \
                                 memtrack_free(MEMTRACK_KMALLOC, (unsigned long)ptr, __FILE__, __LINE__);  \
                                 kfree(ptr);                                           \
                               }                                                       \
                               while(0)

#endif

/* various */
#define MT_DPC_SCHEDULE    schedule()
#define MLOCK(__buff, __size) 0

#else				/* not MT_KERNEL */
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>		/* for mlock function */
#define MLOCK(__buff, __size) mlock(__buff, __size)

#ifdef MTL_LOG_MALLOC
#define MALLOC(x)  ( {void* mdebug_tmp; mdebug_tmp= malloc(x); MTL_DEBUG5(MT_FLFMT("MT_MALLOC_DEBUG( %d bytes @ %p )"),x,mdebug_tmp); mdebug_tmp;} )
#define FREE(x) MTL_DEBUG5(MT_FLFMT("MT_FREE_DEBUG( %p )\n"),x) ; free(x)
#define VMALLOC(bsize) MALLOC(bsize)
#define VFREE(bsize)   FREE(bsize)
#define INTR_MALLOC(bsize) MALLOC(bsize)
#else
#define VMALLOC(bsize) malloc(bsize)
#define VFREE(bsize) free(bsize)
#define MALLOC(bsize) malloc(bsize)
#define INTR_MALLOC(bsize) malloc(bsize)
#define FREE(ptr)  free(ptr)
#endif				/* MTL_LOG_MALLOC */

# endif				/* MT_KERNEL */

#define __INLINE__  inline

#endif				/* H_MTL_SYS_DEFS_H */
