package Device::mtcr;
use strict;
use Carp;

=head1 NAME

Device::mtcr - Linux Kernel 2.2+ Mellanox /usr/mellanox/dev/MT*pci*

=head1 DESCRIPTION

This program uses the linux /usr/mellanox/dev/MT*pci* devices, which
means of course that you must have access to that device, which is simply
a matter changing permissions for your system, much the same as you
would if you where giving the user access to the sound card (eg: /dev/dsp)

=head1 NOTES

Note that this is a temporary hack for now, full version to come soon...

=head1 COPYRIGHT

Copyright (c) 2002 Mellanox Technologies LTD.

=head1 AUTHOR

Victor Kaplansky L<victork@mellanox.co.il>, L<http://www.mellanox.com>

=head1 SEE ALSO

L<driver/mtcr.[ch] - Mellanox Tools Simple Driver>

=cut

require DynaLoader;
our @ISA = qw(DynaLoader);
our $VERSION = '0.1';

bootstrap Device::mtcr $VERSION;

# Standard function to return information from this driver
sub INFO {
	return {
		'os' => 'linux',
		'ver' => '>= 2.2',
		'type' => 'word',
	};
}

sub new {
    my ($class, $dev_name, $dev_type) = @_;
    my $this = bless {}, ref($class) || $class;
    $dev_type ||= "MST_TAVOR";
    $this->{DATA}->{DEV_NAME} = $dev_name;
    if (($dev_type ne "MST_TAVOR") && ($dev_type ne "MST_GAMLA"))
    {
        croak "device type should be \"MST_TAVOR\" or \"MST_GAMLA\"\n";
    }
    $this->{DATA}->{DEV} = mtcr_opendev ($this->{DATA}->{DEV_NAME}, $dev_type);
    if ($this->{DATA}->{DEV} == 0) {
        croak "Failed to open " . $this->{DATA}->{DEV_NAME} . "\n"
              . "try to run `mst [start|restart]' as super-user"
    }
    return $this;
}

sub set_word {
    my ($this, $addr, $val) = @_;
    mtcr_wr_data ($this->{DATA}->{DEV}, $addr, $val);
}

sub get_word {
    my ($this, $addr) = @_;
    return mtcr_rd_data ($this->{DATA}->{DEV}, $addr);
}

sub DESTROY {
    my ($this) = @_;

    if (defined ($this->{DATA}->{DEV})) {
        mtcr_closedev ($this->{DATA}->{DEV});
    }
}

1;

