#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include <sys/ioctl.h>
#include <sys/types.h> /*This header likes to sit before linux/ headers*/
#include <sys/stat.h>

/*  Linux USB Support */
#include <linux/types.h>
#include <linux/usb.h>
#include <linux/usbdevice_fs.h>

#include "usbdescriptor.h"

#define __DIMAX__
#ifdef __DIMAX__
#define DIMAX_V 0x0abf
#define DIMAX_P 0x3370
#else
#define DIMAX_V 0x0abf
#define DIMAX_P	0x03e9
#endif

#define STRLEN 100
#define BUS 	"Bus="
#define DEV 	"Dev#="
#define VENID   "Vendor="
#define PRDID   "ProdID="

#define DIMAX_IF   		0x00
#define DIMAX_ALTIF		0x01
#define DIMAX_CONF 		0x01
#define DIMAX_IN_EP		0x84
#define DIMAX_OUT_EP	0x02


int main( int argc, char *argv[] )
{
	int 	rc=0;
	char* 	devf;
	int 	fd = -1;
	unsigned int tmp;
	struct usbdevfs_setinterface setif_st;
        struct usbdevfs_bulktransfer bulk_st;
        char buf[300];
        char resbuf[300];
  struct usb_device_descriptor desc;

	
	/*  Find device */
	
	if (argc <= 1)
	{
		printf("Device Vend=0x%04x Prod=0x%04x not found\n", DIMAX_V, DIMAX_P ); 
		return -1;
	}

	devf = argv[1];
	if( devf )
		printf( "Device file: \"%s\"\n", devf );

	
	/*  Open Device File */
	
	fd = open( devf, O_RDWR | O_SYNC );
	if( fd == -1 )
	{
		rc = errno;
		printf( "Device File open error. rc=%d, \"%s\"\n", rc, strerror(rc) );
		goto cleanup;
	}

//Do it in mst now  
//        fchmod( fd, 0666 );
	
        
  rc=usb_get_device_descriptor( fd, & desc);
	if( rc )
	{
		rc = errno;
		printf( "GET DESCRIPTOR ioctl error. rc=%d, \"%s\"\n", rc, strerror(rc) );
		goto cleanup;
	}
  
  if (desc.idVendor != DIMAX_V ||
      desc.idProduct != DIMAX_P) {
    rc=1;
		printf("Device Vendor/Product id mismatch: Expected %x/%x Detected %x/%x\n",
       DIMAX_V, DIMAX_P, desc.idVendor, desc.idProduct);
		goto cleanup;
  }
  


	/*  Select configuration */
	
	tmp = DIMAX_CONF;
	rc = ioctl( fd, USBDEVFS_SETCONFIGURATION, &tmp );
	if( rc )
	{
		rc = errno;
		printf( "SETCONFIGURATION ioctl error. rc=%d, \"%s\"\n", rc, strerror(rc) );
		goto cleanup;
	}

	
	/*  Claim Interface */
	
	tmp = DIMAX_IF;
	rc = ioctl( fd, USBDEVFS_CLAIMINTERFACE, &tmp );
	if( rc )
	{
		rc = errno;
		printf( "CLAIMINTERFACE ioctl error. rc=%d, \"%s\"\n", rc, strerror(rc) );
                if( rc == EBUSY )
                    printf( "Please rmmod ezusb2131. This can be a reason\n" );
		goto cleanup;
	}

	
	/*  Set Alternative Configuration for Interface */
	
	setif_st.interface = DIMAX_IF;
	setif_st.altsetting = DIMAX_ALTIF;

	rc = ioctl( fd, USBDEVFS_SETINTERFACE, &setif_st );
	if( rc )
	{
		rc = errno;
		printf( "SETINTERFACE ioctl error. rc=%d, \"%s\"\n", rc, strerror(rc) );
		goto cleanup1;
	}


        /* Try to read FW version */
        tmp = 5;
        while( tmp )
        {
            tmp--;

            buf[0]=0x0a;
            bulk_st.data = buf;
            bulk_st.ep = DIMAX_OUT_EP;
            bulk_st.len = 1;
            bulk_st.timeout = 100;  /*  ms */
            rc = ioctl( fd, USBDEVFS_BULK, &bulk_st );
            if( rc != 1 )
            {
                rc = errno;
                sprintf( resbuf,  "BULK ioctl OUT error. rc=%d, \"%s\"\n", rc, strerror(rc) );
                continue;
            }

            bulk_st.data = buf;
            bulk_st.ep = DIMAX_IN_EP;
            bulk_st.len = 3;
            bulk_st.timeout = 200;  /*  ms */
            rc = ioctl( fd, USBDEVFS_BULK, &bulk_st );
            if( rc != 3 )
            {
                rc = errno;
                sprintf( resbuf, "BULK ioctl IN error. rc=%d, \"%s\"\n", rc, strerror(rc) );
                continue;
            }
            sprintf( resbuf, "MTUSB-1 FW Version %u.%u\n", (int)buf[0], (int)buf[1] );
            break;
        }
        printf( "%s", resbuf);
	
	/*  Cleanup and Exit */
cleanup1:
        /* Release Interface */
	tmp = DIMAX_IF;
	rc = ioctl( fd, USBDEVFS_RELEASEINTERFACE, &tmp );
	if( rc )
	{
		rc = errno;
		printf( "RELEASEINTERFACE ioctl error. rc=%d, \"%s\"\n", rc, strerror(rc) );
	}
cleanup:            
	if( fd != -1 )
		close( fd );
	return rc;
}

#if 0
static char *get_usb_dev()
{
	FILE         *fp;
	char         str[STRLEN], *p;
	static char  buf[50];
	unsigned int venid=0, prdid=0, bus=0, dev=0;

	if ((fp = fopen("/proc/bus/usb/devices", "r")) == NULL)
		return NULL;
	while (fgets(str, STRLEN, fp))
	{
		if (!strncmp(str, "T:", 2))
		{
			if ((p = strstr(str, BUS)) != NULL)
			{
				p += strlen(BUS);
				sscanf(p, "%d", &bus);
			}
			if ((p = strstr(str, DEV)) != NULL)
			{
				p += strlen(DEV);
				sscanf(p, "%d", &dev);
			}
		}
		if (!strncmp(str, "P:", 2))
		{
			if ((p = strstr(str, VENID)) != NULL)
			{
				p += strlen(VENID);
				venid = strtoul(p, NULL, 16);
			}
			if ((p = strstr(str, PRDID)) != NULL)
			{
				p += strlen(PRDID);
				prdid = strtoul(p, NULL, 16);
			}
			if( (venid == DIMAX_V    &&  prdid == DIMAX_P) )

				{
					sprintf(buf, "/proc/bus/usb/%03d/%03d", bus, dev);
					fclose(fp);
					return buf;
				}
		}
	}
	fclose(fp);
	return NULL;
}
#endif

