#include <linux/fs.h>
#include <linux/version.h>
#include <linux/netdevice.h>
#include <linux/config.h>
#include <asm/semaphore.h>

#if CONFIG_COMPAT && ! MST_NO_CONFIG_COMPAT && !HAVE_COMPAT_IOCTL
#include <asm/ioctl32.h>
#endif
/* Note that MST_NO_CONFIG_COMPAT can get defined by asm/ioctl32.h */
#if CONFIG_COMPAT && ! MST_NO_CONFIG_COMPAT && !HAVE_COMPAT_IOCTL
#include <linux/syscalls.h>
typedef int (*mst_ioctl_trans_handler_t)(unsigned int, unsigned int, unsigned long, struct file *);
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <linux/init.h>
#include <linux/moduleparam.h>
#endif

static int major=-1;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
module_param(name, charp, 0);
#else
MODULE_PARM(name,"s");
#endif
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

/* TODO: 
 * implement READ4/WRITE4 IOCTLS
 * (+more sizes?)
 * implement lseek?
 * http://www.tldp.org/LDP/khg/HyperNews/get/devices/addrxlate.html
 */

static int open (struct inode *inode, struct file *file)
{
  int minor=MINOR(inode->i_rdev);
  int ret=0;

  if (down_interruptible(&devices[minor].sem)) {
    return -ERESTARTSYS;
  }

  file->private_data=&devices[minor];

  /* Truncate to 0 length on open for writing */
  if (file->f_flags & O_APPEND) {
    file->f_pos=devices[minor].bufused;
  } else if ( (file->f_flags & O_TRUNC) || (file->f_flags & O_WRONLY ) ) {
    devices[minor].bufused=0;
  } 

  /*fin:*/
  up(&devices[minor].sem);
  return ret;
}


static  int release (struct inode *inode, struct file *file)
{
  return 0;
}

static ssize_t read (struct file * file, char *buf, size_t count, loff_t *f_pos)
{
  struct dev_data* dev=(struct dev_data*) file->private_data;
   if (down_interruptible(&dev->sem))
     return -ERESTARTSYS;

  if (*f_pos >= dev->bufused) {
      count=0;
      goto fin;
  }
  if (*f_pos + count > dev->bufused)
    count = dev->bufused - *f_pos;

  if (copy_to_user(buf,dev->buf+*f_pos, count)) {
    count=-EFAULT;
    goto fin;
  }
  *f_pos+=count;

  fin:
  up(&dev->sem);
  return count;
}

static ssize_t write (struct file *file, const char * buf, size_t count, loff_t *f_pos)
{
  struct dev_data* dev=(struct dev_data*) file->private_data;
   if (down_interruptible(&dev->sem))
     return -ERESTARTSYS;

  if (*f_pos >= BUFFER_SIZE) {
    count=0;
    goto fin;
  }
  if (*f_pos + count > BUFFER_SIZE)
    count = BUFFER_SIZE - *f_pos;

  if (copy_from_user(dev->buf+*f_pos, buf, count)) {
    count=-EFAULT;
    goto fin;
  }
  *f_pos+=count;
  if (dev->bufused < *f_pos) dev->bufused=*f_pos;

  fin:
  up(&dev->sem);
  return count;
}

#if HAVE_COMPAT_IOCTL
static long compat_ioctl (struct file *f, unsigned int o, unsigned long d)
{
	struct inode *n = f->f_dentry->d_inode;
	return ioctl(n, f, o, d);
}
#endif

/* implement lseek ? */

static struct file_operations fops = {
 /*llseek:  scull_llseek,*/
 read:  read,
 write:  write,
 ioctl:  ioctl,
 open:  open,
 release: release,
#ifdef SUPPORT_MMAP
 mmap: mmap,
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
 owner: THIS_MODULE,
#endif
#if HAVE_COMPAT_IOCTL
 compat_ioctl: compat_ioctl,
#endif
};


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
static int mst_init_module(void)
#else
int init_module(void)
#endif
{
 int i;
 major=register_chrdev(0, name, &fops);
 if (major<=0) {
   printk("Unable to register character device %s\n",name);
   return -1;
 }

#if CONFIG_COMPAT && ! MST_NO_CONFIG_COMPAT && !HAVE_COMPAT_IOCTL
 {
   int j;
   int rc=0;
   for(i=0;i<sizeof(ioctl_cmds)/sizeof(ioctl_cmds[0]);++i)
   {
     rc=register_ioctl32_conversion(ioctl_cmds[i],
		     ((mst_ioctl_trans_handler_t)sys_ioctl));
     if (rc)
     {
       printk("Unable to register compatibility ioctl %d[%d] for device %s\n",
		       i, ioctl_cmds[i], name);
       for(j=0;j<i;++j)
       {
         unregister_ioctl32_conversion(ioctl_cmds[j]);
       }
       unregister_chrdev(major, name);
       return rc;
     }
   }
 }
#endif
 
 

 for(i=0;i<MAXMINOR;++i)  {
   sema_init(&devices[i].sem,1);
#ifdef MST_PPC_DRIVER
   devices[i].cs=(unsigned int)(-1);   
#endif   
 } 
#if 0
 printk("MST Driver initialised\n");
 printk("Device name:%s\n",name);
 printk("Major number:%d\n",major);
 printk("Minor numbers:%d\n",MAXMINOR);
 printk("IOCTL numbers:\n");
 printk("INIT:0x%x\n", INIT);
 printk("STOP:0x%x\n", STOP);
#ifdef WRITE4
 printk("WRITE4:0x%x\n", WRITE4);
 printk("READ4:0x%x\n",  READ4);
 printk("MODIFY:0x%x\n", MODIFY);
#endif
#endif


 return 0;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
static void mst_cleanup_module(void)
#else
void cleanup_module(void)
#endif
{
  /* don't have to destroy semaphores in linux */

#if CONFIG_COMPAT && ! MST_NO_CONFIG_COMPAT && !HAVE_COMPAT_IOCTL
  int i;
  for(i=0;i<sizeof(ioctl_cmds)/sizeof(ioctl_cmds[0]);++i)
  {
    unregister_ioctl32_conversion(ioctl_cmds[i]);
  }
#endif
  unregister_chrdev(major, name);
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
module_init(mst_init_module);
module_exit(mst_cleanup_module);
#endif
