/*
 *
 *  minit.c - CR Space access init
 *
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "mtcr.h"

#define PRINT_DEV_BOARD(DEV,MT_TARGET,NICK) do { printf("%-32s --> %-15s (%s)\n",(DEV),(MT_TARGET),(NICK)); }while(0)

void usage(const char *n)
{
    printf( "USAGE %s [MST_DEVICE]\n", n);  
    printf( "\tList mst device(s) with connection target\n");
    printf( "DESCRIPTION\n" );
    printf( "\tIf device not specifyed lists all existing mst devices\n" );
    exit(1);
}

int target_auto_detect( char* devname )
{
    mfile* mf;
    int rc=0;
    unsigned tmp;

    /*  Open MST device */
    mf=mopen( devname );
    if( ! mf )
    {
        /*printf( "Can't open \"%s\" (%s)\n", devname, strerror(errno) );*/
        return -1;
    }

    switch( mf->tp )
    {
    case MST_PCI:
    case MST_PCICONF:
    case MST_PPC:
        if( ! mf->is_i2cm )
        {
            if( strstr( devname, "mt43132" ) )
                PRINT_DEV_BOARD( devname, "mt43132", "anafa1" );
            else if( strstr( devname, "mt47396" ) )
                PRINT_DEV_BOARD( devname, "mt47396", "anafa2" );
            else if( strstr( devname, "mt21108" ) )
                PRINT_DEV_BOARD( devname, "mt21108", "gamla" );
            else if( strstr( devname, "mt23108" ) )
                PRINT_DEV_BOARD( devname, "mt23108", "tavor" );
            else if( strstr( devname, "mt25208" ) )
                PRINT_DEV_BOARD( devname, "mt25208", "arbel" );
            else if( strstr( devname, "mt25218" ) )
                PRINT_DEV_BOARD( devname, "mt25218", "arbel1" );
            else if( strstr( devname, "mt25228" ) )
                PRINT_DEV_BOARD( devname, "mt25228", "arbel2" );
            else if( strstr( devname, "mt25238" ) )
                PRINT_DEV_BOARD( devname, "mt25238", "arbel3" );
            else if( strstr( devname, "mt24204" ) )
                PRINT_DEV_BOARD( devname, "mt24204", "sinai" );
            else if( strstr( devname, "mt24204" ) )
                PRINT_DEV_BOARD( devname, "mt25204", "sinai8" );
            else
            {
                PRINT_DEV_BOARD( devname, "unknown", "???" );
                rc=-1;
            }
            break;
        }
        else;   /*  No break!!! Pass to I2C devices  */

    case MST_CALBR:
    case MST_USB:
    case MST_USB_DIMAX:
    case MST_IF:
#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
#endif

        mf->dtype = MST_NOADDR;
        mf->i2c_slave = 0x70;
        rc=mread64( mf, 0, &tmp, 1 );   /* Try 0x70 */
        if( rc==1 )
        {
            /* 0x70-yes */
            mf->i2c_slave = 0x71;
            rc=mread64( mf, 0, &tmp, 1 );   /* Try 0x71 */
            if( rc==1 )
            {
                /* 0x70-yes 0x71-yes */
                mf->i2c_slave = 0x73;
                rc=mread64( mf, 0, &tmp, 1 );   /* Try 0x73 */
                if( rc==1 )
                {
                    /* 0x70-yes 0x71-yes 0x73-yes */
                    PRINT_DEV_BOARD( devname, "MT31000", "gazelle" );
                    rc=0;
                    break;
                }
                /* 0x70-yes 0x71-yes 0x73-no */
            }
            /* 0x70-yes 0x71-no */
            PRINT_DEV_BOARD( devname, "MT34000", "rhino" );
            rc=0;
            break;
        }
        else
        {
            /* 0x70-no */
            mf->i2c_slave = 0x71;
            rc=mread64( mf, 0, &tmp, 1 );   /* Try 0x71 */
            if( rc==1 )
            {
                /* 0x70-no 0x71-yes */
                PRINT_DEV_BOARD( devname, "MT34000", "rhino" );
                rc=0;
                break;
            }
            /* 0x70-no 0x71-no */
            /* Try other */    
        } 

        /*  Try Reindeer */
        mf->dtype = MST_NOADDR;
        mf->i2c_slave = 0x77;
        rc=mread64( mf, 0, &tmp, 1 );
        if( rc==1 )
        {
            PRINT_DEV_BOARD( devname, "MT33000", "reindeer" );
            rc=0;
            break;
        }

        /*  Try Anafa2 EVB */
        mf->dtype = MST_DIMM;
        mf->i2c_slave = 0x20;
        rc=mread64( mf, 0, &tmp, 1 );
        if( rc==1 )
        {
            PRINT_DEV_BOARD( devname, "MT11020", "EVB_anafa2" );
            rc=0;
            break;
        }

        /*  Try Anafa1 */
        mf->dtype = MST_GAMLA;
        mf->i2c_slave = 0x5a;
        rc=mread4(mf, 0x2800, &tmp );
        if( rc==4 && (((tmp & 0xffff0000) >> 16) == 43132) )
        {
            PRINT_DEV_BOARD( devname, "mt43132", "anafa1" );
            rc=0;
            break;
        }

        /*  Unknown */
        PRINT_DEV_BOARD( devname, "unknown", "???" );
        rc=0;
        break;

    case MST_IB:
        break;   /*  TBD */
    }

    return rc;
}

int main(int ac, char *av[])
{
    char          *devname;
    int           rc = 0;
    char          buf[2000];  
    char*         pb;
    int           i;
    int           len;

    if (ac > 2)
        usage(av[0]);

    if( ac == 2 )
    {
        if( av[1][0] == '-' )
            usage(av[0]);
        devname = av[1];
        rc = target_auto_detect(devname);
    }
    else
    {
        pb=buf;
        i = mdevices( pb, 2000, (int)(-1));
        if( i < 0 )
            return i;

        while( (len=strlen(pb)) && i-- )
        {
            target_auto_detect(pb);
            pb+=(len+1);
        }
    }

    return rc;
}

