/*
 *
 *  mremoet.c - mtcr remote agent
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "mtcr.h"
#include "tcp.h"

void usage(const char *n)
{

    printf("%s <host:port> <command>\n", n);
    exit(1);
}

int main(int ac, char *av[])
{
    int  ic, port, con;
    char *host_p, *port_p, buf[1024];;

    if (ac < 3)
        usage(av[0]);

    /*  Device and port pointers */
    host_p = av[1];
    port_p = strchr(host_p, ':');
    if (!port_p)
        usage(av[0]);
    *port_p = '\0';
    port_p++;

    /*  Open connection */
    port = strtol(port_p, 0, 10);
    con = open_cli_connection(host_p, port);
    if (con < 0)
    {
        perror("Open connection (client side)");
        exit(1);
    }

    for (ic = 2; ic < ac; ic++)
    {
        char *cmd = av[ic];

        /*  Send command */
        if (writes(con, cmd) < 0)
        {
            perror("Write to socket");
            exit(1);
        }

        /*  Get answer */
        if (reads(con, buf, 1024) < 0)
        {
            perror("Read from socket");
            exit(1);
        }

        if (*buf != 'O')
        {
            printf("%s\n", buf);
            exit(1);
        }

        if (*cmd == 'L')
        {
            /*  List command - special treatment */
            int i, ndevs = strtol(buf+2, 0, 10);
            for (i=0; i<ndevs; i++)
            {
                if (reads(con, buf, 1024) < 0)
                {
                    perror("Read from socket");
                    exit(1);
                }
                printf("%s ", buf);
            }
            printf("\n");
        }
        else
        {
            /*  Other command - just print result */
            if (strlen(buf) > 2)
                printf("%s\n", buf+2);
        }
    }

    close(con);
    return 0;
}
