#include <linux/kernel.h>
#include <linux/module.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/pci.h>
#include "mst_calbr.h"

/* #include "calbr_i2c.h" */
#include "i2c.h"
#define  INIT   CALBR_INIT
#define  STOP   CALBR_STOP
#define  TRANS  CALBR_TRANS

static char* name="mst_calbr";

#if CONFIG_COMPAT && !HAVE_COMPAT_IOCTL
static unsigned int
ioctl_cmds[] ={
  TRANS
};
#endif

/* Allow minor numbers 0-255 */
#define MAXMINOR 256
#define BUFFER_SIZE 256

struct dev_data {
	unsigned short base;
	unsigned char  own;
	calbr_i2c_clk_rate_t  clk_rate; /*Pass enum clk_rate_t as a value*/
  volatile int valid;
  struct semaphore sem;
  char buf[BUFFER_SIZE];
  int bufused;
};

static struct dev_data devices[MAXMINOR];

/* #include "calbr_i2c.c" */

static int ioctl (struct inode *inode, struct file *file, unsigned int opcode, unsigned long udata_l)
{
  void* udata=(void*)udata_l;
  int minor=MINOR(inode->i_rdev);
  struct dev_data* dev=&devices[minor];
  int ret=0;

  /* By convention, any user gets read access
   * and is allowed to use the device.
   * Commands with no direction are administration
   * commands, and you need write permission
   * for this */

  if ( _IOC_DIR(opcode) == _IOC_NONE ) {
    if (! ( file->f_mode & FMODE_WRITE) ) return -EPERM;
  } else {
    if (! ( file->f_mode & FMODE_READ) ) return -EPERM;
  }

  if (down_interruptible(&devices[minor].sem)) {
    return -ERESTARTSYS;
  }


  switch (opcode) {

    case INIT:
      {
        struct mst_calbr_init_st initd;
        /* Now have to init the new device */

        if (copy_from_user(&initd, udata, sizeof(initd))) {
          ret=-EFAULT;
          goto fin;
        }

#if 0
        /*Init global vars */
        access_key = 0; /* initialize access key */
        transaction_on = 0; /* no transacion in progress */
        lock_time = jiffies;


        /* Return busy ? */
        if ( check_region(initd.base, 2) < 0 ) return -ENXIO;
        printk("REQUEST region name %s\n", name);

        request_region(initd.base, 2, name);

        if ((ret=i2c_setup(initd.base, initd.own, initd.clk_rate)) )
          goto fin;
#endif

        if (! i2c_present()) {
          ret=-EINVAL;
          goto fin;
        }

        dev->base=initd.base;

        dev->own=initd.own;
        dev->clk_rate=initd.clk_rate;

        dev->valid=1;

        goto fin;
      }

    case STOP:
#if 0
      release_region(dev->base, 2);
#endif
      dev->valid=0;
      goto fin;


    case TRANS:
      {
        struct mst_trans_st transd;
        u_int32_t key=0;
        /*We don't actually need to copy the read buffer etc */
        if (copy_from_user(&transd, udata, sizeof(transd))) {
          ret=-EFAULT;
          goto fin;
        }

        if (!dev->valid) {
          ret=-ENOTTY;
          goto fin;
        }

        if (transd.wr_size > MAX_WR_TRANS_SIZE || transd.rd_size > MAX_RD_TRANS_SIZE) {
          ret=-EOVERFLOW;
          goto fin;
        }

        if (transd.wr_size) {
          if ((ret=i2c_master_transmit(transd.slv_addr, transd.wr_buff,transd.wr_size, & transd.wr_count, transd.rd_size == 0,  & key))) {
            goto trans_fin;
          }

          if (transd.wr_count != transd.wr_size) {
            i2c_send_stop(key);
            ret=-ENOLINK;
            goto trans_fin;
          }
        }

        if (transd.rd_size) {
          if ((ret=i2c_master_receive(transd.slv_addr, transd.rd_buff,transd.rd_size, & transd.rd_count, 1, key))) {
            goto trans_fin;
          }

          if (transd.rd_count != transd.rd_size) {
            ret=-ENOLINK;
            goto trans_fin;
          }
        } else {
          transd.rd_count =0;
        }

trans_fin:
	if (copy_to_user(udata, &transd, sizeof(transd))) {
		ret=ret?ret:-EFAULT;
	}
        goto fin;
      }
    default:
      ret= -ENOTTY;
      goto fin;

  }


fin:
  up(&dev->sem);
  return ret;
}

#include "driver_common.c"

