/*
 *
 *  mtcr.c - Mellanox Software tools (mst) driver implementation
 *
 */

#include <stdio.h>

#include <arpa/inet.h>
#include <asm/byteorder.h>
#include <dirent.h>
#include <endian.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/un.h>
#include <unistd.h>

#ifndef MST_CPU_ppc_8xx
#include "usbdescriptor.h"
#endif
#include "mtcr.h"
#include "mtcr_types.h"
#include "mtcr_mif.h"

#include "mtcr_ib.h"
#include "mtcr_i2c.h"
#include "mst_pci.h"
#include "mst_pciconf.h"
#include "mst_calbr.h"
#include "usbioctl.h"

#ifdef MST_CPU_ppc_8xx
 #include "mst_ppc.h"
#endif

#ifdef ENABLE_MST_DEV_I2C
 #include <linux/i2c.h>
 #include <linux/i2c-dev.h>
#endif 

#define CLEAR(st) memset(&(st), 0, sizeof(st))
#define MST_DIR   "/dev/mst"
#define DEVASYS_V 0x0abf
#define DEVASYS_P 0x3e9
#define DIMAX_V   0x0abf
#define DIMAX_P   0x3370
#define VENID     "Vendor="
#define PRDID     "ProdID="
#define BUS       "Bus="
#define DEV       "Dev#="
#define STRLEN    512
#define MAX_USB_RD_RETRY 10

/*
 * We do this ugly trick here just
 * not to export names from tcp.c
 */
#define INSIDE_MTCR static
#include "tcp.c"


static void  sig_pipe(int signo);
static int get_device_flags(const char *name);


/*
 * This function is never called.
 * It needs tio shutup stupid compiler warnings
 * "defined but not used" for static functions
 */
void mst_dummy()
{
    readn(-1, 0, 0);
    readnl(-1, 0, 0);
    writenl(-1, "");
    open_serv_connection(0);
}

static int remote_open(const char *name, mfile *mf, DType dtype)
{
    int  con, port, i;
    char buf[40], *name_p, nameb[1024];
    char *dev_p, *port_p;
      
    /*  Skip path */
    if ((name_p = strrchr(name, '/')) == 0)
        name_p = (char *)name;
    else
        name_p++;

    /*  Copy remote name, change '@' to '/' */
    for (i=0; i<1022 && name_p[i]; i++)
        nameb[i] = (name_p[i] == '@') ? '/' : name_p[i];
    nameb[i] = '\0';

    /*  Device and port pointers */
    dev_p = strchr(nameb, ',');
    port_p = strchr(nameb, ':');
    if (!dev_p || !port_p)
    {
        errno = EINVAL;
        return -1;
    }
    *port_p = '\0';
    port_p++;
    dev_p++;

    /*  Non-verbose */
    logset(0);

    /*  Open connection */
    port = strtol(port_p, 0, 10);
    con = open_cli_connection(nameb, port);
    if (con < 0)
        return -1;

    /*  Open device */

    sprintf(buf, "O 0x%08x ", dtype);
    if (writen(con, buf, 13) < 0)
        return -1;

    if (writes(con, dev_p) < 0)
        return -1;
    if (reads(con, buf, 10) < 0)
        return -1;
    if (*buf != 'O')
    {
        errno = ENOENT;
        return -1;
    }

    if(signal(SIGPIPE, sig_pipe) == SIG_ERR)
    {
        printf("Error installing signal handler\n");
        exit(1);
    }

    mf->sock = con;
    mf->dtype = dtype;
    mf->flags=get_device_flags(name);
    return 0;
}

#define PUTC(c) do {     \
    *p++ = (c);        \
     if (++cnt >= len) { \
         closedir(dir);  \
         return -1;      \
     } } while(0)
#define PUTS(s) do { for(i=0; (s)[i]; i++) PUTC((s)[i]); } while (0)

char* find_tavor(const char* name)
{
  char* ret;
  ret=strstr(name, "23108");
  if (ret) return ret;
  ret=strstr(name, "25208");
  if (ret) return ret;
  ret=strstr(name, "25218");
  if (ret) return ret;
  ret=strstr(name, "25228");
  if (ret) return ret;
  ret=strstr(name, "25238");
  if (ret) return ret;
  ret=strstr(name, "24204");
  if (ret) return ret;
  ret=strstr(name, "25204");
  if (ret) return ret;
  return ret;
}


static int get_device_flags(const char *name)
{
  int mask=0;
  if ( find_tavor(name) && strstr(name, "pci_ddr") ) mask|=MDEVS_TAVOR_DDR;
  if ( find_tavor(name) && strstr(name, "pci_uar") ) mask|=MDEVS_TAVOR_UAR;
  if ( find_tavor(name) && strstr(name, "pci_cr") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt23108_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt25208_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt25218_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt25228_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt25238_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt24204_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt25204_pciconf") ) mask|=MDEVS_TAVOR_CR;
  if ( strstr(name, "mt21108") ) mask|=MDEVS_GAMLA;
  if ( strstr(name, "vtop")  ) mask|=MDEVS_MEM;
  if ( strstr(name, "calbr") ) mask|=MDEVS_I2CM;
  if ( strstr(name, "calibre") ) mask|=MDEVS_I2CM;
  if ( strstr(name, "mtusb") ) mask|=MDEVS_I2CM;
  if ( strstr(name, "mif") ) mask|=MDEVS_IF;
  if ( strstr(name, "dev-i2c") ) mask|=MDEVS_DEV_I2C;
  if ( strstr(name, "ppc") ) mask|=MDEVS_PPC;
  if ( strstr(name, "i2cm") ) mask|=MDEVS_I2CM;
  if ( strstr(name, ":") ) mask|=MDEVS_REM;
  return mask;
}

int mdevices(char *buf, int len, int mask)
{
    DIR           *dir;
    struct dirent *dirent;
    int           i, rc=0, cnt=0;
    char         *p = buf;

    /* Regular devices */
    if ((dir = opendir(MST_DIR)) != NULL)
    {

        while ((dirent = readdir(dir)) != NULL)
        {
            char *name = dirent->d_name;

            /* Skip "." and ".." */
            if (name[0] == '.')
                continue;

            /* According to mask */
            if (get_device_flags(name)&mask)
            {
                PUTS(MST_DIR);
                PUTC('/');
                PUTS(name);
                PUTC('\0');
                rc++;
            }
        }
        closedir(dir);
    }

    return rc;
}

mfile *mopen(const char *name)
{
    return mopend(name, MST_TAVOR);
}

mfile *mopend(const char *name, DType dtype)
{
    int  err;
    char *real_name = (char *)name;
    char *p;


    /*  Local open */
    /* allocate mfile struct */
    mfile *mf = malloc(sizeof(mfile));
    if (!mf)
    {
        errno = ENOMEM;
        return 0;
    }


    /*  Remote connection? - Special open */
    if (strchr(name, ':'))
    {
        mf->is_i2cm = 0;
        if (remote_open(name, mf, dtype) < 0)
            return 0;
        else
            return mf;

    }

    /*  IB device? special open */
    if ((p = strstr(name, "IB-")) != 0)
    {
        if (mib_open(p+3, mf) < 0)
            return 0;
        else
            return mf;
    }


    mf->sock = -1;

    /* Type of device */
    mf->dtype = dtype;
    if (dtype == MST_TAVOR)
        mf->i2c_slave = 0x48;
    else
        mf->i2c_slave = 0x5a;


    /* Use the device as I2C master? */
    mf->is_i2cm = strstr(name, "i2cm") ? 1 : 0;

    /* Type of interface (relevant when is_i2cm==1 or target==MST_DIMM,
       unused otherwise */
    if (find_tavor(name))
        mf->itype = MST_TAVOR;
    else if (strstr(name, "mt47396"))
        mf->itype = MST_TAVOR;         /*  Anafa-2 like Tavor!!! */
    else if (strstr(name, "mt21108"))
        mf->itype = MST_GAMLA;


    /* Type of file */
    if (strstr(name, "pciconf"))
        mf->tp = MST_PCICONF;
    else if (strstr(name, "pci"))
        mf->tp = MST_PCI;
    else if (strstr(name, "mtusb"))
        mf->tp = MST_USB;
    else if (strstr(name, "calbr") || strstr(name, "calibre"))
        mf->tp = MST_CALBR;
    else if (strstr(name, "mif") != 0)
        mf->tp = MST_IF;
#ifdef ENABLE_MST_DEV_I2C
    else if (strstr(name, "dev-i2c") != 0)
        mf->tp = MST_DEV_I2C;
#endif
    else if (strstr(name, "ppc") != 0)
        mf->tp = MST_PPC;       /*  PPC CPU Interface ( PowerQuick ) */
    else
    {
        free(mf);
        errno = ENOENT;
        return 0;
    }

    /* Open it */
    if ((mf->fd = open(real_name, O_RDWR | O_SYNC)) < 0)
    {
        err = errno;
        free(mf);
        errno = err;
        return 0;
    }

    /* Post open actions */
    switch (mf->tp)
    {
    case MST_PCI:
    {
        /* Map memory for PCI interface */
        struct mst_pci_params_st pars;
        int rc;
        CLEAR(pars);

        rc=ioctl(mf->fd, PCI_PARAMS, &pars);
        if (rc < 0)
        {
            err = errno;
            close(mf->fd);
            free(mf);
            errno = err;
            return 0;
        }


        mf->ptr = mmap(NULL, pars.size, PROT_READ | PROT_WRITE,
                       MAP_SHARED, mf->fd, 0);

        if (mf->ptr == MAP_FAILED)
        {
            err = errno;
            close(mf->fd);
            free(mf);
            errno = err;
            return 0;
        }
        break;
    }
    case MST_CALBR:
    /* { */
     /* Dummy transaction (first transaction may fail) */
    /*     u_int32_t val; */
    /*     mread4(mf, mf->dtype == MST_TAVOR ? 0xf0014 : 0x2800, &val); */
          break;
    /* } */
    case MST_PCICONF:
          break;

#ifndef MST_CPU_ppc_8xx
    case MST_USB:
    {
      int rc;
      struct usb_device_descriptor desc;
      rc=usb_get_device_descriptor(mf->fd, &desc);
      if (rc) {
        err = errno;
        close(mf->fd);
        free(mf);
        errno = err;
        return 0;
      }
      if (desc.idVendor == DEVASYS_V && desc.idProduct == DEVASYS_P)
      {
        mf->tp = MST_USB;
      }
      else if (desc.idVendor ==  DIMAX_V && desc.idProduct == DIMAX_P)
      {
        mf->tp = MST_USB_DIMAX;
      }
      else
      {
        close(mf->fd);
        free(mf);
        errno = ENOTTY;
        return 0;
      }

    }
#endif
    case MST_IB:
    case MST_IF:
#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
#endif
        break;

    case MST_PPC:
    {
#ifdef MST_CPU_ppc_8xx
        /* Map memory for PCI interface */
        struct mst_ppc_params_st pars;
        CLEAR(pars);

        if (ioctl(mf->fd, PPC_PARAMS, &pars) < 0)
        {
            err = errno;
            close(mf->fd);
            free(mf);
            errno = err;
            return 0;
        }
        mf->ptr = mmap(NULL, pars.size, PROT_READ | PROT_WRITE,
                       MAP_SHARED, mf->fd, 0);
        if (mf->ptr == MAP_FAILED)
        {
            err = errno;
            close(mf->fd);
            free(mf);
            errno = err;
            return 0;
        }
#endif
        break;
    }

    default:
      close(mf->fd);
      free(mf);
      errno = EINVAL;
      return 0;
    }

    mf->flags=get_device_flags(name);
    /* OK */
    return mf;
}

int mclose(mfile *mf)
{
    int rc;

    if (mf->tp == MST_IB)
        return mib_close(mf);

    if (mf->sock != -1)
    {
        /*  Remote connection */
        writes(mf->sock, "C");
        close(mf->sock);
        mf->sock = -1;
        return 0;
    }

    rc = close(mf->fd);
    free(mf);
    return rc;
}

void maccelerate(mfile *mf)
{
    if (mf->is_i2cm)
        i2c_master_set(mf);
}

void mrestore(mfile *mf)
{
    if (mf->is_i2cm)
        i2c_master_restore(mf);
}

static
int prepare_i2c_buf(void* maddr, DType dtype, u_int32_t offset)
{
        switch(dtype)
        {
        case MST_TAVOR:
        {
            u_int32_t offs = __cpu_to_be32(offset);
            memcpy(maddr, &offs, 4);
            return 4;
        }
        case MST_GAMLA:
        {
            u_int16_t offs = offset & 0xffff;
            offs = __cpu_to_be16(offs);
            memcpy(maddr, &offs, 2);
            return 2;
        }
        case MST_DIMM:
        {
            u_int8_t offs1 = offset & 0xff;
            memcpy(maddr, &offs1, 1);
            return 1;
        }
        default:
            return 0;
        }
}

static inline
int prepare_i2c_data(char* buf, DType dtype, u_int32_t offset, void* data,
    int data_length)
{
  int len;
  len=prepare_i2c_buf(buf,dtype,offset);
  memcpy(buf+len,data, data_length);
  return data_length+len;
}

int mread4(mfile *mf, unsigned int offset, u_int32_t *value)
{
    int rc;
    mif_param_t param;

    if (mf->sock != -1)
    {
        /*  Remote connection */
        char buf[20];

        sprintf(buf, "R 0x%08x", offset);
        writes(mf->sock, buf);
        reads(mf->sock, buf, 20);
        if (*buf == 'O')
        {
            *value = strtoul(buf+2, 0, 0);
            return 4;
        }
        else
        {
            errno = EIO;
            return -1;
        }
    }

    switch(mf->tp)
    {
    case MST_IB:
        return mib_read4(mf, offset, value);

    case MST_IF:
        param.cmd = MREAD4;
        param.buf = value;
        param.len = 4;
        param.addr  = mf->i2c_slave;
        param.offset = offset;
        param.dtype = mf->dtype;
        return(ioctl(mf->fd, param.cmd, &param));


#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
    {
        struct i2c_rdwr_ioctl_data i2c_rdwr;
        struct i2c_msg i2c_msg[2];
        char    maddr[4];
        char    data[4];
        
        i2c_msg[0].addr = mf->i2c_slave;
        i2c_msg[0].flags = 0;
        i2c_msg[0].buf = &maddr[0];
        i2c_msg[1].addr = mf->i2c_slave;
        i2c_msg[1].flags = I2C_M_RD;
        i2c_msg[1].len = 4;
        i2c_msg[1].buf = data;
        i2c_rdwr.msgs = i2c_msg;
        i2c_rdwr.nmsgs = 2;

        i2c_msg[0].len = prepare_i2c_buf(maddr, mf->dtype, offset);

        if (!i2c_msg[0].len)
        {
            i2c_msg[0].flags = I2C_M_RD;
            i2c_msg[0].len = 4;
            i2c_msg[0].buf = data;
            i2c_rdwr.nmsgs = 1;
        }

        rc=ioctl( mf->fd, I2C_RDWR, &i2c_rdwr );
        if( rc<0 ) return rc;
        *value = __be32_to_cpu(*((u_int32_t *)&data[0]));
        return 4;
        return -1;
    }/*MST_DEV_I2C*/
#endif

    case MST_PCI:
    case MST_PPC:
        if (!mf->ptr)
        {
            errno = EFAULT;
            return -1;
        }
        if (mf->is_i2cm)
            return i2c_master_read_cr(mf, value, offset, 4);

        if (mf->dtype == MST_TAVOR)
            *value = __be32_to_cpu(*((u_int32_t *)((char *)mf->ptr + offset)));
        else
            *value = __le32_to_cpu(*((u_int32_t *)((char *)mf->ptr + offset)));
        break;
    case MST_PCICONF:
    {
        struct mst_read4_st r4;

        if (mf->is_i2cm)
            return i2c_master_read_cr(mf, value, offset, 4);

        CLEAR(r4);
        r4.offset = offset;
        if ((rc = ioctl(mf->fd, PCICONF_READ4, &r4)) < 0)
            return rc;
        *value = r4.data;
        break;
    }
    case MST_USB:
    case MST_USB_DIMAX:
    {
        int i;
        int addr_transaction = 1;

        for (i=0; i<MAX_USB_RD_RETRY; i++)
        {
            I2C_TRANS tr;

            tr.byTransType = I2C_TRANS_NOADR;
            tr.bySlvDevAddr = mf->i2c_slave << 1;
            switch(mf->dtype)
            {
            case MST_TAVOR:
            {
                u_int32_t offs = __cpu_to_be32(offset);
                memcpy(tr.Data, &offs, 4);
                tr.wCount = 4;
                break;
            }
            case MST_GAMLA:
            {
                u_int16_t offs = offset & 0xffff;
                offs = __cpu_to_be16(offs);
                memcpy(tr.Data, &offs, 2);
                tr.wCount = 2;
                break;
            }
            case MST_DIMM:
            {
                u_int8_t offs1 = offset & 0xff;
                memcpy(tr.Data, &offs1, 1);
                tr.wCount = 1;
                break;
            }
            case MST_NOADDR:
                addr_transaction = 0;
                tr.wCount = 4;
                break;

            }

            if( mf->tp == MST_USB_DIMAX )
            {
                /*  Dimax Device */
                if( ! addr_transaction )
                    tr.wCount = 0;
                rc = dimax_ReadI2c( mf->fd, &tr, 4 );
                if( rc )
                    return -1;
                *value = __be32_to_cpu(*((u_int32_t *)&tr.Data[0]));
                return 4;
            }
            else
            {
                /*  Write address if neccesary */
                rc = addr_transaction ? mst_WriteI2c(mf->fd, &tr) : 4;
                if (rc > 0)
                {
                    if (rc != tr.wCount)
                    {
                        errno = EIO;
                        return -1;
                    }

                    /*  Read result */
                    tr.wCount = 4;
                    if ((rc = mst_ReadI2c(mf->fd, &tr)) > 0)
                    {
                        if (rc != tr.wCount)
                        {
                            errno = EIO;
                            return -1;
                        }

                        *value = __be32_to_cpu(*((u_int32_t *)&tr.Data[0]));
                        return 4;
                    }
                }
            }
        }
        errno = EPIPE;
        return -1;
    }
    case MST_CALBR:
    {
        struct mst_trans_st tr;
        CLEAR(tr);

        tr.slv_addr = mf->i2c_slave;
        tr.wr_size = prepare_i2c_buf(tr.wr_buff, mf->dtype, offset);
        tr.rd_size = sizeof(u_int32_t);

        if ((rc = ioctl(mf->fd, CALBR_TRANS, &tr)) < 0)
            return -1;
        if (tr.rd_count <  sizeof(u_int32_t))
            return tr.rd_count;
        *value = __be32_to_cpu(*((u_int32_t *)&tr.rd_buff[0]));
        break;
    }
    }
    return 4;
}

int mwrite4(mfile *mf, unsigned int offset, u_int32_t value)
{
    int rc;
    mif_param_t param;

    if (mf->sock != -1)
    {
        /*  Remote connection */
        char buf[40];

        sprintf(buf, "W 0x%08x 0x%08x", offset, value);
        /* printf("W 0x%08x 0x%08x", offset, value); */
        writes(mf->sock, buf);
        reads(mf->sock, buf, 40);
        if (*buf == 'O')
            return 4;
        else
        {
            errno = EIO;
            return -1;
        }
    }

    switch(mf->tp)
    {
    case MST_IB:
        return mib_write4(mf, offset, value);

    case MST_IF:
        param.cmd = MWRITE4;
        param.buf = &value;
        param.len = 4;
        param.addr = mf->i2c_slave;
        param.offset = offset;
        param.dtype = mf->dtype;
        return(ioctl(mf->fd, param.cmd, &param));

#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
    {
        struct i2c_rdwr_ioctl_data i2c_rdwr;
        struct i2c_msg i2c_msg[1];
        char data[8];
        
        i2c_msg[0].addr = mf->i2c_slave;
        i2c_msg[0].flags = 0;
        i2c_msg[0].buf = data;
        i2c_rdwr.msgs = i2c_msg;
        i2c_rdwr.nmsgs = 1;

        value = __cpu_to_be32(value);
        i2c_msg[0].len = prepare_i2c_data(data, mf->dtype, offset,
            &value, sizeof(value));

        rc=ioctl( mf->fd, I2C_RDWR, &i2c_rdwr );
        if( rc<0 ) return rc;
        return 4;
    } /*MST_DEV_I2C*/
#endif

    case MST_PCI:
    case MST_PPC:
        if (!mf->ptr)
        {
            errno = EFAULT;
            return -1;
        }
        if (mf->is_i2cm)
            return i2c_master_write_cr(mf, value, offset, 4);

        if (mf->dtype == MST_TAVOR)
            *((u_int32_t *)((char *)mf->ptr + offset)) = __cpu_to_be32(value);
        else
            *((u_int32_t *)((char *)mf->ptr + offset)) = __cpu_to_le32(value);
        break;
    case MST_PCICONF:
    {
        struct mst_write4_st r4;

        if (mf->is_i2cm)
            return i2c_master_write_cr(mf, value, offset, 4);

        CLEAR(r4);
        r4.offset = offset;
        r4.data = value;
        if ((rc = ioctl(mf->fd, PCICONF_WRITE4, &r4)) < 0)
            return rc;
        break;
    }
    case MST_USB:
    case MST_USB_DIMAX:
    {
        int i;
        for (i=0; i<MAX_USB_RD_RETRY; i++)
        {
            I2C_TRANS tr;

            tr.byTransType = I2C_TRANS_NOADR;
            tr.bySlvDevAddr = mf->i2c_slave << 1;
            value = __cpu_to_be32(value);
            tr.wCount = prepare_i2c_data(tr.Data, mf->dtype, offset,
                &value, sizeof(value));

            if( mf->tp == MST_USB_DIMAX )
            {
                tr.wCount -= sizeof(value);     /*  leave only address size */
                rc = dimax_WriteI2c( mf->fd, &tr, 4 );
                if( rc )
                    return -1;
                return 4;
            }
            else
            {
                /*  Write address and data */
                if ((rc = mst_WriteI2c(mf->fd, &tr)) > 0)
                {
                    if (rc != tr.wCount)
                    {
                        errno = EIO;
                        return -1;
                    }
                    return 4;
                }
            }
        }
        errno = EPIPE;
        return -1;
    }
    case MST_CALBR:
    {
        struct mst_trans_st tr;
        CLEAR(tr);

        tr.slv_addr = mf->i2c_slave;
        value = __cpu_to_be32(value);
        tr.wr_size=prepare_i2c_data( tr.wr_buff, mf->dtype, offset, &value,
            sizeof(value));

        tr.rd_size = 0;

        if ((rc = ioctl(mf->fd, CALBR_TRANS, &tr)) < 0)
            return -1;
        if (tr.wr_count <  sizeof(u_int32_t))
            return tr.wr_count;
        break;
    }
    }
    return 4;
}

int mread64(mfile *mf, unsigned int offset, void *data, int length)
{
    int                 rc;
    mif_param_t         param;

    if (length > MAX_TRANS_SIZE)
    {
        errno = EINVAL;
        return -1;
    }

    switch(mf->tp)
    {
    case MST_IB:
        return mib_read64(mf, offset, data, length);


    case MST_IF:
        param.cmd = MREAD64;
        param.buf = data;
        param.len    = length;
        param.addr   = mf->i2c_slave;
        param.offset = offset;
        param.dtype = mf->dtype;
        return(ioctl(mf->fd, param.cmd, &param));

#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
    {
        struct i2c_rdwr_ioctl_data i2c_rdwr;
        struct i2c_msg i2c_msg[2];
        char    maddr[4];
        
        i2c_msg[0].addr = mf->i2c_slave;
        i2c_msg[0].flags = 0;
        i2c_msg[0].buf = &maddr[0];
        i2c_msg[1].addr = mf->i2c_slave;
        i2c_msg[1].flags = I2C_M_RD;
        i2c_msg[1].len = length;
        i2c_msg[1].buf = data;
        i2c_rdwr.msgs = i2c_msg;
        i2c_rdwr.nmsgs = 2;
        i2c_msg[0].len=prepare_i2c_buf(maddr,mf->dtype,offset);

        if (!i2c_msg[0].len)
        {
            i2c_msg[0].flags = I2C_M_RD;
            i2c_msg[0].len = length;
            i2c_msg[0].buf = data;
            i2c_rdwr.nmsgs = 1;
        }

        rc=ioctl( mf->fd, I2C_RDWR, &i2c_rdwr );
        if( rc<0 ) return rc;
        return length;
        }/*MST_DEV_I2C*/
#endif

    case MST_CALBR:
    {
        struct mst_trans_st tr;
        CLEAR(tr);

        tr.slv_addr = mf->i2c_slave;
        tr.wr_size =prepare_i2c_buf(tr.wr_buff, mf->dtype, offset);
        tr.rd_size = length;

        if ((rc = ioctl(mf->fd, CALBR_TRANS, &tr)) < 0)
            return -1;
        memcpy(data, tr.rd_buff, tr.rd_count);
        return tr.rd_count;
    }
    case MST_USB:
    case MST_USB_DIMAX:
    {
        int       addr_transaction = 1;
        I2C_TRANS tr;
        CLEAR(tr);

        tr.byTransType = I2C_TRANS_NOADR;
        tr.bySlvDevAddr = mf->i2c_slave << 1;
        tr.wCount = prepare_i2c_buf(tr.Data, mf->dtype, offset);
        if (!tr.wCount)
        {
            addr_transaction = 0;
            tr.wCount = 4;
        }

        if( mf->tp == MST_USB_DIMAX )
        {
            /*  Dimax Device */
            if( addr_transaction == 0 )
                tr.wCount = 0;
            rc = dimax_ReadI2c( mf->fd, &tr, length );
            if( rc )
                return -1;
            memcpy(data, &tr.Data[0], length);
            return length;
        }
        else
        {
            /*  Write address */
            rc = addr_transaction ? mst_WriteI2c(mf->fd, &tr) : 4;
            if (rc < 0)
                return -1;
            /*  Read result */
            tr.wCount = length;
            if ((rc = mst_ReadI2c(mf->fd, &tr)) < 0)
                return -1;
            memcpy(data, &tr.Data[0], length);
            return length;
        }
    }
    case MST_PCI:
    case MST_PCICONF:
    {
        int i;
        unsigned char *cdata = (unsigned char *)data;

        for (i=0; i<length; i++)
        {
            rc = i2c_master_read_cr(mf, cdata++, offset++, 1);
            if (rc < 0)
                return rc;
            else if (rc < 1)
                return i;
        }
        return length;
    }
    case MST_PPC:
        errno = ECANCELED;
        return -1;
    }

    errno = EPERM;
    return -1;
}

int mwrite64(mfile *mf, unsigned int offset, void *data, int length)
{
    int                 rc;
    mif_param_t param;

    if (length > MAX_TRANS_SIZE)
    {
        errno = EINVAL;
        return -1;
    }


    switch(mf->tp)
    {
    case MST_IB:
        return mib_write64(mf, offset, data, length);


    case MST_IF:
        param.cmd = MWRITE64;
        param.buf = data;
        param.offset= offset;
        param.len   = length;
        param.addr  = mf->i2c_slave;
        param.dtype = mf->dtype;
        return(ioctl(mf->fd, param.cmd, &param));

#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
    {
        struct i2c_rdwr_ioctl_data i2c_rdwr;
        struct i2c_msg i2c_msg[1];
        char buf[64+4];
        i2c_msg[0].addr = mf->i2c_slave;
        i2c_msg[0].flags = 0;
        i2c_msg[0].buf = buf;
        i2c_rdwr.msgs = i2c_msg;
        i2c_rdwr.nmsgs = 1;

        i2c_msg[0].len = prepare_i2c_data(buf, mf->dtype, offset, data, length);
        rc=ioctl( mf->fd, I2C_RDWR, &i2c_rdwr );
        if( rc<0 ) return rc;
        return length;
    }/*MST_DEV_I2C*/
#endif

    case MST_CALBR:
    {
        struct mst_trans_st tr;
        int                 offs_size=0;
        CLEAR(tr);

        tr.slv_addr = mf->i2c_slave;
        tr.wr_size = prepare_i2c_data( tr.wr_buff, mf->dtype, offset, data,
            length);

        tr.rd_size = 0;

	offs_size=tr.wr_size-length;

        if ((rc = ioctl(mf->fd, CALBR_TRANS, &tr)) < 0)
            return -1;
        return (tr.wr_count < offs_size) ? 0 : tr.wr_count - offs_size;
    }
    case MST_USB:
    case MST_USB_DIMAX:
    {
        I2C_TRANS tr;
        CLEAR(tr);

        tr.byTransType = I2C_TRANS_NOADR;
        tr.bySlvDevAddr = mf->i2c_slave << 1;
        tr.wCount = prepare_i2c_data(tr.Data, mf->dtype, offset, data, length);

        if( mf->tp == MST_USB_DIMAX )
        {
            tr.wCount -= length;        /*  leave only address size */
            rc = dimax_WriteI2c( mf->fd, &tr, length );
            if( rc )
                return -1;
            return length;
        }
        
        /*  Write address and data */
        if ((rc = mst_WriteI2c(mf->fd, &tr)) < 0)
            return -1;
        return length;
    }
    case MST_PCI:
    case MST_PCICONF:
    {
        int i;
        unsigned char *cdata = (unsigned char *)data;

        for (i=0; i<length; i++)
        {
            rc = i2c_master_write_cr(mf, *cdata++, offset++, 1);
            if (rc < 0)
                return rc;
            else if (rc < 1)
                return i;
        }
        return length;
    }
    case MST_PPC:
        errno = ECANCELED;
        return -1;

    }

    errno = EPERM;
    return -1;
}

//Read up to length
//Length is guaranteed to be up to what get_chunk_size returns
static int 
mread_chunk(mfile *mf, unsigned int offset, void *data, int length)
{
  int                 rc;
  mif_param_t         param;

  if (mf->sock != -1)
  {
    u_int32_t value;
    /*  Remote connection */
    char buf[20];

    sprintf(buf, "R 0x%08x", offset);
    writes(mf->sock, buf);
    reads(mf->sock, buf, 20);
    if (*buf != 'O')
    {
      errno = EIO;
      return -1;
    }
    value = strtoul(buf+2, 0, 0);
    value = __cpu_to_be32(value);
    memcpy(data,&value,length);
    return length;
  }

  if (mf->is_i2cm) return i2c_master_read(mf, data, offset, length);

  switch(mf->tp)
  {
    case MST_IB:
      return mib_read64(mf, offset, data, length);


    case MST_IF:
      param.cmd = MREAD64;
      param.buf = data;
      param.len    = length;
      param.addr   = mf->i2c_slave;
      param.offset = offset;
      param.dtype = mf->dtype;
      return(ioctl(mf->fd, param.cmd, &param));

#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
      return 0;
#endif

    case MST_CALBR:
      {
        struct mst_trans_st tr;
        CLEAR(tr);

        tr.slv_addr = mf->i2c_slave;
        tr.wr_size = prepare_i2c_buf(tr.wr_buff, mf->dtype, offset);
        tr.rd_size = length;

        if ((rc = ioctl(mf->fd, CALBR_TRANS, &tr)) < 0)
          return -1;
        memcpy(data, tr.rd_buff, tr.rd_count);
        return tr.rd_count;
      }
    case MST_USB:
    case MST_USB_DIMAX:
      {
        int i;
        int       addr_transaction = 1;
        for (i=0; i<MAX_USB_RD_RETRY; i++)
        {
          I2C_TRANS tr;
          CLEAR(tr);

          tr.byTransType = I2C_TRANS_NOADR;
          tr.bySlvDevAddr = mf->i2c_slave << 1;
          tr.wCount = prepare_i2c_buf(tr.Data, mf->dtype, offset);
          if (!tr.wCount)
          {
            addr_transaction = 0;
            tr.wCount = 4;
          }

          if( mf->tp == MST_USB_DIMAX )
          {
            /*  Dimax Device */
            if( addr_transaction == 0 )
              tr.wCount = 0;
            rc = dimax_ReadI2c( mf->fd, &tr, length );
            if( rc )
              return -1;
            memcpy(data, &tr.Data[0], length);
            return length;
          }
          else
          {
            /*  Write address */
            rc = addr_transaction ? mst_WriteI2c(mf->fd, &tr) : 4;
            if (rc < 0)
              return -1;
            /*  Read result */
            tr.wCount = length;
            if ((rc = mst_ReadI2c(mf->fd, &tr)) < 0)
              return -1;
            memcpy(data, &tr.Data[0], length);
            return length;
          }
        }
        errno = EPIPE;
        return -1;
      }
    case MST_PCI:
    case MST_PPC:
      {
        memcpy(data,(char *)mf->ptr + offset,length);
        return length;
      }
    case MST_PCICONF:
      {
        struct mst_read4_st r4;
        CLEAR(r4);
        r4.offset = offset;
        if ((rc = ioctl(mf->fd, PCICONF_READ4, &r4)) < 0)
          return rc;
        //PCI CONF returns data in CPU format, put it back to original.
        r4.data=__cpu_to_be32(r4.data);
        memcpy(data,&r4.data,length);
        return rc;
      }

    default:
      errno = EPERM;
      return -1;
  }
}

static int get_chunk_size(mfile *mf)
{
	/*  Remote connection */
    if (mf->sock != -1) return 4;
    switch(mf->tp)
    {
    case MST_IB:
    case MST_IF:
    case MST_CALBR:
    case MST_USB:
        return 64;
    case MST_USB_DIMAX:
        return 256; //DIMAX FW uses 8-bit counters, thats why.
#ifdef ENABLE_MST_DEV_I2C
    case MST_DEV_I2C:
        return 0;   /* Will be 8K after it will be implemented in mread_by_chunks */
#endif
    default:
	//TODO: for i2c masters we can also
	//implement speedups by doing bursts
	return 4;
    }
}

int mread_by_chunks(mfile *mf_cr,
		unsigned offset, void* dest, int size)
{
	int chunk_size;
	int left_size;
	int bytes;
	void* dest_ptr=dest;

	chunk_size=get_chunk_size(mf_cr);
	
	for (left_size=size;left_size>0;left_size-=chunk_size)
	{
		int toread;
		toread=(left_size>=chunk_size)?chunk_size:left_size;
		bytes=mread_chunk (mf_cr, offset, dest_ptr, toread);
		if (bytes!=toread) {
			return size-left_size;
		}

		offset += chunk_size;
		dest_ptr+=chunk_size;
	}
	return size;

}



/*
 * Get signal and do nothing.
 *
 */
static void
sig_pipe(int signo)
{
    printf("Got SIGPIPE...\n");
    return;
}

unsigned char mset_i2c_slave(mfile *mf, unsigned char new_i2c_slave)
{
    unsigned char ret;
    if (mf)
    {
       ret = mf->i2c_slave;
       mf->i2c_slave = new_i2c_slave;
    }
    else
       ret = 0xff;
    return ret;
}

void *
mget_free_pages(mfile *mf, unsigned int order)
{
    unsigned page_size = 4096;

    if (mf->sock != -1)
    {
        /* unsupported on remote connection */
        errno = -EOPNOTSUPP;
        return 0;
    }

    switch(mf->tp)
    {
    case MST_PPC:
        {
            unsigned int *addr;
            addr = mmap (0, page_size << order, PROT_READ | PROT_WRITE,
                        MAP_SHARED, mf->fd, page_size);

            if (addr == MAP_FAILED)
                    return 0;
            else
                    return addr;
        }

    default:
        errno = -EOPNOTSUPP;
        return 0;
    }
}

int
mfree_pages(mfile *mf, void *addr, unsigned int order)
{
    unsigned page_size = 4096;

    if (mf->sock != -1)
    {
        /* unsupported on remote connection */
        errno = -EOPNOTSUPP;
        return -1;
    }

    switch (mf->tp)
    {
    case MST_PPC:
        {
            return munmap (addr, page_size << order);
        }

    default:
        errno = -EOPNOTSUPP;
        return -1;
    }
}

unsigned long
mvtop (mfile *mf, void *va)
{
#ifdef MST_CPU_ppc_8xx
    struct mst_ppc_vtop vtop;

    vtop.va = va;
#endif

    switch(mf->tp)
    {
#ifdef MST_CPU_ppc_8xx
    case MST_PPC:
        if (ioctl (mf->fd, PPC_VTOP, &vtop) < 0) {
                return 0;
        } else {
                return vtop.pa;
        }
#endif
    default:
        errno = -EOPNOTSUPP;
        return 0;
    }
}


