#include <sys/ioctl.h>
#include <sys/types.h> /*This header likes to sit before linux/ headers*/

#include <linux/version.h>
/*  Linux USB Support */
#include <linux/usb_ch9.h>
#include <linux/usbdevice_fs.h>
#include <asm/byteorder.h>


static inline int usb_control_msg(int fd, int requesttype, int request,
  int value, int index, char *bytes, int size, int timeout)
{
  struct usbdevfs_ctrltransfer ctrl;
  int ret;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
  ctrl.requesttype = requesttype;
  ctrl.request = request;
  ctrl.value = value;
  ctrl.index = index;
  ctrl.length = size;
#else
  ctrl.bRequestType = requesttype;
  ctrl.bRequest = request;
  ctrl.wValue = value;
  ctrl.wIndex = index;
  ctrl.wLength = size;
#endif

  ctrl.data = bytes;
  ctrl.timeout = timeout;

  ret = ioctl(fd, USBDEVFS_CONTROL, &ctrl);

  //return ret<0?ret:0;
  return ret<0?ret:0;
}
        
static inline int usb_get_descriptor(int fd, unsigned char type,
  unsigned char index, void *buf, int size)
{
  memset(buf, 0, size);

  return usb_control_msg(fd, USB_DIR_IN|USB_TYPE_STANDARD, 
      USB_REQ_GET_DESCRIPTOR, (type << 8) + index, 0, buf, size, 1000);
}

static inline int usb_get_device_descriptor(int fd, struct usb_device_descriptor* buf)
{
  int rc=usb_get_descriptor(fd, USB_DT_DEVICE, 0,
      buf, USB_DT_DEVICE_SIZE);
  buf->idVendor=__le16_to_cpu(buf->idVendor);
  buf->idProduct=__le16_to_cpu(buf->idProduct);
  return rc;
}
