/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

#ifndef __CLASS_PORT_INFO_H__
#define __CLASS_PORT_INFO_H__
 
#include "ib-types.h"
 
typedef struct _ib_class_port_info
{
        uint8_t  base_version;
        uint8_t  class_version;
        uint16_t capability_mask;

#if CPU_BE
        uint32_t reserved1 : 27;
        uint32_t resp_time_value : 5;
#else /* CPU_LE */
        uint32_t resp_time_value : 5;
        uint32_t reserved1 : 27;
#endif

        vv_gid_t redirect_gid;

#if CPU_BE
        uint32_t redirect_tc : 8;
        uint32_t redirect_sl : 4;
        uint32_t redirect_fl : 20;
#else /* CPU_LE */
        uint32_t redirect_fl : 20;
        uint32_t redirect_sl : 4;
        uint32_t redirect_tc : 8;
#endif

        vv_lid_t redirect_lid;
        uint16_t redirect_p_key;

#if CPU_BE
        uint32_t reserved2   : 8;
        uint32_t redirect_qp : 24;
#else /* CPU_LE */
        uint32_t redirect_qp : 24;
        uint32_t reserved2   : 8;
#endif

        uint32_t redirect_q_key;
        vv_gid_t trap_gid;

#if CPU_BE
        uint32_t trap_tc : 8;
        uint32_t trap_sl : 4;
        uint32_t trapfl  : 20;
#else /* CPU_LE */
        uint32_t trapfl  : 20;
        uint32_t trap_sl : 4;
        uint32_t trap_tc : 8;
#endif

        vv_lid_t trap_lid;
        uint16_t trap_p_key;

#if CPU_BE
        uint32_t trap_hl : 8;
        uint32_t trap_qp : 24;
#else /* CPU_LE */
        uint32_t trap_qp : 24;
        uint32_t trap_hl : 8;
#endif

        uint32_t trap_q_key;
} ib_class_port_info_t;

#if CPU_LE
static inline void class_port_ntoh( IN char *dest, IN char *src, IN u8 size )
{
        u8      i;
        char    temp;
        if( src == dest )
        {
                // Swap in place if source and destination are the same.
                for( i = 0; i < size / 2; i++ )
                {
                        temp = dest[i];
                        dest[i] = src[size - 1 - i];
                        src[size - 1 - i] = temp;
                }
        }
        else
        {
                for( i = 0; i < size; i++ )
                        dest[i] = src[size - 1 - i];
        }
}
#else
#define class_port_ntoh(a,b,c) if(a != b)memcpy(a,b,c)
#endif

static inline void class_port_info_swap( ib_class_port_info_t *info) 
{
        info->capability_mask    = htons(info->capability_mask);
        info->reserved1          = htonl(info->reserved1)>>5;
        class_port_ntoh((char *)&info->redirect_gid, (char *)&info->redirect_gid, sizeof(info->redirect_gid));
        info->redirect_fl        = htonl(info->redirect_fl)>>12;
        info->redirect_lid       = htons(info->redirect_lid);
        info->redirect_p_key     = htons(info->redirect_p_key);
        info->redirect_qp        = (htonl(info->redirect_qp))>>8;
        info->redirect_q_key     = htonl(info->redirect_q_key);
        class_port_ntoh((char *)&info->trap_gid, (char *)&info->trap_gid, sizeof(info->trap_gid));
        info->trapfl             = htonl(info->trapfl)>>12;
        info->trap_lid           = htons(info->trap_lid);
        info->trap_p_key         = htons(info->trap_p_key);
        info->trap_qp            = htonl(info->trap_qp)>>8;
        info->trap_q_key         = htonl(info->trap_q_key);
}
 
#endif /* __CLASS_PORT_INFO_H__ */
