/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

/*
* File: debug-support.h - Voltaire common debug support.          
* $Header: /export/cvs/cvs-repository/nvigor/ib-code/include/debug-support.h,v 1.16 2005/01/03 13:35:45 galiz Exp $
*
* TODO: Cleanup the VD_ERR remains.
*
*
* Purpose: Provide debug support for Voltaire Software Teams.
*
* Author: Moni Levy <monil@voltaire.com>.
* All rights reserved (C) Voltaire, 2001.
*
* Defines the IN,OUT words ,messaging macros and general debug accessories.
*
* Depends on: 
* VDBG - debug version (Makefile)
* VD_DEBUGGER_ON(Makefile) - debugger present (will enable embedding of hardcoded int 3 at asserts and errors).
* VD_MODULE_NAME (optional)(Makefile) - name of the module.
* VD_TRACE_LEVEL (optional)(Makefile) - Used as initializer for the max_trace threshold (if not defined will be 5).
* VD_ERROR_LEVEL (optional)(Makefile) - Used as initializer for the max_error threshold (if not defined will be 5).
* VD_ENTERLEAVE_LEVEL (optional)(Makefile) - Used as initializer for the trace level of the VD_*****FUNC macros.
*
* VD_MAIN_MODULE(module) - should be defined only in one of the modules BEFOR this file is included (the two "level" flags are 
*                     allocated in the main module, and in all the other are defined as "extern" in order to enable only 
*		      one initialization per project).
*
* VD_MODULE_NAME_trace_level(for runtime only) - trace level (all the trace messages below the defined will be printed).
* VD_MODULE_NANE_error_level(for runtime only) - errol level (all the error messages below the defined will stop the program flow).
* 
* Defines the next macros:
* VD_ENTERFUNC(x...) - Start of function execution.
* VD_LEAVEFUNC(x...) - End of function execution.
* VD_NOT_IMPLEMENTED() - Function not implemented.
* VD_DEBUGMSG(fmt, args...) - Debug message preceded by standard header(KERN_DEBUG).
* VD_ERRORMSG(fmt, args...) - Debug message preceded by standard header(KERN_ERR).
* VD_ASSERT(expr) - Assert (if VD_DEBUGGER_ON - WILL generate INT3).
* VD_TRACE(lvl, fmt, args...) - Debug message preceded by standard header depends on max_trace (see trace level).
* VD_ERR(lvl, fmt, args...) - Debug message preceded by standard header depends on max_error (see error level).If VD_DEBUGGER_ON - MAY generate INT3.
* VD_BREAKPOINT() - Hardcoded breakpoint. (if VD_DEBUGGER_ON - WILL generate INT3)
* VD_DBGCODE(lvl, code) - The "code" vanishes in the release version.
*
* VD_DEBUGMSGNL(fmt, args...) - Automatically adds newline to the printed message.
* VD_ERRORMSGNL(fmt, args...) - Automatically adds newline to the printed message.
* VD_TRACENL(lvl, fmt, args...) - Automatically adds newline to the printed message.
* VD_ERRORNL(lvl, fmt, args...) - Automatically adds newline to the printed message.
*
* Severity levels - additional user defined levels may be defined (7 and above) and be visible only to the user if desired.
* VD_PANIC - Memory/Integrity corruptions or non recoverable errors.(0)
* VD_ERROR - Recoverable errors.(1)
* VD_WARNING - Warnings.(2)
* VD_DEBUG - Regular debug messages.(3)
* VD_FLOW - Program flow messages.(4)
* VD_INFO - General info messages.(5)
* VD_DUMP - Memory buffers dumps.(6)
* 
*/

#ifndef _DEBUG_SUPPORT_H_
#define _DEBUG_SUPPORT_H_

#ifdef __KERNEL__
#include <linux/version.h>	// bring KERNEL_VERSION and related macros
#endif

#ifdef VD_LOGGER_ON
 #undef VDBG
#endif

// IN, OUT empty declarations.
#ifndef IN
# define IN
#endif

#ifndef OUT
# define OUT
#endif

#define TOSTRTOKEN(X) #X
#define TOSTRING(A,B) A ## B
#define STRINGIFY(X) TOSTRTOKEN(X)

#ifdef __KERNEL__
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
#define VD_FILE_NAME STRINGIFY(KBUILD_BASENAME) STRINGIFY(.c)
#else
#define VD_FILE_NAME __FILE__
#endif
#else
#define VD_FILE_NAME __FILE__
#endif


#ifdef VDBG

//severity levels
#define VD_PANIC        0 // Memory/Integrity corruptions or non recoverable errors.
#define VD_ERROR        1 // Recoverable errors.
#define VD_WARNING      2 // Warnings
#define VD_INFO         3 // General info messages
#define VD_DEBUG        4 // Regular debug messages
#define VD_FLOW         5 // Program flow messages
#define VD_DUMP         6 // Memory buffers dumps

// Module name and level defaults
# ifndef VD_MODULE_NAME
#  define VD_MODULE_NAME  vgeneral
# endif

# ifndef VD_TRACE_LEVEL
#  define VD_TRACE_LEVEL 5
# endif

# ifndef VD_ERROR_LEVEL
#  define VD_ERROR_LEVEL 5
# endif

# ifndef VD_ENTERLEAVE_LEVEL 
#  define VD_ENTERLEAVE_LEVEL 10
# endif


// Level variables tail string token
# define MAX_TRACE_TOKEN  _trace_level
# define MAX_ERROR_TOKEN  _error_level



# define VD_MAIN_MODULE_INIT(max_trace,max_error)             \
         INITMAINL(VD_MODULE_NAME , MAX_TRACE_TOKEN , MAX_ERROR_TOKEN , max_trace, max_error)
# define VD_SECONDARY_MODULE_INIT()                           \
         INITOTHERL(VD_MODULE_NAME , MAX_TRACE_TOKEN , MAX_ERROR_TOKEN)

# define VD_ENTERFUNC(x...)                                                        \
  	     ENTERFUNCL (__LINE__ , VD_MODULE_NAME , MAX_TRACE_TOKEN , VD_ENTERLEAVE_LEVEL , ##x)

# define VD_LEAVEFUNC(x...)                                                        \
  	     LEAVEFUNCL (__LINE__ , VD_MODULE_NAME , MAX_TRACE_TOKEN , VD_ENTERLEAVE_LEVEL , ##x)
 
# define VD_DEBUGMSG(fmt,args...)   DEBUGMSGL (__LINE__ , VD_MODULE_NAME , fmt , ##args)
# define VD_ERRORMSG(fmt,args...)   ERRORMSGL (__LINE__ , VD_MODULE_NAME , fmt , ##args)

# define VD_NOT_IMPLEMENTED() VD_DEBUGMSGNL ("FUNCTION NOT IMPLEMENTED !!!") 

# define VD_ASSERT(expr) 						\
        do                                                              \
        {                                                               \
 	  if(!(expr)) 							\
 	  {						                \
  		VD_DEBUGMSGNL("ASSERTION FAILED - (%s)" , #expr); 	\
 		BREAKPOINTL();						\
 	  }                                                             \
	} while(0)                                              
 
# define VD_TRACE(lvl,fmt,args...) TRACEL(VD_MODULE_NAME , MAX_TRACE_TOKEN , lvl, fmt , ##args)
//# define VD_ERR(lvl,fmt,args...) ERRORL(VD_MODULE_NAME , MAX_ERROR_TOKEN , lvl, fmt , ##args)
# define VD_ERR(lvl,fmt,args...) "VD_ERR macro is not supported anymore !!!"


# define INITMAINL(module_name,trace_token,error_token,max_trace,max_error)     \
	int TOSTRING(module_name,trace_token) = max_trace;                      \
	int TOSTRING(module_name,error_token) = max_error                            

# define INITOTHERL(module_name,trace_token,error_token)                        \
	extern int TOSTRING(module_name,trace_token);                           \
	extern int TOSTRING(module_name,error_token)                            

# define TRACEL(module_name, trace_token, lvl, fmt, args...)                    \
        do                                                                      \
        {                                                                       \
  	  if((TOSTRING(module_name,trace_token)) >= lvl)                        \
 	  {						                        \
 	     DEBUGMSGL(__LINE__ , VD_MODULE_NAME , fmt , ##args);	        \
 	  }		                                                        \
	} while(0)                                              
	
# define ERRORL(module_name, error_token, lvl, fmt, args...)    \
        do                                                      \
        {                                                       \
          ERRORMSGL(__LINE__ , VD_MODULE_NAME , fmt , ##args);	\
	  if((TOSTRING(module_name,error_token)) >= lvl)        \
	  {						        \
	       BREAKPOINTL();                                   \
	  }                                                     \
	} while(0)                                              

# define VD_BREAKPOINT() BREAKPOINTL()
# define VD_DEBUGMSGNL(fmt, args...) DEBUGMSGL (__LINE__ , VD_MODULE_NAME , fmt "\n" , ##args)
# define VD_ERRORMSGNL(fmt, args...) ERRORMSGL (__LINE__ , VD_MODULE_NAME , fmt "\n" , ##args)
# define VD_TRACENL(lvl, fmt, args...) VD_TRACE(lvl , fmt "\n" , ##args)
# define VD_ERRORNL(lvl, fmt, args...) "VD_ERRORNL macro is not supported anymore !!!"
# define VD_DBGCODE(lvl, code)                                                  \
        do                                                                      \
        {                                                                       \
          if((TOSTRING(module_name,trace_token)) >= lvl)                        \
  	  {                                                                     \
             code;                                                              \
          }		                                                        \
        } while(0)



// Initialization on the level variables.
# ifdef VD_MAIN_MODULE
 VD_MAIN_MODULE_INIT(VD_TRACE_LEVEL , VD_ERROR_LEVEL);
# else
 VD_SECONDARY_MODULE_INIT();
# endif

#  define VD_LOGGER_INIT(branch)         
#  define VD_LOGGER_EXIT()              

# ifdef __KERNEL__
// Lower level functions valid only in Kernel mode

#  define DEBUGMSGL(line, module_name, fmt, args...)           \
          printk (KERN_DEBUG TOSTRTOKEN( module_name ) ":%s(%s," TOSTRTOKEN(line) "): " fmt , __FUNCTION__ , VD_FILE_NAME, ##args)
#  define ERRORMSGL(line, module_name, fmt, args...)           \
          printk (KERN_ERR TOSTRTOKEN( module_name ) ":%s(%s," TOSTRTOKEN(line) "): " fmt , __FUNCTION__ , VD_FILE_NAME, ##args)

#  ifdef VD_DEBUGGER_ON
#   define BREAKPOINTL() asm("int $3")
#  else
#   define BREAKPOINTL() VD_DEBUGMSGNL("!!! BREAK HERE !!!")
#  endif 

#  define ENTERFUNCL(line, module_name, lvltkn, lvl, x...)                                                             \
        do                                                                                                             \
        {                                                                                                              \
  	  if((TOSTRING(module_name, lvltkn)) >= lvl)                                                                   \
 	  {						                                                               \
          printk (KERN_DEBUG TOSTRTOKEN( module_name ) ":%s (%s," TOSTRTOKEN(line) "): {\n" , __FUNCTION__, VD_FILE_NAME); \
 	  }		                                                                                               \
	} while(0)                                              

#  define LEAVEFUNCL(line, module_name, lvltkn, lvl, x...)                                                             \
        do                                                                                                             \
        {                                                                                                              \
  	  if((TOSTRING(module_name, lvltkn)) >= lvl)                                                                   \
 	  {						                                                               \
          printk (KERN_DEBUG TOSTRTOKEN( module_name ) ":%s(%s," TOSTRTOKEN(line) "): }\n" , __FUNCTION__, VD_FILE_NAME); \
 	  }		                                                                                               \
	} while(0)                                              

# else //__KERNEL__ not defined
// Lower level functions valid only in User mode.

#  define DEBUGMSGL(line, module_name, fmt, args...)           \
          fprintf (stderr, TOSTRTOKEN( module_name ) ":" __FUNCTION__ "(" __FILE__ "," TOSTRTOKEN(line) "): " fmt, ##args)
#  define ERRORMSGL(line, module_name, fmt, args...)           \
          DEBUGMSGL(line , module_name , fmt , ##args)
#  define BREAKPOINTL() VD_DEBUGMSGNL("!!! BREAK HERE !!!")			
#  define ENTERFUNCL(line, module_name, x...)                  \
          fprintf (stderr, KERN_DEBUG TOSTRTOKEN( module_name ) ":" __FUNCTION__ "(" __FILE__ "," TOSTRTOKEN(line) "): {\n" x)
#  define LEAVEFUNCL(line, module_name, x...)                  \
          fprintf (stderr, KERN_DEBUG TOSTRTOKEN( module_name ) ":" __FUNCTION__ "(" __FILE__ "," TOSTRTOKEN(line) "): }\n" x)

# endif //def __KERNEL__

#else 
# ifdef VD_LOGGER_ON //VDBG not defined
# include "logger.h"

#define VD_PANIC        VDLOG_PANIC // Memory/Integrity corruptions or non recoverable errors.
#define VD_ERROR        VDLOG_ERROR // Recoverable errors.
#define VD_WARNING      VDLOG_WARNING // Warnings
#define VD_INFO         VDLOG_INFO // Regular debug messages
#define VD_DEBUG        VDLOG_DEBUG // Program flow messages
#define VD_FLOW         VDLOG_FUNC // General info messages
#define VD_DUMP         VDLOG_FUNC // Memory buffers dumps

# ifdef VD_MAIN_MODULE
 INIT_VLOGGER_TRACE_LEVEL;  //declares the runtime logger level 
# endif
  
#  define VD_ENTERFUNC(x...)             
#  define VD_LEAVEFUNC(x...)             
#  define VD_NOT_IMPLEMENTED()
#  define VD_DEBUGMSG(fmt, args...)      VD_LOG(VD_DEBUG ,"%s:(%s):" fmt , __FUNCTION__ , VD_FILE_NAME, ##args)
#  define VD_ERRORMSG(fmt, args...)      VD_LOG(VD_ERROR ,"%s:(%s):" fmt , __FUNCTION__ , VD_FILE_NAME, ##args)
#  define VD_ASSERT(expr)
#  define VD_TRACE(lvl, fmt, args...)    VD_LOG(lvl,"%s:(%s):" fmt , __FUNCTION__ , VD_FILE_NAME, ##args)
#  define VD_ERR(lvl, fmt, args...)      "VD_ERR macro is not supported anymore !!!"
#  define VD_BREAKPOINT()

#  define VD_DEBUGMSGNL(fmt, args...)    VD_LOG(VD_DEBUG ,"%s:(%s):" fmt "\n" , __FUNCTION__ , VD_FILE_NAME, ##args) 
#  define VD_ERRORMSGNL(fmt, args...)    VD_LOG(VD_ERROR ,"%s:(%s):" fmt "\n" , __FUNCTION__ , VD_FILE_NAME, ##args)
#  define VD_TRACENL(lvl, fmt, args...)  VD_TRACE(lvl , fmt "\n" , ##args)
#  define VD_ERRORNL(lvl, fmt, args...)  "VD_ERRORNL macro is not supported anymore !!!"
#  define VD_DBGCODE(lvl, code)
#  define VD_LOGGER_INIT(branch)         vd_logger_init(branch)
#  define VD_LOGGER_EXIT()               vd_logger_exit()

 
# else // nor VDBG nor VD_LOGGER_ON
 
// Empty declarations for release version.

#  define VD_MAIN_MODULE_INIT(max_trace, max_error)
#  define VD_SECONDARY_MODULE_INIT()

#  define VD_ENTERFUNC(x...)
#  define VD_LEAVEFUNC(x...)
#  define VD_NOT_IMPLEMENTED()
#  define VD_DEBUGMSG(fmt, args...)
#  define VD_ERRORMSG(fmt, args...)
#  define VD_ASSERT(expr)
#  define VD_TRACE(lvl, fmt, args...)
#  define VD_ERR(lvl, fmt, args...)
#  define VD_BREAKPOINT()

#  define VD_DEBUGMSGNL(fmt, args...)
#  define VD_ERRORMSGNL(fmt, args...)
#  define VD_TRACENL(lvl, fmt, args...)
#  define VD_ERRORNL(lvl, fmt, args...)
#  define VD_DBGCODE(lvl, code)

#  define VD_LOGGER_INIT(branch)         
#  define VD_LOGGER_EXIT()              

# endif //def VD_LOGGER_ON
#endif //def VDBG
	  

#endif /* #define _DEBUG_SUPPORT_H_ */
