/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

#ifndef _DEVucm_H
#define _DEVucm_H



typedef unsigned int cm_cep_uhandle_t;
typedef unsigned long cm_cookie_t;

typedef struct {
	cm_cookie_t ucm_cookie;
	void *ucm_callback;
} ucm_callback_key_t;

/*
 * Input data structures
 */
struct ucm_reject {
	cm_cep_uhandle_t ucm_reject_handle;
	cm_reject_data_t ucm_reject_data;
	cm_return_t ucm_reject_stat;
};

struct ucm_disconnect {
	cm_cep_uhandle_t ucm_disc_handle;
	int ucm_disc_senddreq;				// DREQ if != 0, DREP if 0
	union {
		cm_drequest_data_t ucm_disc_req;	// Send DREQ
		cm_dreply_data_t ucm_disc_rep;		// Send DREP
	} ucm_disc;
	cm_return_t ucm_disc_stat;
};

struct ucm_accept {
	cm_cep_uhandle_t ucm_acc_handle;
	int ucm_acc_reply;				// server if != 0, client if 0
	ucm_callback_key_t ucm_acc_key;
	union {
		cm_reply_data_t ucm_acc_reply_data;	// server side only
		cm_rtu_data_t ucm_acc_rtu_data;		// client side only
	} ucm_acc;
	cm_return_t ucm_acc_stat;
};

struct ucm_connect {
	cm_cep_uhandle_t ucm_conn_handle;
	unsigned int ucm_conn_use_ibarp;
	ucm_callback_key_t ucm_conn_key;
	cm_request_data_t ucm_conn_req;
	cm_return_t ucm_conn_stat;
};

struct ucm_connect_with_ibarp {
	cm_cep_uhandle_t ucm_conn_handle;
	ucm_callback_key_t ucm_conn_key;
	u32 ucm_ibinf_req_ip;
	u32 ucm_ibinf_src_ip;
	ibat_paths_t ucm_ibat_paths;
	cm_request_data_t ucm_conn_req;
	cm_return_t ucm_conn_stat;
};

struct ucm_listen {
	u32              ucm_listen_any;    /* 1 --> listen_any , 0 --> listen */
	cm_cep_uhandle_t ucm_listen_handle;
	ucm_callback_key_t ucm_listen_key;
	cm_listen_data_t ucm_listen_req;
	cm_return_t ucm_listen_stat;
};

struct ucm_get_ibinfo {			// for IB-ARP queries
	cm_cookie_t ucm_getinfo_cookie;
	u32 ucm_ibinf_req_ip;
	u32 ucm_ibinf_src_ip;
	ibat_paths_t ucm_ibinf_paths_count;
	ibat_arp_data_t ucm_ibinf_arp_data;
	u8 ucm_ibinf_bInvalidate;
	ibat_stat_t ucm_ibinf_stat;
};

struct ucm_get_callback {
	cm_cep_uhandle_t xc_uhandle;	// user handle
	ucm_callback_key_t xc_key;
};
	
struct ucm_handoff {
        cm_cep_uhandle_t  ucm_handoff_handle;
        cm_request_data_t ucm_handoff_conn_req;
        u64               ucm_handoff_sid;
        cm_return_t       ucm_handoff_stat;
};

struct ucm_ibrarp {			// for IB-RARP queries
        cm_cookie_t     ucm_ibrarp_cookie;
        vv_gid_t        ucm_ibrarp_req_gid;
	ibat_arp_data_t ucm_ibrarp_arp_data;
	ibat_stat_t     ucm_ibrarp_stat;
};


struct ucm_qp_h {
        u32 ucm_qph_add;   /* 1 --> add, 0 --> remove */
        long ucm_qph_handle;
        u32 ucm_qph_num;
};

struct ucm_lid_gid{
    vv_hca_h_t  hca_handle;
    int         port_num;    
    vv_lid_t    lid;
    vv_gid_t    gid;
    vv_return_t rc;
};

struct ucm_pkey{
    vv_hca_h_t  hca_handle;
    int         port_num;    
    vv_p_key_t  p_key;
    int         p_key_ix;
    vv_return_t rc;
};

struct ucm_ipoib_addr_info {
  u8     ucm_ipoib_if_name[16];    
  u32    ucm_ipoib_ip_addr;
};

/*
 * One callback_data structure is returned (read by user) per CM event
 */
typedef struct {
	int xbk_cm;			// 1 if this is a CM event
	ucm_callback_key_t xbk_key; 	// for cm events
	cm_cep_uhandle_t xbk_uhandle;	// (cm) user handle
	cm_conn_data_t xbk_conn_info;	// for cm events
	ibat_stat_t xbk_ibarp_stat;	// get_ibinfo, connect_with_ibarp
	ibat_arp_data_t xbk_ibarp_info;	// get_ibinfo, connect_with_ibarp
	cm_cookie_t xbk_ucookie;	// get_ibinfo user cookie
} ucm_callback_data;




/*
 * ioctl command codes
 */
#define ucm_CREATE_CEP	_IO(0xba, 1)			// arg is cm_cep_trans_t
#define ucm_DESTROY_CEP	_IO(0xba, 2)			// arg is cm_cep_uhandle_t
#define ucm_CANCEL	_IO(0xba, 3)			// arg is cm_cep_uhandle_t
#define ucm_CONNECT	_IOWR(0xba, 4, struct ucm_connect)
#define ucm_DISCONNECT	_IOWR(0xba, 5, struct ucm_disconnect)
#define ucm_LISTEN	_IOWR(0xba, 6, struct ucm_listen)
#define ucm_ACCEPT	_IOWR(0xba, 7, struct ucm_accept)
#define ucm_REJECT	_IOWR(0xba, 8, struct ucm_reject)
#define ucm_GET_IBINFO	_IOWR(0xba, 9, struct ucm_get_ibinfo)
#define	ucm_GET_CALLBACK	_IOWR(0xba, 10, struct ucm_get_callback)
#define	ucm_CONNECT_IBARP	_IOWR(0xba, 11, struct ucm_connect_with_ibarp)
#define	ucm_HANDOFF	        _IOWR(0xba, 12, struct ucm_handoff)
#define ucm_IBRARP          _IOWR(0xba, 13, struct ucm_ibrarp)
#define ucm_QP_HANDLE       _IOWR(0xba, 14, struct ucm_qp_h)
#define ucm_PORT2LID        _IOWR(0xba, 15, struct ucm_lid_gid)
#define ucm_PORT2GID        _IOWR(0xba, 16, struct ucm_lid_gid)
#define ucm_GET_IPOIB_ADDR  _IOWR(0xba, 17, struct ucm_ipoib_addr_info)
#define ucm_PKEY2IX         _IOWR(0xba, 18, struct ucm_pkey)

#endif // _DEVucm_H
