/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

#ifndef _DEVGSI_H
#define _DEVGSI_H

#include <linux/ioctl.h>

#include "vv-types.h"

#define GSI_HCA_NAME_LEN  32
#define GSI_RCVQLEN  8

/* calls gsi_registration ioctl */
struct gsi_registration {
	u_int8_t  gsi_class;
	u_int8_t  version;
	char      hca_name[GSI_HCA_NAME_LEN];
	u_int     port;
	u_int8_t  server;
	u_int8_t  rmpp;
};


/* calls gsi_registration ioctl */
struct gsi_copy_rmpp_buf {
	void *buf_k;
	void *buf_u;
	int size;
};


struct gsi_hca_name {
		char name[GSI_HCA_NAME_LEN];
};

struct gsi_hca_properties {
	int  num_hca_names;
	struct gsi_hca_name names[0];
};




#define gsi_REGISTER    _IOW(0xbb, 1, struct gsi_registration)
#define gsi_UNREGISTER  _IO(0xbb, 2)

/* datagram pool manipulation */
#define gsi_CREATEPOOL  _IOW(0xbb, 3, int)
#define gsi_DESTROYPOOL _IO(0xbb, 4)

/* pool reservation */
#define gsi_POOLRESERVE_DTGM _IOR(0xbb, 5, unsigned long)
#define gsi_POOLRELEASE_DTGM _IOW(0xbb, 6, unsigned long)

/* turn completion notification on/off */
#define gsi_MUXMODE_ON  _IO(0xbb, 7)
#define gsi_MUXMODE_OFF _IO(0xbb, 8)

/* RMPP buffer copy */
#define gsi_COPY_RMPP_BUF _IOR(0xbb, 9, unsigned long)


#define gsi_HCA_LIST _IOR(0xbb, 10, struct gsi_hca_properties)


#endif


