/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

// ib_cm.h
// Interface to the IB communication manager
// Copyright Voltaire LTD 2002

//TODO: 

// * decide the priority of the listenall
// * decide if ListenAll works on all the serv_id's or only on the lower ones.										   


#ifndef _IB_CM_H_
#define _IB_CM_H_


#include <linux/config.h>
#include "debug-support.h"
#include "cm-ib-types.h"
#include "sa-mads.h"	// ib_path_rec_t



// CM-specific error & status codes

typedef u32 cm_return_t;


#define cm_stat_success			0x00
#define cm_stat_error			0x01
#define cm_stat_invalid_state		0x02
#define cm_stat_insufficient_mem	0x03
#define cm_stat_not_found		0x04
#define cm_stat_invalid_event		0x05
#define cm_stat_invalid_handle		0x06
#define cm_stat_address_in_use		0x07
#define cm_stat_invalid_ctx             0x08
									  
#define cm_event_conn_timeout		0x1000
#define cm_event_sidr_req_timeout	0x1001
#define cm_event_conn_reject		0x1002
#define cm_event_conn_request		0x1003
#define cm_event_disconn_request	0x1004
#define cm_event_disconn_reply		0x1005
#define cm_event_conn_reply		0x1006
#define cm_event_connected		0x1007
#define cm_event_disconnected		0x1008
#define cm_event_sidr_request		0x1009
#define cm_event_sidr_response		0x100A
#define cm_event_disconn_timeout	cm_event_disconnected

// Private data lengths (bytes)

#define cm_REQ_priv_data_len		92
#define cm_REP_priv_data_len		204 // changed in IB 1.1 port from 204 to 196
#define cm_REJ_priv_data_len		204
#define cm_REJ_add_priv_data_len	16	
#define cm_RTU_priv_data_len		224
#define cm_MRA_priv_data_len		222
#define cm_DRQ_priv_data_len		220
#define cm_DRP_priv_data_len		224
#define cm_SIDR_REQ_priv_data_len	216
#define cm_SIDR_RESP_priv_data_len	212
#define cm_LAP_priv_data_len		168
#define cm_APR_priv_data_len		207

//union used in the CM callbacks
struct cm_conn_data_st;

// CM callback prototype
typedef void (*cm_cb_t)(void* CepHandle, struct cm_conn_data_st* ConnectionInfo, void* user_context );

// Transport type
typedef enum {
	cm_cep_transp_rc=0,			    
	cm_cep_transp_uc,				
	cm_cep_transp_rd,				
	cm_cep_transp_ud				
}cm_cep_transp_t;

typedef enum {
	cm_cep_mode_active=1,
	cm_cep_mode_passive,
} cm_cep_mode_t;

// Rejection reason code
typedef enum {
	cm_rej_code_no_qp=1,
	cm_rej_code_no_eec,
	cm_rej_code_no_res,
	cm_rej_code_timeout,
	cm_rej_code_unsupp_req,
	cm_rej_code_inv_cmd,
	cm_rej_code_inv_comm_inst,
	cm_rej_code_inv_sid,
	cm_rej_code_inv_trans_serv,
	cm_rej_code_stale_conn,
	cm_rej_code_inv_rdc,
	cm_rej_code_prim_dgid_rej,
	cm_rej_code_prim_dlid_rej,
	cm_rej_code_prim_sl,
	cm_rej_code_inv_prim_tc,
	cm_rej_code_inv_prim_hl,
	cm_rej_code_inv_prim_pr,
	cm_rej_code_alt_dgid,
	cm_rej_code_alt_dlid,
	cm_rej_code_inv_alt_sl,
	cm_rej_code_inv_alt_tc,
	cm_rej_code_inv_alt_hl,
	cm_rej_code_inv_alt_pr,
	cm_rej_code_cm_port_redir,
	cm_rej_code_port_redir,
	cm_rej_code_inv_path_mtu,
	cm_rej_code_insuff_resp_res,
	cm_rej_code_usr_rej,
	cm_rej_code_rnr_cnt_rej,
	cm_rej_code_dup_local_comm_id,    // see IB 1.1 page 584
	cm_rej_code_unsupp_class_version, // see IB 1.1 page 584
	cm_rej_code_inv_prim_fl,          // see IB 1.1 page 585
	cm_rej_code_inv_alt_fl            // see IB 1.1 page 585
} cm_rej_code_t;


// SIDR response status
typedef enum {
	cm_sidr_resp_stat_valid_qpn=0,		
	cm_sidr_resp_stat_sidr_not_supp,
	cm_sidr_resp_stat_provider_rej,
	cm_sidr_resp_stat_no_qp
} cm_sidr_resp_stat_t;


typedef struct cm_cep_addr_st {
	struct {
		u32		qpn:24;
		u32		eecn:24;
		u64		sid;
	} end_pt;
	struct {
		vv_gid_t		gid;
		vv_lid_t		lid;
	} port;
} cm_cep_addr_t;


typedef struct cm_cep_path_data_st {
	// If subn_local, the global routing info will not be used.
	boolean			    subn_local;
	ib_path_record_v2_t    	    path;

} cm_cep_path_data_t;


typedef struct cm_cep_data_st {

	u64		ca_guid;				
	u8		end_to_end_flow_ctrl;		// Ca implements end-to-end flow control ?

	//
	// Local port information
	// 
	u64		port_guid;
	u8	        local_port_num;

	//
	// Local QP information
	//
	u32		start_psn;			   
	u32		qpn;
	u32		qkey;						

	u8		retry_cnt;					
	u8		rtr_retry_cnt;				
	u8		ack_timeout;				
	u8		alt_ack_timeout;		

	u8		offered_resp_res;  
	u8		offered_initiator_depth;

	//
	// RD information
	//
	u32		loc_eecn:24;
	u32		rmt_eecn:24;
} cm_cep_data_t;

typedef struct cm_listen_data_st {
	cm_cep_addr_t		listen_addr;
        u32                     local_pid; // for CM's netstat like /proc
} cm_listen_data_t;


typedef struct cm_request_data_st {

	u64					    sid;
	cm_cep_data_t			cep_data;
	cm_cep_path_data_t		path_data;		   
	cm_cep_path_data_t		alt_path_data; 

	u8					priv_data[cm_REQ_priv_data_len];
        u32                             local_pid; // for CM's netstat like /proc
} cm_request_data_t;


typedef struct cm_reply_data_st {

	u32			qkey;
	u32			qpn:24;
	u32			eecn:24;
	u32			start_psn:24;
	u32			arb_resp_res:8;
	u32			arb_initiator_depth:8;
	u32			targ_ack_delay:5;
	u32			failover_accepted:2;
	u32			end_to_end_flow_ctrl:1;
	u32			rnr_retry_count:3;
        u64		        ca_guid; // added in IB 1.1 port
	u8			priv_data[cm_REP_priv_data_len]; 
        u32                     local_pid; // for CM's netstat like /proc
}  cm_reply_data_t;


typedef struct cm_reject_data_st {

	u32			reason:16; // cm_rej_code_t
	u32			rej_info_len:4;

	u8			rej_info[cm_REJ_add_priv_data_len]; // 16 bytes
	u8			priv_data[cm_REJ_priv_data_len]; // 204 bytes

} cm_reject_data_t;


typedef struct cm_rtu_data_st {

	u8			priv_data[cm_RTU_priv_data_len]; // 224 bytes

} cm_rtu_data_t;


typedef struct cm_mra_data_st {
	
	u16			service_timeout:5;
	u8			priv_data[cm_MRA_priv_data_len]; // 222 bytes

} cm_mra_data_t;


typedef struct cm_drequest_data_st {

	u8			priv_data[cm_DRQ_priv_data_len]; // 220 bytes

} cm_drequest_data_t;

typedef struct cm_dreply_data_st {

	u8			priv_data[cm_DRP_priv_data_len]; // 224 bytes

} cm_dreply_data_t;

#ifdef  CONFIG_PPC
#pragma pack(1)
#else
#pragma pack(push,1)
#endif

typedef struct cm_sidr_resp_data_st {


	u32		qpn:24;
	u32		status:8;	// cm_sidr_resp_stat_t
	u32		qkey;		
				
	u8		priv_data[cm_SIDR_RESP_priv_data_len]; // 212 bytes

} cm_sidr_resp_data_t;

typedef struct cm_sidr_req_data_st {

	// Remote endpoint info
	cm_cep_addr_t		rem_addr;			// Fill in the sid, gid and lid

	// Local endpoint info
	u64			port_guid;

	u32			sl:8;
	u32			rate:8;
	u32			path_bits:8;
	u32			reserved1:8;

	u8			priv_data[cm_SIDR_REQ_priv_data_len]; // 216 bytes

} cm_sidr_req_data_t;

#ifdef  CONFIG_PPC
#pragma pack(4)
#else
#pragma pack(pop)
#endif

typedef struct cm_sidr_reg_data_st {

	u64		serv_id;
	u32		qpn:24;
	u32		qkey;

} cm_sidr_reg_data_t;


typedef union conn_data_st {
	cm_request_data_t			request;
	cm_reply_data_t				reply;
	cm_rtu_data_t				rtu;	
	cm_reject_data_t			reject;
	cm_mra_data_t				mra;
	cm_drequest_data_t			disconnect_req;
	cm_dreply_data_t			disconnect_rep;
	cm_sidr_req_data_t			sidr_req;
	cm_sidr_resp_data_t			sidr_resp;

	cm_listen_data_t			listen_req;
	cm_sidr_reg_data_t			sidr_register;
} conn_data_t;

typedef struct cm_conn_data_st {
	cm_return_t				status;
	conn_data_t				data;
}cm_conn_data_t;


////////////////////////////////////////////////////////////////////
// Function prototypes
//


//
// cm_create_cep
//
//	This routine creates a connection endpoint object (CEP). This object
//	is used to establish a connection to the remote endpoint or listen for
//	incoming connections on this endpoint. This object is also used to register
//	the endpoint to resolve SIDR request
//
//
 
cm_cep_handle_t
cm_create_cep(
	IN cm_cep_transp_t	transp_type
		  );

//
// cm_destroy_cep
//
//	This routine destroys the CEP object. This routine is called when the handle
//	to the CEP object is no longer in use.
//
 
cm_return_t
cm_destroy_cep(
	IN cm_cep_handle_t				cep_handle
		  );


//
// cm_connect
//
//	This routine is used by a client to establish a connection to the remote 
//	server endpoint.
//

cm_return_t
cm_connect(
	IN cm_cep_handle_t				cep_handle,
	IN cm_request_data_t*			conn_req,		// Send REQ
	IN cm_cb_t						connect_cb,	// Recv REP, REJ
	IN void*						context
	);


//
// cm_listen
//
//	This routine is used by a server to listen for incoming clients connections 
//	on the specified endpoint.
//
 
cm_return_t
cm_listen(
	IN cm_cep_handle_t	cep_handle,
	IN cm_listen_data_t*	listen_req,
	IN cm_cb_t		listen_cb,			// Recv REQ
	IN void*		context
	);


cm_return_t
cm_listen_any(
	IN cm_cep_handle_t	cep_handle,
	IN cm_listen_data_t*	listen_req,
	IN cm_cb_t		listen_cb,			// Recv REQ
	IN void*		context,
	OUT u64*		sid
	);

//
// cm_listen_all
//
//	This routine is used by the Voltaire's nVigor SDP ONLY to listen for all incoming clients connections 
//	
 
cm_return_t
cm_listen_all(
	IN cm_cep_handle_t	cep_handle,
	IN cm_cb_t		listen_cb,			// Recv REQ
	IN void*		context
	);

//
// cm_get_free_sid
//
//	This routine returns an unused service ID for sequential listeners. 
//		
cm_return_t
cm_get_free_sid(
	OUT u64			*sid
       );

//
// cm_cancel
//
//	This routine cancels a pending connect or listen request.
//

cm_return_t
cm_cancel(
	IN cm_cep_handle_t				cep_handle
	);


//
// cm_accept
//
//	This routine is used by a client to accept the connect reply from the server
//	and set this endpoint into a connected state. This routine is used by a server to
//	accept and reply to the client's connect request.  
//
//
// Server
//
//	cep_handle      - Handle to the CEP object. This handle must be the same handle passed by the notofication callback
//					
//
//	reply_data	- Specify the reply info to send back to the remote client endpoint.
//
//
//
//
//
// Client
//
//	cep_handle	- Handle to the CEP object. This handle must be the same handle passed
//			to cm_connect().
//
//	rtu_data	    - Specify the rtu info to send back to the remote server endpoint.
//
//
cm_return_t
cm_accept(
	IN cm_cep_handle_t			cep_handle,						// CEP to be accepted.
	IN cm_reply_data_t*       	reply_data,                 // In use only for server side
	IN cm_rtu_data_t*			rtu_data,					// In use only for client side
	IN cm_cb_t					user_cb,		            // In case we send REP it will serve the endpoint for arrival of RTU ,timeout or reject
														// In case we send RTU it will serve the endpoint for arrival of REJ DREQ or timeout
	IN void*					context
	);


//
// cm_reject
//
//	This routine is used by a client to reject the connect reply from the server .
//	This routine is also used by a server to reject the incoming client's connect request.
//
cm_return_t
cm_reject(
	IN cm_cep_handle_t			cep_handle,
	IN cm_reject_data_t*		rej_data				// Send REJ
	);

cm_return_t
cm_handoff(
	IN cm_cep_handle_t     cep_handle,  // CEP created for this REQ
	IN cm_request_data_t*  conn_req,    // REQ to be handoffed
	IN u64                 handoff_sid  // SID to handoff the REQ
	);

//
// cm_disconnect
//
//	This routine is used by a client or a server to close the current connection
//	on the specified endpoint.
//
cm_return_t
cm_disconnect(
	IN cm_cep_handle_t			cep_handle,
	IN cm_drequest_data_t*		disconn_req,	// Send DREQ
	IN cm_dreply_data_t*		disconn_rep	    // Send DREP
	);



//
// Server ID resolution routines
//

cm_return_t
cm_sidr_register(
	IN cm_cep_handle_t				cep_handle,
	IN cm_sidr_reg_data_t*			sidr_reg_data,
	IN cm_cb_t						query_cb,
	IN void*						context
	);

//
// cm_sidr_deregister
//
cm_return_t
cm_sidr_deregister(
	IN cm_cep_handle_t	cep_handle
	);

//
// cm_sidr_response
//
cm_return_t
cm_sidr_response(
	IN cm_cep_handle_t			cep_handle,
	IN cm_sidr_resp_data_t*		sidr_resp		// Send SIDR_RESP
	);

//
// cm_sidr_query
//
cm_return_t
cm_sidr_query(
	IN cm_cep_handle_t			cep_handle,
	IN cm_sidr_req_data_t*		sidr_req,		// Send SIDR_REQ
	IN cm_cb_t					query_cb,		// Recv SIDR_RESP
	IN void*					context
	);

#endif
