/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */





# ifndef __VV_IB_TYPES_H__
# define __VV_IB_TYPES_H__


#include <linux/config.h>
#include <linux/types.h>

/////  OLD  ///////////
/* old defnition of the IB data types.*/
typedef unsigned short ib_lid_t; // Local ID, 16bits
typedef unsigned long long ib_gid_t; // Global ID, 64bits
typedef unsigned short ib_sid_t; // Service ID, ??
typedef unsigned int   ib_qp_t; // QP number.
/////  OLD  ///////////

////////////////////////////////////////////////////////////////////////
// BUGBUG - need to define elswere

// from Mellanox - maybe need to be define a 0
#ifndef NULL
#  define NULL ((void *)0)    
#endif

#ifndef VV_BOOL_T
#  define VV_BOOL_T  unsigned char
#endif

#ifndef true
#  define true  1
#endif

#ifndef false
#  define false 0
#endif
 
# ifndef IN
#  define IN
# endif
# ifndef OUT
#  define OUT
# endif

////////////////////////////////////////////////////////////////////////
// constants
                                    
// headers size in bytes (p. 127)
#define VV_LRH_HEADER_SIZE      (8)
#define VV_GRH_HEADER_SIZE      (40)
#define VV_BTH_HEADER_SIZE      (12)
#define VV_RDETH_HEADER_SIZE    (4)
#define VV_DETH_HEADER_SIZE     (8)
#define VV_RETH_HEADER_SIZE     (16)
#define VV_ATOM_ETH_HEADER_SIZE (28)
#define VV_AETH_HEADER_SIZE     (4)
#define VV_ATOM_ACK_HEADER_SIZE (8) 
#define VV_IMM_DT_HEADER_SIZE   (4)
#define VV_MAX_PAYLOAD_SIZE     (4096)
#define VV_ICRC_SIZE            (4)
#define VV_VCRC_SIZE            (2)
 
////////////////////////////////////////////////////////////////////////
// parameters defenitions

typedef u_int64_t vv_eui64_t;   // EUI-64 24- company_id, 40- extention identifier

#ifdef  CONFIG_PPC
#pragma pack(1)
#else
#pragma pack(push,1)
#endif

// vv GID is in host order
typedef union {
	u_int8_t	gid[16];
#if CPU_LE
	union {
		struct {
			vv_eui64_t		eui64;
			struct {
				u_int64_t   reserved: 54;
				u_int64_t   identify: 10;
			} s;
		} ll; // link local

		struct {
			vv_eui64_t		eui64;
			struct {
				u_int64_t subnet:   16;
				u_int64_t reserved: 38;
				u_int64_t identify: 10;
			} s;
		} sl; // site local
		
		struct {
			vv_eui64_t eui64;
			u_int64_t  subnet;
		} g; // global
		
		struct {
			u_int8_t  multicast[14];
			struct {
				u_int64_t scope:    4;
				u_int64_t flag:     4;
				u_int64_t identify: 8;
			} s;
		} m; // multicast
	} scope;
#else //CPU_BE
	union {
		struct {
            struct {
                u_int64_t   identify: 10;
                u_int64_t   reserved: 54;
			} s;
            vv_eui64_t		eui64;
		} ll; // link local

		struct {
            struct {
				u_int64_t identify: 10;
                u_int64_t reserved: 38;
				u_int64_t subnet:   16;
			} s;
            vv_eui64_t		eui64;
		} sl; // site local
		
		struct {
            u_int64_t  subnet;
            vv_eui64_t eui64;
		} g; // global
		
		struct {
            struct {
				u_int64_t identify: 8;
                u_int64_t flag:     4;
				u_int64_t scope:    4;
			} s;
            u_int8_t  multicast[14];
		} m; // multicast
	} scope;
#endif
} vv_gid_t;

#ifdef  CONFIG_PPC
#pragma pack(4)
#else
#pragma pack(pop)
#endif

// p. 117 Default GID prefix
#define VV_DEFAULT_GID_PREFIX	(0xFE80000000000000)

#define VV_PRINT_GID_ARRAY(gid)\
                            ({                                                     \
                             u_int32_t i_for = 0;                                  \
                             for(i_for=0; i_for<=15; i_for++)                      \
                             {                                                     \
                                 printk("%d: %d\n",i_for,(u_int32_t)(gid[i_for])); \
                             }                                                     \
                            })


typedef unsigned short      vv_lid_t;      // Local ID, 16 bits

// p. 123
#define VV_PERMIS_LID (0xFFFF)               // Permissive DLID
#define VV_MIN_UC_LID (0x0001)               // Start Unicast LID
#define VV_MAX_UC_LID (0xBFFF)               // End Unicast LID
#define VV_MIN_MC_LID (0xC000)               // Start Multicast LID
#define VV_MAX_MC_LID (0xFFFE)               // End Multicast LID

typedef u_int64_t           vv_guid_t;  // GUID

typedef unsigned short      vv_sid_t;      // Service ID
typedef unsigned int        vv_qp_t;       // QP number
typedef u_int8_t            vv_port_num_t; // HCA port number

typedef u_int32_t           vv_l_key_t;    // 
typedef u_int32_t           vv_r_key_t;    // RDMA and Atomic

typedef u_int64_t           vv_phy_addr_t; // The same as in mlx, need to see what about virtual addres we use void*
typedef u_int64_t           vv_iova_t;     // IO Virtual Address for physical memory registrations

typedef u_int16_t           vv_p_key_t;    // p. 455 high order bit of the p_key is the type of the membership (limited, full)
#define VV_DEFULT_P_KEY     (0xFFFF)       // p. 455

typedef u_int32_t           vv_q_key_t;    // Construct used in Datagram Service QPs to validate a remote sender's
                                           // right to access a local Receive Queue. In Modify QP and WR.
                                           // Not managed below the verb.
#define VV_GSI_Q_KEY        (0x80010000)   // from IBM (ib-defs.h)

typedef void*               vv_hca_context_t; // BUGBUG - application hca context is NOT supported yet by the adaptor
typedef void*               vv_cq_context_t;
typedef void*               vv_qp_context_t;
typedef void*               vv_eec_context_t;


# endif // __VV_IB_TYPES_H__

