/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

#ifndef _LIBucm_H
#define _LIBucm_H

#ifndef __KERNEL__
#define _LINUX_TYPES_H
#define _LINUX_IF_H

#define u8 ___u8
#define u16 ___u16
#define u32 ___u32
#define u64 ___u64

#define ib_lid_t ___ib_lid_t
#define ib_gid_t ___ib_gid_t

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef unsigned long long u64;

#include <sys/types.h>
#include <net/if.h>
#include <linux/ioctl.h>

#if __BYTE_ORDER == LITTLE_ENDIAN
#define cpu_to_be64(x) __swab64(x)
#else
#define cpu_to_be64(x) (x)
#endif

#ifdef TRUE
#undef TRUE
#undef FALSE
#endif

#endif

#include <cm/cm-ib-types.h>
#include <cm/ib-cm.h>
#include <ibat.h>

#include <devucm.h>

#ifndef __KERNEL__

#undef u8
#undef u16
#undef u32
#undef u64

#undef ib_lid_t
#undef ib_gid_t

#endif

int ucm_cm_init(int sync);
int ucm_cm_fini(void);
void *ucm_cm_process(void *junk);

// user mode CM callback prototype
typedef void (*cm_u_cb_t)(cm_cep_uhandle_t cep_handle, cm_conn_data_t *conn_info, ibat_arp_data_t *arp_data, void *user_context);

cm_cep_uhandle_t
ucm_cm_create_cep(IN cm_cep_transp_t transp_type);

cm_return_t
ucm_cm_destroy_cep(IN cm_cep_uhandle_t cep_handle);

cm_return_t
ucm_cm_connect(IN cm_cep_uhandle_t cep_handle, IN cm_request_data_t *conn_req, IN cm_u_cb_t connect_cb, IN void *context, OUT cm_conn_data_t *ret);

cm_return_t
ucm_cm_connect_with_ibarp(IN cm_cep_uhandle_t cep_handle, IN __u32 req_ip, IN __u32 src_ip, IN cm_request_data_t *conn_req, IN cm_u_cb_t connect_cb, IN void *context, OUT cm_conn_data_t *ret);

cm_return_t
ucm_cm_listen(IN cm_cep_uhandle_t cep_handle, IN cm_listen_data_t *listen_req, IN cm_u_cb_t listen_cb, IN void *context, OUT cm_conn_data_t *ret);

cm_return_t
ucm_cm_listen_any(IN cm_cep_uhandle_t cep_handle, IN cm_listen_data_t *listen_req, IN cm_u_cb_t listen_cb, IN void *context, OUT __u64 *sid, OUT cm_conn_data_t *ret);

cm_return_t
ucm_cm_cancel(IN cm_cep_uhandle_t cep_handle);

cm_return_t
ucm_cm_accept(IN cm_cep_uhandle_t cep_handle, IN cm_reply_data_t *reply_data, IN cm_rtu_data_t *rtu_data, IN cm_u_cb_t user_cb, IN void *context, OUT cm_conn_data_t *ret);

cm_return_t
ucm_cm_reject(IN cm_cep_uhandle_t cep_handle, IN cm_reject_data_t *rej_data);

cm_return_t
ucm_cm_disconnect(IN cm_cep_uhandle_t cep_handle, IN cm_drequest_data_t *disconn_req, IN cm_dreply_data_t *disconn_rep);

typedef void (*ucm_ibdata_reply_fn_t)(IN ibat_stat_t status, IN ibat_arp_data_t *arp_data, IN void *context);

ibat_stat_t
ucm_ibat_get_ib_data(IN __u32 req_ip, IN __u32 src_ip, IN ibat_paths_t path_records_count, IN OUT ibat_arp_data_t *arp_data, IN ucm_ibdata_reply_fn_t rep_cb, IN void *context, IN __u8 bInvalidate);

int ucm_get_callback(cm_cep_uhandle_t cep_handle, ucm_callback_key_t *kp);

cm_return_t
ucm_cm_handoff(IN cm_cep_uhandle_t cep_handle, IN cm_request_data_t *conn_req, IN __u64  handoff_sid);

ibat_stat_t
ucm_ats_ibrarp(IN vv_gid_t *dgid, IN ucm_ibdata_reply_fn_t rep_cb, IN void *context, 
	       OUT ibat_arp_data_t *arp_data);

int
ucm_vverb_add_qp_handle(IN long qp_handle, IN __u32 qp_num);

int
ucm_vverb_rem_qp_handle(IN long qp_handle, IN __u32 qp_num);

vv_return_t 
ucm_port_num2base_lid(IN vv_hca_h_t hca, IN int port_num, OUT vv_lid_t  *lid);

vv_return_t 
ucm_port_num2base_gid(IN vv_hca_h_t hca, IN int port_num, OUT vv_gid_t  *gid);

vv_return_t 
ucm_pkey2pkey_index(IN vv_hca_h_t hca, IN int port_num, IN vv_p_key_t pkey, OUT int *pkey_idx);


int 
ucm_get_ipoib_addr(IN __u8 ipoib_if_name[], OUT __u32 *ipoib_ip_addr); 

#endif // _LIBucm_H
