/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

#ifndef __VD_LOGGER_H__
#define __VD_LOGGER_H__

#include <linux/config.h>
#include "repository.h"

#ifdef CONFIG_KLR
#ifdef CONFIG_BARAK4
#include <vtr/generic.h>
#else
#include <galileo/generic.h>
#endif
#endif

#ifndef VD_MODULE_NAME
#error  LOGGER: VD_MODULE_NAME is not defined!!!
#endif

#define LOGTOKENTOSTR(X) #X
#define LOGTOKENCAT(A,B) A ## B

#define LOG_LEVEL_TOKEN  _trace_level
#define LOG_HANDLE_TOKEN _log_rep_h

#define VDLOG_PANIC         0
#define VDLOG_ERROR         1
#define VDLOG_WARNING       2
#define VDLOG_INFO          3
#define VDLOG_DEBUG         4
#define VDLOG_FUNC          5

static char *vdlog_levels[] __attribute__ ((unused)) = {
    "PANIC  ",
    "ERROR  ",
    "WARNING",
    "INFO   ",
    "DEBUG  ",
    "FUNC   "
};

/* 
#define VDLOG_STR_0         "PANIC"
#define VDLOG_STR_1         "ERROR"
#define VDLOG_STR_2         "WARNING"
#define VDLOG_STR_3         "INFO"
#define VDLOG_STR_4         "DEBUG"
#define VDLOG_STR_5         "FUNC"
*/

#define BUILD_VLOGGER_NAME(mod_name,  prefix) LOGTOKENCAT(mod_name , prefix)
#define VLOGGER_TRACE_LEVEL_NAME BUILD_VLOGGER_NAME(VD_MODULE_NAME,  LOG_LEVEL_TOKEN)
#define VLOGGER_TRACE_LEVEL_H    BUILD_VLOGGER_NAME(VD_MODULE_NAME,  LOG_HANDLE_TOKEN)
#define INIT_VLOGGER_TRACE_LEVEL int VLOGGER_TRACE_LEVEL_NAME = VDLOG_PANIC; rep_notifier_t VLOGGER_TRACE_LEVEL_H; \
		EXPORT_SYMBOL_NOVERS(VLOGGER_TRACE_LEVEL_NAME);   EXPORT_SYMBOL_NOVERS(VLOGGER_TRACE_LEVEL_H);

#ifdef CONFIG_KLR
#define MAX_LOGGER_MSG	1024
extern unsigned int KernelLogCounter;
#endif

extern int logger_trace_level;                                      
extern int VLOGGER_TRACE_LEVEL_NAME;
extern rep_notifier_t VLOGGER_TRACE_LEVEL_H;

extern void trace_lvl_change_cb(rep_key_t key, long change, void *context);

#ifdef __KERNEL__

// Tzahi-BUGBUG KLR counter is not being advanved atomicaly
#ifdef CONFIG_KLR
	#define VD_LOG_PRINT(__mod_name, __level_tok, __level, __fmt , __args...) \
	do \
	{ \
		if( ((__level) <= logger_trace_level) && ((__level) <= LOGTOKENCAT(__mod_name, __level_tok)) ) \
		{ \
			char *temp_buff; \
			printk(KERN_INFO "%s : " LOGTOKENTOSTR( __mod_name ) " : " __fmt , vdlog_levels[(__level)] , ##__args);  \
			temp_buff = kmalloc(MAX_LOGGER_MSG, GFP_ATOMIC); \
			sprintf(temp_buff, "%d: J=%ld: " KERN_INFO "%s : " LOGTOKENTOSTR( __mod_name ) " : " __fmt , KernelLogCounter++, jiffies, vdlog_levels[(__level)] , ##__args); \
			klr_write_log(temp_buff); \
			kfree(temp_buff); \
		} \
	}while(0)
#else
	#define VD_LOG_PRINT(__mod_name, __level_tok, __level, __fmt , __args...) \
	do 	{      \
		if( ((__level) <= logger_trace_level) && ((__level) <= LOGTOKENCAT(__mod_name, __level_tok)) ) \
			printk(KERN_INFO "%s : " LOGTOKENTOSTR( __mod_name ) " : " __fmt , vdlog_levels[(__level)] , ##__args);  \
	}while(0)
#endif

#define VD_LOG(__level, fmt, args...) \
do { \
    VD_LOG_PRINT(VD_MODULE_NAME, LOG_LEVEL_TOKEN, __level, fmt , ##args); \
}while(0)

/*
#define VD_LOG_PRINT(__mod_name, __level_tok, __level, level_str ,__fmt , __args...) \
do {      \
    if( ((__level) <= logger_trace_level) && ((__level) <= LOGTOKENCAT(__mod_name, __level_tok)) ) \
        printk("%s : " LOGTOKENTOSTR( __mod_name ) " : " __fmt , LOGTOKENTOSTR(level_str) , ##__args);  \
}while(0)


#define VD_LOG(__level, fmt, args...) \
do { \
    VD_LOG_PRINT(VD_MODULE_NAME, LOG_LEVEL_TOKEN, __level, LOGTOKENCAT(VDLOG_STR_, __level) , fmt , ##args); \
}while(0)
*/

/////////////////////////////////////////////////////////////////////////////////////////
//<key> is access key to the repository, under which <log_trace_level> is defined
/////////////////////////////////////////////////////////////////////////////////////////
static inline void vd_logger_init(char *key)
{
    rep_key_t               root_key;
    long                    trace_level = 0;
    long                    len;

    if(rep_open_key (rep_get_root(), key, 0, &root_key)) {
        printk("VD_LOGGER: rep_open_key failed for, %s -- disabling logging\n", key);
        goto out;
    }


    len = sizeof(long);
    
    if (rep_get_value(root_key, "debug-level", NULL, (char *)&trace_level, &len)) {
        printk("VD_LOGGER: rep_get_value failed for, %s -- disabling logging\n", key);
        goto out_close;
    }

    printk("LOGGER: Added key %s, trace level %d\n", key, (int)trace_level);

    rep_notify_key_change(root_key, 0, REP_NOTIFY_CHANGE_VALUE, trace_lvl_change_cb, &(VLOGGER_TRACE_LEVEL_NAME), &(VLOGGER_TRACE_LEVEL_H));

out_close:
    rep_close_key(root_key);

out:
    VLOGGER_TRACE_LEVEL_NAME = (int)trace_level;

}

static inline void vd_logger_exit(void)
{
    rep_notify_del(VLOGGER_TRACE_LEVEL_H);
}


#else//__KERNEL__

#error  LOGGER is compiled outside of __KERNEL__

#endif//__KERNEL__

#endif//__VD_LOGGER_H__

