/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

#ifndef  __MAD_H__
#define  __MAD_H__

#ifdef __KERNEL__
#include <asm/byteorder.h>
#else
#include <netinet/in.h>
#include <endian.h>
#endif

#include <linux/types.h>

//
// Defines 
//
#define MAD_BLOCK_SIZE                                256

#define MAD_IB_BASE_VERSION                1 
#define MAD_IB_CLASS_VERSION               1

// Management Class Values

#define MAD_CLASS_SM_LID_ROUTED            0x01
#define MAD_CLASS_SM_DIRECTED_ROUTE        0x81

#define MAD_CLASS_RESERVED_02              0x02

#define MAD_CLASS_SUBN_ADM                 0x03        // Subnet Administration class 
                                                       // All Nodes hosting a 
                                                       // Subnet Manager
#define MAD_CLASS_PERF                     0x04        // Performance Management class 
#define MAD_CLASS_BM                       0x05        // Baseboard Management class 
#define MAD_CLASS_DEV_MGT                  0x06        // Device Management class 
#define MAD_CLASS_COMM_MGT                 0x07        // Communications Management class
#define MAD_CLASS_SNMP                     0x08        // SNMP Tunneling class (tunneling
                                                       // of the SNMP protocol through 
                                                       // the IBA fabric)
//
// Vendor Defined classes 0x09-0x0F
//
#define MAD_CLASS_CON_MGT                  0x09        // Connection Management class
#define MAD_CLASS_DEV_CONF_MGT             0x10        // Device Configuration Management
                                                       // class 


//
// MANAGEMENT CLASS METHODS
// Common Management Methods (including R bit) Description

// 0x00, 0x80 Reserved.

#define MAD_MTHD_GET                        0x01        // Request (read) an attribute from a node
#define MAD_MTHD_SET                        0x02        // Request a set (write) of an attribute 
                                                        // in a node 
#define MAD_MTHD_SEND                       0x03        // Send a datagram. Does not require 
                                                        // a response.
#define MAD_MTHD_RESERVED_04                0x04        // Reserved.
#define MAD_MTHD_TRAP                       0x05        // Unsolicited datagram sent from a node 
                                                        // indicating an event occured that may 
                                                        // be of interest. 
                                                        // A trap method shall always use the 
                                                        // Notice attribute.
#define MAD_MTHD_REPORT                     0x06        // Forwarding an event/trap/notice to 
                                                        // interested party.
#define MAD_MTHD_TRAP_REPRESS               0x07        // Instruct a Trap sender to stop sending
                                                        // a repeated Trap.
// 0x08-0x0F - Reserved
// 0x10-0x7F - Class-specific methods defined by Class

#define MAD_MTHD_GET_RESP                0x81        // The response from an attribute Get or 
                                                                        // Set request.
#define MAD_MTHD_RESERVED_82        0x82        // Reserved.
#define MAD_MTHD_RESERVED_83        0x83        // Reserved.
#define MAD_MTHD_RESERVED_84        0x84        // Reserved.
#define MAD_MTHD_RESERVED_85        0x85        // Reserved.

#define MAD_MTHD_REPORT_RESP        0x86        // Response to a Report(). 


// 0x87-0x8F - Reserved.
// 0x90-0xFF - Class-dependent methods. Usage is defined by the class. 






//
// STATUS FIELD
//
// MAD Status Field Bit Values
//
#define MAD_STATUS_MASK											0x00ff
#define MAD_STATUS_SUCCESS                                      0x0000
#define MAD_STATUS_BUSY                                         0x0001        
#define MAD_STATUS_REDIRECT_REQD                                0x0002        

//
// bits 2-4 Code for invalid field
//        0 -no invalid fields
//        1 -The class version specified is not supported.
//        2 -The method specified is not supported
//        3 -The method/attribute combination is not supported
//        4-6:Reserved
//        7 -One or more fields in the attribute contain an invalid value.
//
#define MAD_STATUS_UNSUPPORTED_CLASS_VER                0x0004        
#define MAD_STATUS_UNSUPPORTED_METHOD                   0x0008        
#define MAD_STATUS_UNSUPPORTED_METHOD_ATTRIB            0x000c
// 0x0010-0x0018 - Reserved
#define MAD_STATUS_INVALID_ATTRIB                       0x001c
        
// 0x0020-0x00ff - Reserved

// 0x100-0xff00 class-dependent values.                        


typedef union
{
        u_int16_t        raw16;

        struct
        {
#if CPU_BE
            u_int16_t busy      :     1;      // Temporarily busy.MAD discarded.
                                           // This is not an error.
            u_int16_t redir_rqrd:     1;      // Redirection. 
                                           // This is not an error.
            u_int16_t inval_fld :     3;
            u_int16_t reserved:       3;

            u_int16_t class_specific: 8;      // Class dependent values

#else //CPU_LE
            u_int16_t class_specific: 8;      // Class dependent values
            u_int16_t reserved:       3;
            u_int16_t inval_fld :     3;
            u_int16_t redir_rqrd:     1;      // Redirection. 
                                           // This is not an error.
            u_int16_t busy      :     1;      // Temporarily busy.MAD discarded.

#endif
        } s;
        
}  __attribute__((packed)) mad_status_t;




//
// MANAGEMENT CLASS ATTRIBUTES
// 
#define        MAD_CLASS_ATTRIB_ID_RESERVED_0                 0x0000        // Reserved
#define        MAD_CLASS_ATTRIB_ID_CLASS_PORT_INFO            0x0001
#define        MAD_CLASS_ATTRIB_ID_NOTICE                     0x0002 
#define        MAD_CLASS_ATTRIB_ID_INFORM_INFO                0x0003 

// 0x0004-0x000F - Reserved
// 0x0010-0x7FFF - Class-dependant values.


typedef struct mad_hdr_st
{
        u_int8_t base_ver;            // Version of management datagram base 
                                      // format.Value is set to 1
        u_int8_t mgt_class;           // Class of operation. 
        u_int8_t class_ver;           // Version of MAD class-specific format.
                                      // Value is set to 1 except 
                                      // for the Vendor class
        union
        {
                u_int8_t        raw8;

                struct {
#if CPU_BE
                    u_int8_t   r               :1;  // Request/response field, 
                                                    // conformant to base class definition
                    u_int8_t   method          :7;  // Method to perform based on 
                                                    // conformant to base class definition
#else //CPU_LE
                    u_int8_t   method          :7;  // Method to perform based on 
                                                    // the management class
                    u_int8_t   r               :1;  // Request/response field, 
                                                    // conformant to base class definition
#endif
                } ms;
        } m;

        union
        {
                //
                // All MADs use this structure
                struct
                {
                        mad_status_t status;            // Code indicating status of method
                        u_int16_t       reserved;       // Reserved. Shall be set to 0
                } ns;                                   // Normal MAD ?

                //
                // This structure is used only for Directed Route SMP's
                struct
                {
                        struct
                        {
#if CPU_BE
                                u_int16_t  d             :1;    // Direction bit to determine 
                                                                // direction of packet
                                u_int16_t  status        :15;   // Code indicating status of method
#else //CPU_LE
                                u_int16_t  status        :15;   // Code indicating status of method
                                u_int16_t  d             :1;    // Direction bit to determine 
                                                                // direction of packet
#endif
                        } s;                                                
                        u_int8_t   hop_ptr;  // used to indicate the current byte 
                                          // of the Initial/Return Path field.
                        u_int8_t   hop_cnt;  // used to contain the number of valid 
                                          // bytes in the Initial/Return Path

               } dr_status; // Directed Route only ?
        } u;

        u_int64_t        transact_id;                    // Transaction specific identifier
        u_int16_t        attrib_id;                      // 
        u_int16_t        reserved;                       // Reserved. Shall be set to 0
        
        u_int32_t        attrib_modifier;                // Provides further scope to 
                                                      // the Attributes, usage 
                                                      // determined by the management class
} __attribute__((packed)) mad_hdr_t; 




//
// MAD Base Fields
//
typedef struct 
{
        mad_hdr_t hdr;

        //
        // usage defined within the scope of the management class
        u_int8_t    data[MAD_BLOCK_SIZE - sizeof(mad_hdr_t)];                
}  __attribute__((packed)) mad_t;

#define MAD_RMPP_NO_RESP_TIME                    0x1F
#define MAD_RMPP_FLAG_ACTIVE                     0x01
#define MAD_RMPP_FLAG_FIRST                      0x02
#define MAD_RMPP_FLAG_LAST                       0x04

typedef struct 
{
        mad_hdr_t           hdr;
 
        u_int8_t             rmpp_version;
        u_int8_t             rmpp_type;
        u_int8_t             rmpp_flags;
        u_int8_t             rmpp_status;
 
        u_int32_t            seg_num;
        u_int32_t            paylen_newwin;
 
}  __attribute__((packed)) mad_rmpp_hdr_t;


#define MAD_SET_VERSION_INFO(mad, base_version, mgmt_class, class_version) {\
        (mad)->hdr.base_ver = base_version; \
        (mad)->hdr.mgt_class = mgmt_class; \
        (mad)->hdr.class_ver = class_version; \
}

#define MAD_GET_VERSION_INFO(mad, base_version, mgmt_class, class_version) {\
        (*base_version) = (mad)->hdr.base_ver; \
        (*mgmt_class) = (mad)->hdr.mgt_class; \
        (*class_version) = (mad)->hdr.class_ver; \
}

#define MAD_GET_STATUS(mad, stat) {\
        (*stat) = (mad)->hdr.u.ns.status.raw16; \
}

#define MAD_SET_STATUS(mad, stat) {\
        (mad)->hdr.u.ns.status.raw16 = stat; \
}


#define MAD_SET_TRANSACTION_ID(mad, id) {\
        (mad)->hdr.transact_id= id; \
}

#define MAD_GET_TRANSACTION_ID(mad, id) {\
        (*id) = (mad)->hdr.transact_id; \
}

#define MAD_SET_ATTRIB_ID(mad, id) {\
        (mad)->hdr.attrib_id = id; \
}

#define MAD_GET_ATTRIB_ID(mad, id) {\
        (*id) = (mad)->hdr.attrib_id; \
}
        
#define MAD_SET_ATTRIB_MOD(mad, amod) {\
        (mad)->hdr.attrib_modifier = amod; \
}

#define MAD_GET_ATTRIB_MOD(mad, amod) {\
        (*amod) = (mad)->hdr.attrib_modifier; \
}
        
#define MAD_SET_METHOD_TYPE(mad, method_type) {\
        (mad)->hdr.m.ms.method = method_type; \
}

#define MAD_GET_METHOD_TYPE(mad, method_type) {\
        (*method_type) = (mad)->hdr.m.ms.method; \
}


//
#ifndef ntoh64
#  ifdef __KERNEL__
#    define ntoh64(__a) __cpu_to_be64(__a)
#  else /* __KERNEL__ */
#    if __BYTE_ORDER == __LITTLE_ENDIAN
#      define ntoh64(__a) ({u_int64_t b;                                  \
                        *(((u_int32_t *)&b)+1)=ntohl(*(u_int32_t *)&__a); \
                        *(u_int32_t *)&b=ntohl(*(((u_int32_t *)&__a)+1)); \
                      b;})
#    else
#      define ntoh64(__a) ((u_int64_t)__a)
#    endif
#  endif /* else __KERNEL__ */
#endif /*ifndef ntoh64*/

#ifndef hton64
#define hton64(__a) ntoh64(__a)
#endif


static inline void mad_swap_header( mad_t *mad )
{
// commented out, not needed.
//        mad->hdr.u.ns.status.raw16 = ntohs(mad->hdr.u.ns.status.raw16);
        mad->hdr.transact_id = ntoh64(mad->hdr.transact_id);
        mad->hdr.attrib_id = ntohs(mad->hdr.attrib_id);
        mad->hdr.attrib_modifier = ntohl(mad->hdr.attrib_modifier);
}
#endif //__MAD_H__
