/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */


#ifndef _SA_MADS_H_
#define _SA_MADS_H_

#include <linux/config.h>
#include "vv-types.h"
#include "mad.h"
    
/* SA methods */
#define SUBN_ADM_GET                    0x01
#define SUBN_ADM_GET_RESP               0x81
#define SUBN_ADM_SET                    0x02
#define SUBN_ADM_DELETE	                0x15
#define SUBN_ADM_DELETE_RESP            0x95
#define SUBN_ADM_INFORM                 0x10
#define SUBN_ADM_INFORM_RESP            0x90
#define SUBN_ADM_REPORT                 0x06
#define SUBN_ADM_REPORT_RESP            0x96
#define SUBN_ADM_GETTABLE               0x12
#define SUBN_ADM_GETTABLE_RESP          0x92
#define SUBN_ADM_GETTRACETABLE          0x13
#define SUBN_ADM_GETMULTI               0x14
#define SUBN_ADM_GETMULTI_RESP          0x94
#define SUBN_ADM_CONFIG                 0x15
#define SUBN_ADM_CONFIG_RESP            0x95

/* SA attribute IDs */
#define SA_CLASS_PORT_INFO              0x0001
#define SA_NOTICE                       0x0002
#define SA_INFORM_INFO                  0x0003
#define SA_NODE_RECORD                  0x0011
#define SA_PORTINFO_RECORD              0x0012
#define SA_SLTOVL_MAPTBL_RECORD         0x0013
#define SA_SWITCH_RECORD                0x0014
#define SA_LINEAR_FWDTBL_RECORD         0x0015
#define SA_RANDOM_FWDTBL_RECORD         0x0016
#define SA_MCAST_FWDTBL_RECORD          0x0017
#define SA_SMINFO_RECORD                0x0018
#define SA_INFORM_RECORD                0x00F3
#define SA_NOTICE_RECORD                0x00F4
#define SA_LINK_RECORD                  0x0020
#define SA_GUIDINFO_RECORD              0x0030
#define SA_SERVICE_RECORD               0x0031
#define SA_PARTITION_RECORD             0x0033
#define SA_RANGE_RECORD                 0x0034
#define SA_PATH_RECORD                  0x0035
#define SA_MULTI_PATH_RECORD            0x003A
#define SA_VL_ARBTBL_RECORD             0x0036
#define SA_MCGROUP_RECORD               0x0037
#define SA_MCMEMBER_RECORD              0x0038
#define SA_RESPONSE                     0x8001


/* SA status values*/
#define SA_STATUS_NO_ERROR 0		// No class-specific error
#define SA_STATUS_ERR_NO_RESOURCES 1	// Insufficient resources to complete the
					// request. This error may be returned
					// for any SA request which could not be
					// processed due to insufficient resources.
#define SA_STATUS_ERR_REQ_INVALID 2	// Supplied request or update is invalid.
#define SA_STATUS_ERR_NO_RECORDS 3	// No records match query. May be returned
					// only by a SubnAdmGetResp(). Never returned
					// for an RMPP response because an RMPP
					// transaction with a payload length of zero
					// is a valid transaction.
#define SA_STATUS_ERR_TOO_MANY_RECORDS 4	// Too many records match query.
#define SA_STATUS_ERR_REQ_INVALID_GID 5		// Invalid GID in SA request.
#define SA_STATUS_ERR_REQ_INSUFFICIENT_COMPONENTS 6	// Request did not contain the
							// required components.


/* Trap numbers */
#define SA_TRAP_NUM_GID_IN          64
#define SA_TRAP_NUM_GID_OUT         65
#define SA_TRAP_NUM_MC_CREATE       66
#define SA_TRAP_NUM_MC_DELETE       67

#ifdef  CONFIG_PPC
#pragma pack(1)
#else
#pragma pack(push,1)
#endif

// IB Record Identifier
typedef union {
    u32 val;
    struct {
        vv_lid_t    lid;
        u8          port_num;
        u8          enumeration;
    } bf;   // bit field value
}  ib_rid_t;

// PathRecord (note: class version 1)
typedef struct {
    ib_rid_t    path_rid;           // Record Identifier
    u32         reserved0;          // 32-bit padding to make GIDs starts from
                                    //      64-bit aligned address
    vv_gid_t    dgid;               // Destination GID
    vv_gid_t    sgid;               // Source GID
    vv_lid_t    dlid;               // Destination lid
    vv_lid_t    slid;               // Source lid
    

#if CPU_BE
    u32     raw_traffic :1;     // for IB Packet = 0 (P_Key must be valie)
    u32     reserved3   :3;
    u32     flow_label  :20;    // used in GRH      
    u32     hop_limut   :8;     // Hop limit used in GRH
#else // CPU_LE
    u32     hop_limut   :8;     // Hop limit used in GRH
    u32     flow_label  :20;    // used in GRH      
    u32     reserved3   :3;
    u32     raw_traffic :1;     // for IB Packet = 0 (P_Key must be valie)
                                    // for Raw Packet= 0 (No P_Key)
#endif



    u8      traffic_class;              // Traffic Class used in GRH

#if CPU_BE
    u8      reserved1   :1;
    u8      numb_path   :7;     // Max number of paths to (be) return(ed)
#else // CPU_LE
    u8      numb_path   :7;     // Max number of paths to (be) return(ed)
    u8      reserved1   :1;
#endif


    u16     pkey;               // Partition Key for this path
    u16     sl_mask;                // Bit Mask values for Service Levels
                                    //  i.e.,   bit 15 for SL15, ...,
                                    //          bit0 for SL0

#if CPU_BE
    
    u8      mtu_selector    :2; 
    u8      mtu         :6;     
    u8      rate_selector :2;   
    u8      rate        :6;     
    u8      pkt_life_time_selector:2;
    u8      pkt_life_time   :6;     

#else // CPU_LE
    u8      mtu         :6;     // Enumeration of MTU required
                                    // 1:       256 bytes
                                    // 2:       512 bytes
                                    // 3:       1024 bytes
                                    // 4:       2048 bytes
                                    // 5:       4096 bytes
                                    // 6-63:    reserved
    u8      mtu_selector    :2; // 0: greater than or equal to MTU specified
                                // 1: less than or equal to MTU specified
                                // 2: exactly the MTU specified

    u8      rate        :6;     // data rate for the path
                                    // 1:       1   GB/sec
                                    // 2:       2.5 GB/sec
                                    // 3:       10  GB/sec
                                    // 4:       30  GB/sec
                                    // 5-64:    reserved
    u8      rate_selector :2;   // 0: greater than or equal to Rate specified
                                // 1: less than or equal to Rate specified
                                // 2: exactly the Rate specified

    u8      pkt_life_time   :6;     // *****************************************
                                    // *** User be aware that the CM LifeTime &
                                    // *** TimeOut values are only 5-bit wide.
                                    // *****************************************
                                    //
                                    // Accumulated packet life time for
                                    // the path specified by an enumeration 
                                    // deried from
                                    //   4.096 usec * 2^PktLifeTime
    u8      pkt_life_time_selector:2;
                                // 0: greater than or equal to PacketLifeTime specified
                                // 1: less than or equal to PacketLifeTime specified
                                // 2: exactly the PacketLifeTime specified

#endif



    u8      reserved2 [7];  // alignment padding to make the next path record
                                // starts from 64-bit alignment address
}  ib_path_record_t;


// PathRecord (note: class version 2)
typedef struct {
    u32       reserved0;            // 64-bit padding to make GIDs starts from
    u32       reserved1;            // 64-bit aligned address.
                                    //      
    vv_gid_t    dgid;               // Destination GID
    vv_gid_t    sgid;               // Source GID
    vv_lid_t    dlid;               // Destination lid
    vv_lid_t    slid;               // Source lid
    

#if CPU_BE
    u32     raw_traffic :1;     // for IB Packet = 0 (P_Key must be valie)
    u32     reserved3   :3;
    u32     flow_label  :20;    // used in GRH      
    u32     hop_limut   :8;     // Hop limit used in GRH
#else // CPU_LE
    u32     hop_limut   :8;     // Hop limit used in GRH
    u32     flow_label  :20;    // used in GRH      
    u32     reserved3   :3;
    u32     raw_traffic :1;     // for IB Packet = 0 (P_Key must be valie)
                                    // for Raw Packet= 0 (No P_Key)
#endif



    u8      traffic_class;              // Traffic Class used in GRH

#if CPU_BE
    u8      reversible  :1;
    u8      numb_path   :7;     // Max number of paths to (be) return(ed)
#else // CPU_LE
    u8      numb_path   :7;     // Max number of paths to (be) return(ed)
    u8      reversible  :1;
#endif


    u16     pkey;               // Partition Key for this path

#if CPU_BE
    u16     reserved4  :12;
    u16     sl         :4;      // Service level
#else // CPU_LE
    u16     sl         :4;      // Service level
    u16     reserved4  :12;
#endif
    
#if CPU_BE
    
    u8      mtu_selector    :2; 
    u8      mtu         :6;     
    u8      rate_selector :2;   
    u8      rate        :6;     
    u8      pkt_life_time_selector:2;
    u8      pkt_life_time   :6;     

#else // CPU_LE
    u8      mtu         :6;     // Enumeration of MTU required
                                    // 1:       256 bytes
                                    // 2:       512 bytes
                                    // 3:       1024 bytes
                                    // 4:       2048 bytes
                                    // 5:       4096 bytes
                                    // 6-63:    reserved
    u8      mtu_selector    :2; // 0: greater than or equal to MTU specified
                                // 1: less than or equal to MTU specified
                                // 2: exactly the MTU specified

    u8      rate        :6;     // data rate for the path
                                    // 1:       1   GB/sec
                                    // 2:       2.5 GB/sec
                                    // 3:       10  GB/sec
                                    // 4:       30  GB/sec
                                    // 5-64:    reserved
    u8      rate_selector :2;   // 0: greater than or equal to Rate specified
                                // 1: less than or equal to Rate specified
                                // 2: exactly the Rate specified

    u8      pkt_life_time   :6;     // *****************************************
                                    // *** User be aware that the CM LifeTime &
                                    // *** TimeOut values are only 5-bit wide.
                                    // *****************************************
                                    //
                                    // Accumulated packet life time for
                                    // the path specified by an enumeration 
                                    // deried from
                                    //   4.096 usec * 2^PktLifeTime
    u8      pkt_life_time_selector:2;
                                // 0: greater than or equal to PacketLifeTime specified
                                // 1: less than or equal to PacketLifeTime specified
                                // 2: exactly the PacketLifeTime specified

#endif


    u8      preference;         // larger means worse path 
    u8      reserved2 [6];      // alignment padding to make the next path record
                                // starts from 64-bit alignment address
}  ib_path_record_v2_t;

typedef struct{

#if CPU_BE
    u32     raw_traffic :1;     // for IB Packet = 0 (P_Key must be valie)
    u32     reserved3   :3;
    u32     flow_label  :20;    // used in GRH      
    u32     hop_limut   :8;     // Hop limit used in GRH
#else // CPU_LE
    u32     hop_limut   :8;     // Hop limit used in GRH
    u32     flow_label  :20;    // used in GRH      
    u32     reserved3   :3;
    u32     raw_traffic :1;     // for IB Packet = 0 (P_Key must be valie)
#endif

    u8      tclass;
    
#if CPU_BE
    u8      reversible  :1;
    u8      numb_path   :7;     // Max number of paths to (be) return(ed)
#else // CPU_LE
    u8      numb_path   :7;     // Max number of paths to (be) return(ed)
    u8      reversible  :1;
#endif

    u16     pkey;               // Partition Key for this path

#if CPU_BE
    u16     reserved4  :12;
    u16     sl         :4;      // Service level
#else // CPU_LE
    u16     sl         :4;      // Service level
    u16     reserved4  :12;
#endif

#if CPU_BE
    
    u8      mtu_selector    :2; 
    u8      mtu         :6;     
    u8      rate_selector :2;   
    u8      rate        :6;     
    u8      pkt_life_time_selector:2;
    u8      pkt_life_time   :6;     

#else // CPU_LE
    u8      mtu         :6;     // Enumeration of MTU required
                                    // 1:       256 bytes
                                    // 2:       512 bytes
                                    // 3:       1024 bytes
                                    // 4:       2048 bytes
                                    // 5:       4096 bytes
                                    // 6-63:    reserved
    u8      mtu_selector    :2; // 0: greater than or equal to MTU specified
                                // 1: less than or equal to MTU specified
                                // 2: exactly the MTU specified

    u8      rate        :6;     // data rate for the path
                                    // 1:       1   GB/sec
                                    // 2:       2.5 GB/sec
                                    // 3:       10  GB/sec
                                    // 4:       30  GB/sec
                                    // 5-64:    reserved
    u8      rate_selector :2;   // 0: greater than or equal to Rate specified
                                // 1: less than or equal to Rate specified
                                // 2: exactly the Rate specified

    u8      pkt_life_time   :6;     // *****************************************
                                    // *** User be aware that the CM LifeTime &
                                    // *** TimeOut values are only 5-bit wide.
                                    // *****************************************
                                    //
                                    // Accumulated packet life time for
                                    // the path specified by an enumeration 
                                    // deried from
                                    //   4.096 usec * 2^PktLifeTime
    u8      pkt_life_time_selector:2;
                                // 0: greater than or equal to PacketLifeTime specified
                                // 1: less than or equal to PacketLifeTime specified
                                // 2: exactly the PacketLifeTime specified

#endif
    u8	    reserved5;

#if CPU_BE    
    u8	    indep_sel:2;
    u8      reserved6:6;
#else
    u8      reserved6:6;
    u8	    indep_sel:2;
#endif
    
    u8	    sgid_count;
    u8	    dgid_count;
    
    u8	    reserved7[7];
    vv_gid_t s_d_gid[0]; // list of sgid_count * sgids followed of dgid_count * dgids ()
    
} ib_multi_path_record_v2_t;

// MCMemberRecord (note: class version 2)

#define IB_MC_JOIN_STATE_FULL			1
#define IB_MC_JOIN_STATE_NON			2
#define IB_MC_JOIN_STATE_SEND_ONLY		4

typedef struct{
   vv_gid_t            	mgid;            /* Multicast GID address for this multicast group. */
   vv_gid_t             port_gid;        /* Valid GID of the endpoint joining this multicast group. */
   vv_q_key_t           qkey;            /* Q_Key to be used by this multicast group */

#if CPU_LE
   u32                  mlid:16;
   u32                  mtu:6;             /* swap 1 */
   u32                  mtu_selector:2;    /* swap 1 */         
   u32                  tclass:8;          /* Another global routing parameter. */
#else
   u32                  mlid:16;
   u32                  mtu_selector:2;             
   u32                  mtu:6;             
   u32                  tclass:8;          /* Another global routing parameter. */
#endif
   
#if CPU_LE
   u32                  pkey:16;            /* Partition key (P_Key) to use for this member. */
   u32                  rate:6;             /* swap 1 */
   u32                  rate_selector:2;    /* swap 1 */
   u32                  pkt_life:6;         /* swap 2 */
   u32                  pkt_life_selector:2;/* swap 2 */
#else
   u32                  pkey:16;            /* Partition key (P_Key) to use for this member. */
   u32                  rate_selector:2;
   u32                  rate:6; 
   u32                  pkt_life_selector:2;    
   u32                  pkt_life:6;   
#endif


#if CPU_LE
   u32                  sl:4;     
   u32                  flow_label:20;
   u32                  hop_limit:8;
#else
   u32                  hop_limit:8;
   u32                  flow_label:20;
   u32                  sl:4;     
#endif

#if CPU_LE
   u32                  join_state:4;
   u32                  scope:4;
   u32                  reserved0:7;
   u32                  proxy_join:1;
   u32                  reserved1:16;
#else
   u32                  scope:4;
   u32                  join_state:4;
   u32                  proxy_join:1;
   u32                  reserved0:7;
   u32                  reserved1:16;
#endif
   u8                   pad[4];
} ib_mcmember_record_v2_t;

//Notice
typedef struct{
#if CPU_LE
   u32                 type:7;
   u32                 is_generic:1;
   u32                 producer_type_or_vendor_id:24;
#else   
   u32                 is_generic:1;
   u32                 type:7;
   u32                 producer_type_or_vendor_id:24;
#endif   

   u16                 trap_num_or_dev_id;
   u16                 issuer_lid;
   
#if CPU_LE
   u16                 notice_toggle:1;
   u16                 notice_count:15;
#else   
   u16                 notice_count:15;
   u16                 notice_toggle:1;
#endif   

   u8                  data_details[54];
   vv_gid_t            issuer_gid;
}ib_notice_v2_t;

//Inform Info
typedef struct{
   vv_gid_t            gid;
   vv_lid_t            lid_range_begin;
   vv_lid_t            lid_range_end;
   u16                 reserved1;
   u8                  is_generic;
   u8                  subscribe;
   u16                 type;
   u16                 trap_num_or_dev_id;

#if CPU_LE
   u32                 qpn:24;
   u32                 resp_time_val:5;
   u32                 reserved2:3;
#else
   u32                 qpn:24;
   u32                 reserved2:3;
   u32                 resp_time_val:5;
#endif   
   
   
   u32                 reserved3:8;
   u32                 producer_type_or_vendor_id:24;
   
}ib_inform_info_v2_t;

//Inform Info Record
typedef struct{
   vv_gid_t            subscriber_gid;
   u16                 _enum;
   u16                 reserved1;
   u32                 reserved2;
   ib_inform_info_v2_t inform_info;
}ib_inform_info_record_v2_t;




// Service Record
typedef struct{
    ib_rid_t    service_rid;                // RID for this Service record
    u32         service_lease;          // Lease period remaining for this
                                            //      service (in secconds).
                                            // 0xFFFFFFFF is infinite lease.
    u16         partition;              // Partition of this service

#if CPU_BE

    u16         service_generic_flags :4;   // Generic information related
                                            //      the service
                                            // bit 0:   Indirection (set if the
                                            //          service provider my
                                            //          redirect requests)
                                            // bit 1:   DHCP-capable (set if a
                                            //          DHCP server or a Directory
                                            //          Agent may automatically
                                            //          register this service
                                            //          by querying the SA)
                                            // bit 2-3: reserved
    u16         service_specific_flags:12;// content is service specific

#else // CPU_LE
    u16         service_specific_flags:12;// content is service specific
    u16         service_generic_flags :4;   // Generic information related
                                            //      the service
#endif


    char        service_name[124];      // Null-terminated service name.
                                            // ASCII string.

    char        service_gid[40];            // The text representation of IPv6
                                            // compliant of Port GID (128-bit) for
                                            // this service, null terminated.
                                            // In x:x:x:x:x:x:x:x form where each
                                            // x is at most 4 hexadecimal char.

    char        service_id[16];             // String of 16 hexadecimal digits
                                            // including any leading zeros.
                                            // NOT null-terminated.
}   ib_service_record_t;



typedef struct{
	u64     	service_id;
	vv_gid_t	service_gid;
	u16		service_pkey;
	u16		reserved1;
	u32		service_lease;
	u8		service_key[16];
	u8		service_name[64];
	u8		service_data8[16];
	u16		service_data16[8];
	u32		service_data32[4];
	u64		service_data64[2];
	
} ib_service_record_v2_t;

typedef struct{
	u8		service_key[16];
	u8		service_name[64];
	
} ib_service_assotiation_record_v2_t;

typedef struct{
	u8 		node_string[64];
} ib_node_desc_v2_t;

typedef struct{
	u8 		base_version;
	u8		class_version;
	u8		node_type;
	u8		num_ports;
	u64		system_image_guid;
	u64		node_guid;
	u64		port_guid;
	u16		partition_cap;
	u16		device_id;
	u32		revision;
	
	u32		local_port_num:8;
	u32		vendor_id:24;

} ib_node_info_v2_t;

typedef struct{
	vv_lid_t	lid;
	u16		reserved;
	ib_node_info_v2_t	node_info;
	ib_node_desc_v2_t	node_description;
	u8       pad[4];
} ib_node_record_v2_t;
	
typedef struct{
	u64		m_key;
	u64		gid_prefix;
	vv_lid_t	lid;
	vv_lid_t	master_sm_lid;
	u32		capability_mask;
	u16		diag_code;
	u16		m_key_lease_period;
	u8		local_port_num;
	u8		link_width_enable;
	u8		link_width_supported;
	u8		link_width_active;
	
#if CPU_BE
	u8		link_speed_supported:4;
	u8		port_state:4;
#else
	u8		port_state:4;
	u8		link_speed_supported:4;
#endif

#if CPU_BE
	u8		port_phys_state:4;
	u8		link_down_default_state:4;
#else
	u8		link_down_default_state:4;
	u8		port_phys_state:4;
#endif

#if CPU_BE
	u8		m_key_protect_bits:2;
	u8		reserved1:3;
	u8		lmc:3;
#else
	u8		lmc:3;
	u8		reserved1:3;
	u8		m_key_protect_bits:2;
#endif

#if CPU_BE
	u8		link_speed_active:4;
	u8		link_speed_enabled:4;
#else
	u8		link_speed_enabled:4;
	u8		link_speed_active:4;
#endif

#if CPU_BE
	u8		neighbor_mtu:4;
	u8		master_sm_sl:4;
#else
	u8		master_sm_sl:4;
	u8		neighbor_mtu:4;
#endif

#if CPU_BE
	u8		vl_cap:4;
	u8		init_type:4;
#else
	u8		init_type:4;
	u8		vl_cap:4;
#endif

	u8		vl_high_limit;
	u8		vl_arbitration_high_cap;
	u8		vl_arbitration_low_cap;

#if CPU_BE
	u8		init_type_reply:4;
	u8		mtu_cap:4;
#else
	u8		mtu_cap:4;
	u8		init_type_reply:4;
#endif

#if CPU_BE
	u8		vl_stall_count:3;
	u8		hoq_life:5;
#else
	u8		hoq_life:5;
	u8		vl_stall_count:3;
#endif

#if CPU_BE
	u8		operational_vls:4;
	u8		partition_enforcement_inbound:1;
	u8		partition_enforcement_outbound:1;
	u8		filter_raw_inbound:1;
	u8		filter_raw_outbound:1;
#else
	u8		filter_raw_outbound:1;
	u8		filter_raw_inbound:1;
	u8		partition_enforcement_outbound:1;
	u8		partition_enforcement_inbound:1;
	u8		operational_vls:4;
#endif

	u16		m_key_violations;
	u16		p_key_violations;
	u16		q_key_violations;
	u8		guid_cap;

#if CPU_BE
	u8		reserved2:3;
	u8		subnet_timeout:5;
#else
	u8		subnet_timeout:5;
	u8		reserved2:3;
#endif
	
#if CPU_BE
	u8		reserved3:3;
	u8		resp_time_value:5;
#else
	u8		resp_time_value:5;
	u8		reserved3:3;
#endif

#if CPU_BE
	u8		local_phy_errors:4;
	u8		overrun_errors:4;
#else
	u8		overrun_errors:4;
	u8		local_phy_errors:4;
#endif
} ib_port_info_v2_t;

typedef struct{
	vv_lid_t	        endport_lid;
	u8                  port_num;
	u8                  reserved;
	ib_port_info_v2_t	port_info;
	u8                  pad[6];
} ib_port_info_record_v2_t;
	

//
// NodeInfo
//

typedef struct node_info_st {

    u8  base_version;       // Supported MAD Base Version.
    u8  class_version;      // Supported Subnet Management Class 
                                // (SMP) Version.
    u8  node_type;  
    u8  num_ports;          // Number of link ports on this node.

    //
    // BUGBUG: GUIDs on 4 byte boundary!!!   
    //
    u64 platform_guid;      

    u64 node_guid;          // GUID of the HCA, TCA, switch, 
                                // or router itself.
    u64 port_guid;          // GUID of this end port itself. 

    u16 partition_cap;      // Number of entries in the Partition Table
                                // for end ports.
    u16 device_id;          // Device ID information as assigned by 
                                // device manufacturer.
    u32 revision;           // Device revision, assigned by manufacturer.

    u8  reserved;           // shall be zero.

    //
    // BUGBUG: Not on 4 byte boundary!!!   
    //
    union _ID_PORT_NO 
    {
        u32 asreg32;

        struct 
        {
            u32 local_port_num: 8;  // The link port number this SMP 
                                        // came on in.
            u32 vendor_id:      24; // Device vendor, per IEEE.
        } port_id;
    } u;

}  node_info_t;


//
// NodeDescription
//
typedef struct {
    
    u8  node_string[32];            // UNICODE string to describe node 
                                    // in text format.
}  node_description_t;


typedef struct  {
    ib_rid_t        node_rid;               // This nodes's RID used as RID of
                                            // this record, 
    node_info_t     node_info_data;
    node_description_t  node_description_data;
}  ib_node_record_t;




typedef struct
{
    mad_hdr_t hdr;
    
    u64       sa_key;           // Subnet Administration KEY
    u64       sm_key;           // SM_KEY
    u32       segment_num;      // Segment number
    u32       payload_len;      // Payload length
    u8        fragment_flag;    // Fragment flag
    u8        edit_mod;         // Edit Modifier
    u16       window;           // Timeout value for multipacket MAD
    u32       end_rid;          // End Record Identifier
    u64       component_mask;   // Component mask for query operation

    unsigned char payload[192];
} sa_mad_t;

typedef struct
{
    mad_hdr_t hdr;
    
    u8        rmpp_version;
    u8	      rmpp_type;
    u8	      rmpp_flags;
    u8	      rmpp_status;

    u32       segment_num;      // Segment number
    u32       payload_len;      // Payload length
    u64       sm_key;           // SM_KEY
    u16       attr_offset;     
    u16       reserved;      
    u64       component_mask;   // Component mask for query operation

    unsigned char payload[200];
} sa_mad_v2_t;

typedef struct
{
    u64       sm_key;           // SM_KEY
    u16       attr_offset;
    u16       reserved;
    u64       component_mask;   // Component mask for query operation
 
    unsigned char payload[200];
} sa_mad_v2_rmpp_payload_t;

#ifdef  CONFIG_PPC
#pragma pack(4)
#else
#pragma pack(pop)
#endif

#endif
