/*
 * Copyright (c) 2002-2003, Network Appliance, Inc. All rights reserved.
 *
 * This Software is licensed under either one of the following two licenses:
 *
 * 1) under the terms of the "Common Public License 1.0" a copy of which is
 *    in the file LICENSE.txt in the root directory. The license is also
 *    available from the Open Source Initiative, see
 *    http://www.opensource.org/licenses/cpl.php.
 * OR
 *
 * 2) under the terms of the "The BSD License" a copy of which is in the file
 *    LICENSE2.txt in the root directory. The license is also available from
 *    the Open Source Initiative, see
 *    http://www.opensource.org/licenses/bsd-license.php.
 *
 * Licensee has the right to choose either one of the above two licenses.
 *
 * Redistributions of source code must retain both the above copyright
 * notice and either one of the license notices.
 *
 * Redistributions in binary form must reproduce both the above copyright
 * notice, either one of the license notices in the documentation
 * and/or other materials provided with the distribution.
 */

/**********************************************************************
 *
 * HEADER: dat_vendor_specific.h
 *
 * PURPOSE:
 *
 * Description: Header file for "uDAPL: User Direct Access Programming
 *		Library, Version: 1.1"
 *
 * Mapping rules:
 *      All global symbols are prepended with "DAT_" or "dat_"
 *      All DAT objects have an 'api' tag which, such as 'ep' or 'lmr'
 *      The method table is in the provider definition structure.
 *
 *
 **********************************************************************/

#ifndef _DAT_VENDOR_SPECIFIC_H_
#define _DAT_VENDOR_SPECIFIC_H_


/* General Interface Adapter attributes. These apply to both udat and kdat. */

/* To support backwards compatibility for DAPL-1.0 */
#define max_rdma_read_per_ep 		max_rdma_read_per_ep_in
#define DAT_IA_FIELD_IA_MAX_DTO_PER_OP 	DAT_IA_FIELD_IA_ MAX_DTO_PER_EP_IN

typedef struct dat_ia_attr
{
    char                        adapter_name[DAT_NAME_MAX_LENGTH];
    char                        vendor_name[DAT_NAME_MAX_LENGTH];
    DAT_UINT32                  hardware_version_major;
    DAT_UINT32                  hardware_version_minor;
    DAT_UINT32                  firmware_version_major;
    DAT_UINT32                  firmware_version_minor;
    DAT_IA_ADDRESS_PTR		ia_address_ptr;
    DAT_COUNT                   max_eps;
    DAT_COUNT                   max_dto_per_ep;
    DAT_COUNT                   max_rdma_read_per_ep_in;
    DAT_COUNT                   max_rdma_read_per_ep_out;
    DAT_COUNT                   max_evds;
    DAT_COUNT                   max_evd_qlen;
    DAT_COUNT                   max_iov_segments_per_dto;
    DAT_COUNT                   max_lmrs;
    DAT_VLEN                    max_lmr_block_size;
    DAT_VADDR                   max_lmr_virtual_address;
    DAT_COUNT                   max_pzs;
    DAT_VLEN                    max_mtu_size;
    DAT_VLEN                    max_rdma_size;
    DAT_COUNT                   max_rmrs;
    DAT_VADDR                   max_rmr_target_address;
    DAT_COUNT                   num_transport_attr;
    DAT_NAMED_ATTR              *transport_attr;
    DAT_COUNT                   num_vendor_attr;
    DAT_NAMED_ATTR              *vendor_attr;
} DAT_IA_ATTR;

typedef enum dat_ia_attr_mask
{
    DAT_IA_FIELD_IA_ADAPTER_NAME                = 0x000001,
    DAT_IA_FIELD_IA_VENDOR_NAME                 = 0x000002,
    DAT_IA_FIELD_IA_HARDWARE_MAJOR_VERSION      = 0x000004,
    DAT_IA_FIELD_IA_HARDWARE_MINOR_VERSION      = 0x000008,
    DAT_IA_FIELD_IA_FIRMWARE_MAJOR_VERSION      = 0x000010,
    DAT_IA_FIELD_IA_FIRMWARE_MINOR_VERSION      = 0x000020,
    DAT_IA_FIELD_IA_ADDRESS_PTR                 = 0x000040,
    DAT_IA_FIELD_IA_MAX_EPS                     = 0x000080,
    DAT_IA_FIELD_IA_MAX_DTO_PER_EP              = 0x000100,
    DAT_IA_FIELD_IA_MAX_RDMA_READ_PER_EP_IN     = 0x000200,
    DAT_IA_FIELD_IA_MAX_RDMA_READ_PER_EP_OUT    = 0x000400,
    DAT_IA_FIELD_IA_MAX_EVDS                    = 0x000800,
    DAT_IA_FIELD_IA_MAX_EVD_QLEN                = 0x001000,
    DAT_IA_FIELD_IA_MAX_IOV_SEGMENTS_PER_DTO    = 0x002000,
    DAT_IA_FIELD_IA_MAX_LMRS                    = 0x004000,
    DAT_IA_FIELD_IA_MAX_LMR_BLOCK_SIZE          = 0x008000,
    DAT_IA_FIELD_IA_MAX_LMR_VIRTUAL_ADDRESS     = 0x010000,
    DAT_IA_FIELD_IA_MAX_PZS                     = 0x020000,
    DAT_IA_FIELD_IA_MAX_MTU_SIZE                = 0x040000,
    DAT_IA_FIELD_IA_MAX_RDMA_SIZE               = 0x080000,
    DAT_IA_FIELD_IA_MAX_RMRS                    = 0x100000,
    DAT_IA_FIELD_IA_MAX_RMR_TARGET_ADDRESS      = 0x200000,
    DAT_IA_FIELD_IA_NUM_TRANSPORT_ATTR          = 0x400000,
    DAT_IA_FIELD_IA_TRANSPORT_ATTR              = 0x800000,
    DAT_IA_FIELD_IA_NUM_VENDOR_ATTR             = 0x1000000,
    DAT_IA_FIELD_IA_VENDOR_ATTR                 = 0x2000000,

    DAT_IA_ALL                                  = 0x3FFFFFF
} DAT_IA_ATTR_MASK;


/* Vendor Specific extensions */

#if defined(_JNI)

#elif defined(_INTEL)

#elif defined(_INFINISWITCH)

#elif defined(_MELLANOX)

#elif defined(_INFINICON)

#elif defined(_TOPSPIN)

#endif

#endif /* _VENDOR_SPECIFIC_H_ */


