/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

# ifndef __VV_VV_TYPES_H__
# define __VV_VV_TYPES_H__

//////////////////////////////////////////////////////////
//
// Author: Yiftah Shahar <yiftahs@voltaire.com>
//
// Description: 
// Voltaire verbs types defenitiond for verbs user applications.
//
//////////////////////////////////////////////////////////
           
           
#include <linux/kernel.h>
#include <linux/types.h>

#include "ib-types.h"

////////////////////////////////////////////////////////////////////////
// constants

// Vendor device numbers (every module can do test on MLX or IBM)
// Default value for all test will be: VV_VEND_ANY_ID
#define VV_VEND_ANY_ID        (0)
#define VV_VEND_MLX_ID        (1)
#define VV_VEND_IBM_ID        (2)

// Different supported string for HCA name
#define VV_DEFAULT_HCA_NAME vv_default_hca_name()
#define VV_ANY_DEVICE_NAME_S1 VV_DEFAULT_HCA_NAME



        
#define VV_MAX_DEV_NAME_SIZE       (100)   // max device name size (like: "mt21108_pci0", "TORRENT")

#define MAX_RETURN_TYPES_NUMBER    (256)   // max return numbers
                                    
#define VV_MAX_SEND_AND_RDMA_SIZE  (65536) //page*16: For now this is the max size of send/receive and RDMA (read/write)
                                           //         operation that can be done. This is because Memory Region Register limitation
                                           //         of mlx and the use of several generic memory regions with overlap (with this size).

#define VV_MAX_PORT_NUMBER_IN_HCA   (2)    // for now this is the max ports per HCA - use in the IBM adaptor
#define VV_MAX_GIDS_NUMBER_PER_PORT (100)  // for now this is the max GIDs per port - use in the IBM adaptor
#define VV_MAX_PKEY_NUMBER_PER_PORT (100)  // for now this is the max pkey's per port - use in the IBM adaptor

#define VV_MAX_SCAT_GAT_LIST_ENTRYS (8)    // max number for the scatter gather list in the work request (send receive)

#define VV_DEFAULT_L_ACK_T (0x12)          // defult local_ack_timeout - in IBM its ~1 sec

///////////////////////////////////////////////////////////////////////
// parameters definitions

typedef u_int8_t            vv_service_level_t;  // need only 4 byte
typedef u_int32_t           vv_psn_t;            // need only 24 byte
typedef u_int64_t           vv_wr_id_t;

//typedef u_int32_t           vv_port_capability_flags_t; // Port Capability flags combination

typedef u_int32_t           vv_mem_w_bind_opt_t; //BUGBUG - need to see what to do with this, it is for the bind memory window 

#ifdef ARCH_64_BIT
#define BASE_TYPE_CAST (u_int64_t)
#else
#define BASE_TYPE_CAST (u_int32_t)
#endif /* ARCH_64_BIT */

////////////////////////////////////////////////////////////////////////
// handles - won't change by the verb consumer

typedef u_int32_t		vv_rdd_h_t;      // Reliable Datagram Domain (RDD) (RD only)	is VAPI_rdd_hndl_t
typedef u_int32_t		vv_eec_h_t;      // 											is VAPI_eec_hndl_t
typedef unsigned long	vv_pd_h_t;       // Protection Domain (PD)						is VAPI_pd_hndl_t

typedef unsigned long	vv_add_h_t;      // Address Vector								is VAPI_ud_av_hndl_t
//typedef unsigned long	vv_cq_h_t;       // Completion Queue (CQ)					   	is VAPI_cq_hndl_t
//typedef unsigned long	vv_qp_h_t;       // QP											is VAPI_qp_hndl_t
typedef u_int32_t		vv_mem_reg_h_t;  // Memory region (physical or virtual are the same p. 517)
																						//		is VAPI_mr_hndl_t
typedef unsigned long	vv_mem_win_h_t;  // Memory Window 								is VAPI_mw_hndl_t


typedef struct vv_hca_st*     vv_hca_h_t;      // HCA
//typedef struct vv_rdd_st*     vv_rdd_h_t;      // Reliable Datagram Domain (RDD) (RD only)
//typedef struct vv_eec_st*     vv_eec_h_t;      //
//typedef struct vv_pd_st*      vv_pd_h_t;       // Protection Domain (PD)
//typedef struct vv_add_st*     vv_add_h_t;      // Address Vector
typedef struct vv_cq_st*      vv_cq_h_t;       // Completion Queue (CQ)
typedef struct vv_qp_st*      vv_qp_h_t;       // QP
//typedef struct vv_mem_reg_st* vv_mem_reg_h_t;  // Memory region (physical or virtual are the same p. 517)
//typedef struct vv_mem_win_st* vv_mem_win_h_t;  // Memory Window
typedef u_int32_t             vv_fmr_h_t;      // FMR // Mellanox specific the same as mlx VAPI_mr_hndl_t (YSBUGBUG - need to check for every version)

/*
typedef void* vv_hca_h_t;      // HCA
typedef void* vv_rdd_h_t;      // Reliable Datagram Domain (RDD) (RD only)
typedef void* vv_eec_h_t;      //
typedef void* vv_pd_h_t;       // Protection Domain (PD)
typedef void* vv_add_h_t;      // Address Vector
typedef void* vv_cq_h_t;       // Completion Queue (CQ)
typedef void* vv_qp_h_t;       // QP
typedef void* vv_mem_reg_h_t;  // Memory region (physical or virtual are the same p. 517)
//typedef void* vv_mem_reg_h_t;    // Physical Memory
//typedef void* vv_mem_h_t;      // Memory
typedef void* vv_mem_win_h_t;  // Memory Window
*/

////////////////////////////////////////////////////////////////////////
//enums definition

// Return types (the list is from the spec. maybe need to add more for different providers)
// For now the numbers are 0, -256, -255, -254.... and this is for mlx adaptor 
typedef enum
{
  // immediate return result
  vv_return_ok  = 0,                        // (0)
  vv_return_err = -MAX_RETURN_TYPES_NUMBER, // (-256) 

  //general
  vv_return_err_insufficient_resources,     // (-255)
  vv_return_err_cq_num,                     // (-254)
  vv_return_err_wr_num,                     // (-253)
  vv_return_err_scatgat_num,                // (-252)
  vv_return_err_to_many_wr_posted,          // (-251)
  vv_return_err_raw_qp_num,                 // (-250)
  vv_return_err_multicast_qp_num,           // (-249)
  vv_return_err_hca_already_open,           // (-248)
  vv_return_err_hca_name,                   // (-247)
  vv_return_err_hca_handle,                 // (-246)
  vv_return_err_port_mtu,                   // (-245)
  vv_return_err_port,                       // (-244) // also for affiliated
  vv_return_err_counter,                    // (-243)
  vv_return_err_counter_value,              // (-242)
  vv_return_err_address_h,                  // (-241)
  vv_return_err_qp_h,                       // (-240)
  vv_return_err_qp_chang_attrib,            // (-239)
  vv_return_err_qp_state,                   // (-238)
  vv_return_err_qp_service,                 // (-237)
  vv_return_err_qp_is_used,                 // (-236)
  vv_return_err_atom_not_support,           // (-235)
  vv_return_err_raw_not_support,            // (-234)
  vv_return_err_rd_not_support,             // (-233)
  vv_return_err_operation_type,             // (-232)
  vv_return_err_scatgat_format,             // (-231)
  vv_return_err_scatgat_length,             // (-230)
  vv_return_err_path_migrat_state,          // (-229)
  vv_return_err_special_qp_type,            // (-228)
  vv_return_err_wq_entry_num,               // (-227)
  vv_return_err_cq_h,                       // (-226)
  vv_return_err_cq_entry_num,               // (-225)
  vv_return_err_cq_connect_to_wqe,          // (-224)
  vv_return_err_cq_empty,                   // (-223)
  vv_return_err_completion_type,            // (-222)
  vv_return_err_eec_h,                      // (-221)
  vv_return_err_eec_state,                  // (-220)
  vv_return_err_eec_change_attrib,          // (-219)
  vv_return_err_rdd_in_use,                 // (-218)
  vv_return_err_rdd,                        // (-217)
  vv_return_err_rnr_nak_timer,              // (-216)
  vv_return_err_pd,                         // (-215)
  vv_return_err_pd_in_used,                 // (-214)
  vv_return_err_va,                         // (-213)
  vv_return_err_length,                     // (-212)
  vv_return_err_phy_buff_list,              // (-211)
  vv_return_err_offset,                     // (-210)
  vv_return_err_l_key,                      // (-209)
  vv_return_err_r_key,                      // (-208)
  vv_return_err_mem_region_h,               // (-207)
  vv_return_err_mem_win_h,                  // (-206)
  vv_return_err_access_control,             // (-205)
  vv_return_err_region_got_win,             // (-204)
  vv_return_err_multicast_dlid,             // (-203)
  vv_return_err_multicast_ipv6_addr,        // (-202)
  vv_return_err_p_key_out_of_range,         // (-201)
  vv_return_err_p_key_table_entry,          // (-200)
  
  // completion return status
  vv_return_err_l_length,                   // (-199)
  vv_return_err_l_qp_opt,                   // (-198)
  vv_return_err_l_eec_opt,                  // (-197)
  vv_return_err_l_protection,               // (-196)
  vv_return_err_wr_flused,                  // (-195)
  vv_return_err_mem_win_bind,               // (-194)
  
  //for Reliable qp
  vv_return_err_r_request,                  // (-193)
  vv_return_err_r_access,                   // (-192)
  vv_return_err_r_opt,                      // (-191)
  vv_return_err_retry_counter,              // (-190)
  vv_return_err_rnr_counter,                // (-189)

  //for RD QP or EEC
  //vv_return_err_r_request, // do not need it, the same as in Reliable qp
  vv_return_err_r_abort,                    // (-188)
  vv_return_eer_eec_num,                    // (-187)
  vv_return_eer_eec_state,                  // (-186)

  //asynchronous event
  vv_return_path_migrated,                  // (-185) // also for affiliated
  vv_return_communicat,                     // (-184)
  vv_return_sed_q_drained,                  // (-183)

  //affiliated asynchronous event           
  vv_return_eer_cq,                         // (-182)
  vv_return_eer_l_wq_catastrofic,           // (-181)
  vv_return_eer_l_eec_catastrofic,          // (-180)
  vv_return_eer_l_catastrofic,              // (-179)

  //voltaire
  vv_return_err_not_implemented,            // (-178)
  vv_return_err_alloc_mem,                  // (-177)
  vv_return_err_general,                    // (-176)
  vv_return_err_invalid_state,              // (-175)
  vv_return_err_busy,                       // (-174)
  vv_return_err_invalid_operation,          // (-173)
  vv_return_err_invalid_parameter,          // (-172)
  vv_return_err_adaptor_data_corruption,    // (-171)
  vv_return_err_invalid_vend_ret            // (-170)

}vv_return_t;


// QP types and the Transport service types in one place
// like CM AMD - (p. 564)
// DO NOT CHANGE
// mlx - translation array
typedef enum
{
  vv_qp_type_r_conn,           // (0) Transport Service
  vv_qp_type_u_conn,           // (1) Transport Service
  vv_qp_type_r_dgram,          // (2) Transport Service
  vv_qp_type_u_dgram,          // (3) Transport Service
  vv_qp_type_special_smi_0,    // (4)
  vv_qp_type_special_gsi_1,    // (5)
  vv_qp_type_special_raw_ipv6, // (6)
  vv_qp_type_special_raw_eter, // (7)
  vv_qp_type_not_valid         // (8)
}vv_qp_type_t;

// QP states (P. 412)
// DO NOT CHANGE
// mlx - translation array
// IBM - translation array
typedef enum
{
  vv_qp_state_reset,      // (0) Newly created QP start here
  vv_qp_state_init,       // (1) can post WR to the queue
  vv_qp_state_rtr,        // (2) Ready to Receive (can process incoming messages)
  vv_qp_state_rts,        // (3) Ready to Send (before transition into this stage Communication establish protocol must be completed)
  vv_qp_state_sqd_to_rts, // (4) For modify QP use only
  vv_qp_state_sqe_to_rts, // (5) For modify QP use only  
  vv_qp_state_sqd,        // (6) Send Queue Drain
  vv_qp_state_sqe,        // (7) Send Queue Error
  vv_qp_state_sqd_ing,    // (8) Draining
  vv_qp_state_sqd_ed,     // (9) draining ended  
  vv_qp_state_error       // (10) Error
}vv_qp_state_t;

// QP and EEC states are the same but i made 2 types for naming clarification
// EEC states
typedef enum
{
  vv_eec_state_reset,
  vv_eec_state_init,
  vv_eec_state_rtr,
  vv_eec_state_rts,
  vv_eec_state_sqd_to_rts, // For modify QP use only
  vv_eec_state_sqe_to_rts, // For modify QP use only
  vv_eec_state_sqe,
  vv_eec_state_sqd,
  vv_eec_state_sqd_ing, // Draining
  vv_eec_state_sqd_ed,  // draining ended  
  vv_eec_state_error
}vv_eec_state_t;

// work request operation types (maybe need to change this p. 528)
// DO NOT CHANGE
// mlx - translation array
// IBM - translation array
typedef enum
{
  vv_wr_send,              // (0) Send
  vv_wr_send_immed,        // (1) I readd this for the mlx to indicate immediate Data
  vv_wr_rdma_read,         // (2) RDMA Read
  vv_wr_rdma_write,        // (3) RDMA Write
  vv_wr_rdma_write_immed,  // (4) I readd this for the mlx to indicate immediate Data
  vv_wr_compare_swap,      // (5) Atomic
  vv_wr_fetch_add,         // (6) Atomic
  vv_wr_receive,           // (7) the only receive type (p. 531)
  vv_wr_mem_win_bind       // (8) for Memory Window bind
}vv_wr_operation_t;

// work completion operation types (p. 533)
// The same as in mlx
// IBM translation array
typedef enum
{
  vv_wc_send_sq,         // (0)
  vv_wc_rdma_write_sq,   // (1)
  vv_wc_rdma_read_sq,    // (2)
  vv_wc_comare_swap_sq,  // (3)
  vv_wc_fetch_add_sq,    // (4)
  vv_wc_mem_win_bind_sq, // (5)
  vv_wc_send_rq,         // (6)
  vv_wc_rdma_immidiate,  // (7) // For RDMA Write Only
  vv_wc_inval            // (8) // invalid value return in case of status error
}vv_wc_operation_t;

// MTU
// enumeration like like PathRecord and CM (p. 718)
// Like MLX
typedef enum
{
  vv_mtu_256 = 1,  // (1) 256
  vv_mtu_512,      // (2) 512
  vv_mtu_1024,     // (3) 1024
  vv_mtu_2048,     // (4) 2048
  vv_mtu_4096      // (5) 4096
}vv_mtu_t;

// Static rate (p. 394)
typedef enum
{
    vv_static_rate_ipd0,
    vv_static_rate_ipd1,
    vv_static_rate_ipd2,
    vv_static_rate_ipd3,
    vv_static_rate_ipd11
}vv_static_rate_t;

// Port state
typedef enum
{
  vv_state_linkNoChange, // No state change - for mlx
  vv_state_linkDoun,
  vv_state_linkInit,
  vv_state_linkArm,
  vv_state_linkActive,
  vv_state_linkActDefer
}vv_port_state_t;

// HCA Capability
typedef enum
{
  vv_hca_cap_resize_outstanding_wqe = 0x01,
  vv_hca_cap_bad_pkey_count         = 0x02,
  vv_hca_cap_bad_qkey_count         = 0x04,
  vv_hca_cap_modify_outstanding_wqe = 0x08,
  vv_hca_cap_raw_molticast          = 0x10,
  vv_hca_cap_path_migration         = 0x20,
  vv_hca_cap_change_phy_port        = 0x40
}vv_hca_capability_t;

// Port Capability (p. 478)
typedef enum
{
  vv_capability_sm            = 0x01,
  vv_capability_dis_sm        = 0x02, // maybe not needed
  vv_capability_snmp          = 0x04,
  vv_capability_dev_men       = 0x08,
  vv_capability_vendor_class  = 0x10,
  vv_capability_pkey          = 0x20,
  vv_capability_qkey          = 0x40,
  vv_capability_max           = 0x80  // this entry is for the mask macro need to be the last one in the enum
}vv_port_capability_t;

typedef u_int32_t vv_port_cap_mask_t;  // use as mask for vv_port_capability_t

#define VV_PORT_ATTR_MASK_CLR_ALL(mask)   ((mask)=0)
#define VV_PORT_ATTR_MASK_SET_ALL(mask)   ((mask)=(vv_capability_max-1))
#define VV_PORT_ATTR_MASK_SET(mask,attr)  ((mask)|=(attr))
#define VV_PORT_ATTR_MASK_CLR(mask,attr)  ((mask)&=(~(attr)))
#define VV_PORT_ATTR_IS_SET(mask,attr)    (((mask)&(attr))!=0)


// Atomic Operation Support
typedef enum
{
  vv_atom_not,    // Atomic not supported
  vv_atom_hca_qp, // Atomic supported for this HCA
  vv_atom_system  // Atomic supported for this system
}vv_atom_support_t;

// for use in create QP
// Send queue completion notification type (p. 487)
// in Intel its BOOL
// use this during create QP
// the same order as in IBM
typedef enum
{
  vv_signal_all,
  vv_selectable_signaling
}vv_completion_notification_t;


// Completion notification request (p. 535)
// For use in vv_request_completion_notification()
// not the same as in mlx - do not change (translation array)
// IBM translation array
typedef enum
{
  vv_next_solicit_event_only,
  vv_next_solicit_unsolicit_event
}vv_request_comp_notify_t;


//p. 422
// the same as in IBM
typedef enum
{
  vv_state_migrate,
  vv_state_rearm,
  vv_state_armed
}vv_path_migrate_state_t;

// Send Queue Drain state
typedef enum
{
  vv_state_draining,
  vv_state_drained
}vv_sqd_state_t;

// WR completion status for error notification
// have some more type (p. 540, 532, 533)
// YS - NOT like mlx so do not change it (will brake mlx adaptor)
// IBM translation array
typedef enum
{
  vv_comp_status_success,              //(0) 
  vv_comp_status_loc_len_err,          //(1)
  vv_comp_status_loc_qp_op_err,        //(2)
  vv_comp_status_loc_eec_op_err,       //(3)
  vv_comp_status_loc_portection_err,   //(4)
  vv_comp_status_wr_flush_err,         //(5)
  vv_comp_status_mw_bind_err,          //(6)
  vv_comp_status_rem_inv_req_err,      //(7)
  vv_comp_status_rem_access_err,       //(8)
  vv_comp_status_rem_op_err,           //(9)
  vv_comp_status_retry_exc_err,        //(10)
  vv_comp_status_rnr_retry_exc_err,    //(11)
  vv_comp_status_rem_inv_rd_req_err,   //(12)
  vv_comp_status_rem_aborted_err,      //(13)
  vv_comp_status_inv_eec_num_err,      //(14)
  vv_comp_status_inv_eec_state_err,    //(15)
  
  // not in the spec.
  vv_comp_status_timeout_err,          //(16) // YSBUGBUG - need to be remove - the same as vv_comp_status_retry_exc_err
  vv_comp_status_rnr_timeout_err,      //(17) // YSBUGBUG - need to be remove - the same as vv_comp_status_rnr_retry_exc_err
  vv_comp_status_general_err,          //(18)

  vv_comp_status_loc_access_err,       //(19) // not in the spec. for MLX
  vv_comp_status_loc_rdd_vio_err,      //(20) // not in the spec. for MLX
  vv_comp_status_bad_resp_err          //(21) // not in the spec. for MLX
}vv_comp_status_t;


// this parameter is for the work request operation type for the adaptor layer
typedef enum
{
  vv_operation_type_send_rc,  // (0)
  vv_operation_type_send_uc,  // (1)
  vv_operation_type_send_rd,  // (2)
  vv_operation_type_send_ud,  // (3)
  vv_operation_type_send_raw, // (4)
  vv_operation_type_receive,  // (5)
  vv_operation_type_bind      // (6)
}vv_wr_operation_type_t;

// 
typedef enum
{
    vv_mem_mod_transl  = 0x01,
    vv_mem_mod_protect = 0x02,
    vv_mem_mod_access  = 0x04
}vv_modif_mem_opt_t;

// Access control combination as noted (p. 511)
// this is also for modify QP RDMA access control (p. 492)
// several access control can be given
// changed this according mlx for now they are the same!!!
// we are the same as IBM
// do not change
typedef enum
{
    vv_acc_l_mem_write = 0x01, // Enable local write
    vv_acc_r_mem_write = 0x02, // Enable remote write, require local write - also for RDMA Write operations (p. 492) (Reliable service only)
    vv_acc_r_mem_read  = 0x04, // Enable remote read, no requirement - also for RDMA Read operations (p. 492) (RC, RD, UC) 
    vv_acc_r_mem_atom  = 0x08, // Enable remote atomic access, require atomic support and local write - Atomic operation (p. 492) (Reliable service only)
    vv_acc_mem_bind    = 0x10, // Enable memory windows bind, no requirements
    vv_acc_max         = 0x20  // this entry is for the mask macro need to be the last one in the enum
}vv_access_con_t;

typedef u_int32_t vv_access_con_bit_mask_t;  // use as mask for vv_access_con_t

#define VV_ACCESS_CONTROL_MASK_CLR_ALL(mask)   ((mask)=0)
#define VV_ACCESS_CONTROL_MASK_SET_ALL(mask)   ((mask)=(vv_acc_max-1))
#define VV_ACCESS_CONTROL_MASK_SET(mask,attr)  ((mask)|=(attr))
#define VV_ACCESS_CONTROL_MASK_CLR(mask,attr)  ((mask)&=(~(attr)))
#define VV_ACCESS_CONTROL_IS_SET(mask,attr)    (((mask)&(attr))!=0)


// Memory region or window types
typedef enum
{
    vv_mr, 
    vv_mw, 
    vv_mpr, 
    vv_shar 
}vv_mr_win_type_t;

// in Intel and IBM it's different (p.17)
// p. 542
// DO NOT CHANGE
// mlx - translation array
// IBM - translation array
typedef enum
{
  vv_async_r_path_migrated_ev     = 0x0001, //qp
  vv_async_r_com_est_ev           = 0x0002, //qp - communication established
  vv_async_r_send_q_drained_ev    = 0x0004, //qp  
  vv_async_r_l_wq_err             = 0x0008, //qp
  vv_async_r_path_migrated_err    = 0x0010, //qp
  vv_async_r_eec_path_migrated_ev = 0x0020, //eec
  vv_async_r_eec_com_est_ev       = 0x0040, //eec - communication established
  vv_async_r_l_eec_err            = 0x0080, //eec
  vv_async_r_cq_err               = 0x0100, //cq
  vv_async_r_l_err                = 0x0200, //hca
  vv_async_r_port                 = 0x0400, //hca - port
  vv_async_r_port_link            = 0x0800, //port
  vv_async_r_port_mtu             = 0x1000, //port
  vv_async_r_port_lid             = 0x2000  //port
#ifdef VV_SLEEPABLE_ASYNC_EVENTS
  ,
  vv_async_r_port_pkey_in         = 0x4000, //port
  vv_async_r_port_pkey_out        = 0x8000, //port
  vv_async_r_port_link_up         = 0x10000,//port
  vv_async_r_port_link_down       = 0x20000,//port
  vv_async_r_port_sm_change       = 0x40000, //port  
  vv_async_r_gid_in               = 0x80000, 
  vv_async_r_gid_out              = 0x100000, 
  vv_async_r_mc_group_create      = 0x200000, 
  vv_async_r_mc_group_delete      = 0x400000 
#endif  
}vv_async_record_t;

#define VV_ASYNC_R_CQ_MASK         (0x100) // only one value
#define VV_ASYNC_R_QP_MASK         (0x1F)
#define VV_ASYNC_R_EEC_MASK        (0x1C0)
#define VV_ASYNC_R_HCA_MASK        (0x600)
#define VV_ASYNC_R_PORT_MASK       (0x3F800)
#define VV_ASYNC_R_ALL_MASK        (0x3FFFF)


// Type for registrating an async event callback for vverbs users
typedef enum
{
    vv_async_even_all_qp  = 0x001,
    vv_async_even_all_cq  = 0x002,
    vv_async_even_all_eec = 0x004,
    vv_async_even_all_hca = 0x008, // this is also for port
    vv_async_even_all     = 0x010,
    vv_async_even_qp      = 0x020,
    vv_async_even_cq      = 0x040,
    vv_async_even_eec     = 0x080,
    vv_async_even_hca     = 0x100,
    vv_async_even_port    = 0x200  // registered for all local ports events
}vv_request_async_record_t;

#define VV_ASYNC_EVENT_ALL_MASK      (0x01F)
#define VV_ASYNC_EVENT_HCA_PORT_MASK (0x300)


// RNR-NAK timer incoding (p. 297)
// this is the delay time (ms) before resending after requestor receive RNR-NAK
typedef enum
{
    vv_rnr_nak_time_delay_655_36 = 0, // (0)
    vv_rnr_nak_time_delay_0_01,       // (1)
    vv_rnr_nak_time_delay_0_02,       // (2)
    vv_rnr_nak_time_delay_0_03,       // (3)
    vv_rnr_nak_time_delay_0_04,       // (4)
    vv_rnr_nak_time_delay_0_06,       // (5)
    vv_rnr_nak_time_delay_0_08,       // (6)
    vv_rnr_nak_time_delay_0_12,       // (7)
    vv_rnr_nak_time_delay_0_16,       // (8)
    vv_rnr_nak_time_delay_0_24,       // (9)
    vv_rnr_nak_time_delay_0_32,       // (10)
    vv_rnr_nak_time_delay_0_48,       // (11)
    vv_rnr_nak_time_delay_0_64,       // (12)
    vv_rnr_nak_time_delay_0_96,       // (13)
    vv_rnr_nak_time_delay_1_28,       // (14)
    vv_rnr_nak_time_delay_1_92,       // (15)
    vv_rnr_nak_time_delay_2_56,       // (16)
    vv_rnr_nak_time_delay_3_84,       // (17)
    vv_rnr_nak_time_delay_5_12,       // (18)
    vv_rnr_nak_time_delay_7_68,       // (19)
    vv_rnr_nak_time_delay_10_24,      // (20)
    vv_rnr_nak_time_delay_15_36,      // (21)
    vv_rnr_nak_time_delay_20_48,      // (22)
    vv_rnr_nak_time_delay_30_72,      // (23)
    vv_rnr_nak_time_delay_40_96,      // (24)
    vv_rnr_nak_time_delay_61_44,      // (25)
    vv_rnr_nak_time_delay_81_92,      // (26)
    vv_rnr_nak_time_delay_122_88,     // (27)
    vv_rnr_nak_time_delay_163_84,     // (28)
    vv_rnr_nak_time_delay_245_76,     // (29)
    vv_rnr_nak_time_delay_327_68,     // (30)
    vv_rnr_nak_time_delay_491_52      // (31)
}vv_rnr_nak_time_delay_t;


////////////////////////////////////////////////////////////////////////
// struct definitions

typedef struct vv_hca_attrib_st            vv_hca_attrib_t;     // HCA attributes
typedef struct vv_hca_vendor_attrib_st     vv_hca_vendor_attrib_t;
typedef struct vv_sgid_table_st            vv_sgid_table_t;     // port source gid table (for every port)
typedef struct vv_parti_table_st           vv_parti_table_t;    // port partition table (for every port)
typedef struct vv_sm_address_info_st       vv_sm_address_info_t;//information about the SM
typedef struct vv_port_attrib_st           vv_port_attrib_t;    // port attributes (for every port)
typedef struct vv_add_vec_st               vv_add_vec_t;        //
typedef struct vv_add_vec_global_d_st      vv_add_vec_global_d_t;
typedef struct vv_alternate_path_st        vv_alternate_path_t;
typedef struct vv_eec_attr_st              vv_eec_attr_t;
typedef struct vv_atomic_operands_st       vv_atomic_operands_t;
typedef union  vv_qp_attr_st               vv_qp_attr_t;        //
typedef struct vv_qp_modify_sqe_rts_opt_st vv_qp_modify_sqe_rts_opt_t;
typedef struct vv_qp_modify_sqd_rts_opt_st vv_qp_modify_sqd_rts_opt_t;
typedef struct vv_scatgat_st               vv_scatgat_t;        //
typedef struct vv_sg_list_st               vv_sg_list_t;
typedef struct vv_phy_buf_st               vv_phy_buf_t;        // As in mlx and the spec. (P. 512)
typedef struct vv_phy_list_st              vv_phy_list_t;       // As in mlx and the spec. (P. 512)
typedef struct vv_wr_st                    vv_wr_t;             //
typedef struct vv_datag_addr_info_st       vv_datag_addr_info_t;
typedef struct vv_wc_st                    vv_wc_t;             //in VIEO
typedef struct vv_event_record_st          vv_event_record_t;
typedef struct vv_request_event_record_st  vv_request_event_record_t;

typedef struct vv_fmr_st                   vv_fmr_t;
typedef struct vv_fmr_map_st               vv_fmr_map_t;

//typedef struct vv_intel_mem_w_bind_opt_st  vv_intel_mem_w_bind_opt_t; // for Intel use only
//typedef struct vv_intel_event_record_st    vv_intel_event_record_t;   // for Intel use only

//typedef struct vv_hca_h_con_st             vv_hca_h_con_t;      // HCA handle and context // cancel
//typedef struct vv_cq_h_con_st              vv_cq_h_con_t;       // CQ handle and context  // cancel
//typedef struct vv_qp_h_con_st              vv_qp_h_con_t;       // QP handle and context  // cancel


////////////////////////////////////////////////////////////////////////
// struct implementation

// Vendor specific information (p. 477)
struct vv_hca_vendor_attrib_st
{
    u_int32_t vendor_id;
    u_int32_t part_id;
    u_int32_t hw_version;
};

// HCA attributes (p. 477)
struct vv_hca_attrib_st
{
  vv_hca_vendor_attrib_t  vendor_attrib;                    // vendor specific attrib
  u_int32_t               max_qp_num;                       // max num QP supported
  u_int32_t               max_outstand_wr_on_qp;            // max num outstanding work requests on every QP
  u_int32_t               max_scatgat_per_wr;               // For all except RD
  u_int32_t               max_rd_scatgat_per_wr;            // For RD only, 0 if not supported
  u_int32_t               max_cq_num;                       // max num CQ supported
  u_int32_t               max_cq_entries;                   // max entries for every CQ
  u_int32_t               max_mem_region_num;               // max num Memory Region supported
  u_int64_t               max_block_size_byte;              // largest contiguous block that can be registered - bytes
  u_int32_t               max_pd_num;                       // max num PD supported
  u_int32_t               page_size;                        // memory page size supported
  vv_port_num_t           port_num;                         // number of ports and size of the port_attrib_list, Used also in VAPI_modify_hca_attr as the port num to modify
  vv_port_attrib_t       *port_attrib_list;                 // pointer into vv_port_attrib_t array, use this in: vvi_mlx_hca_modify??
  //u_int32_t               max_vl_num;                       // max num VL supported (p. 478) for HCA or port???
  u_int32_t               max_partition_num;                // max num partitions supported
  vv_guid_t               guid;                             // HCA GUID
  u_int32_t               ack_delay;                        // Log2 4.096usec max time between receive message and transmit ACK/NAK
  VV_BOOL_T               bad_p_key_count_f;                // indicator
  u_int32_t               bad_p_key_count;                  // Counter
  VV_BOOL_T               bad_q_key_count_f;                // indicator
  u_int32_t               bad_q_key_count;                  // Counter
  u_int32_t               max_read_atom_qp_outstanding;     // max num RDMA read and atomic operation outstanding per QP when this HCA is the target
  u_int32_t               max_read_atom_eec_outstanding;    // max num RDMA read and atomic operation outstanding per EEC when this HCA is the target  
  u_int32_t               max_read_atom_resources;          // max num of recourses for RDMA read and atomic operation when this HCA is the target
  u_int32_t               max_qp_depth_for_init_read_atom;  // max num RDMA read and atomic operation outstanding per QP when this HCA is the initiator
  u_int32_t               max_eec_depth_for_init_read_atom; // max num RDMA read and atomic operation outstanding per QP when this HCA is the initiator
  vv_atom_support_t       atom_suport;                      // support for atomic operation (p. 479)
  u_int32_t               max_eec_num;                      // max num EEC supported
  u_int32_t               max_rdd_num;                      // max num RDD supported (if support RD then >= 2)
  u_int32_t               max_mem_win_num;                  // max num Memory Windows supported
  u_int32_t               max_raw_ipv6_qp_num;
  u_int32_t               max_raw_eter_qp_num;
  VV_BOOL_T               support_modify_outstand_wr;       // support for modify the max num of outstanding WR
  u_int32_t               max_multicast_group;              // max num Multicast groups supported (0 if not supported)
  u_int32_t               max_qp_num_for_all_multicast_groups;
  u_int32_t               max_qp_num_per_multicast_group;
  VV_BOOL_T               support_raw_multicast;            // support for row multicast
  VV_BOOL_T               support_path_migration;           // support for path migrate
  u_int32_t               max_add_handle;                   // max num Address Handles supported
  VV_BOOL_T               ability_change_prim_phy_port;

  u_int32_t               max_fmr_num;                      // max fmrs number
  u_int32_t               max_maps_per_fmr;                 // max maps per fmr

  // BUGBUG - hca application context is NOT supported yet by the adaptor
  //vv_hca_context_t        user_context;  // will hold the user context in the adaptor (query, modify HCA attributes)
};

// SM address info
struct vv_sm_address_info_st
{
  vv_port_num_t      port_num;     // for use by HCA query for every port, for port attrib we don't use it
  vv_lid_t           sm_lid;
  vv_service_level_t service_level;
};

// Port Attributes
struct vv_port_attrib_st
{
  vv_port_num_t         port_num;
  vv_mtu_t              port_mtu;
  u_int32_t             port_message_size;     // Max massage size supported 
  vv_port_state_t       port_state;
  vv_lid_t              port_base_lid;
  u_int32_t             port_lmc;              // The port LMC specifies how many bits come from the AV Path bit
  //vv_sgid_table_t*      port_sgid_table;       // port source GID table - for use by vv_get_port_gid_tbl()
  u_int32_t             port_sgid_table_size;  // the number of element that port_sgid_table point to
  //vv_parti_table_t*     port_p_table;          // port partition table - for use by vv_get_port_partition_tbl()
  u_int32_t             port_parti_table_size; // the number of element that port_p_table point to
  u_int8_t              port_max_vl_num;       // for every HCA (p. 478) for port
  u_int32_t             port_bad_p_key_count;
  u_int32_t             port_bad_q_key_count;
  vv_sm_address_info_t  port_sma_address_info;
  vv_port_cap_mask_t    port_capability_support;  
};

// Address Vector global destination
// use by UD QP only
// for RC UC (p. 497)
struct vv_add_vec_global_d_st
{                                                     
  u_int32_t  flow_lable;                              // UD  // 20 bits
  u_int8_t   hope_limit;                              // UD  // 8 bits
  u_int32_t  traffic_class;                           // UD  // 8 bits
  u_int32_t  s_gid_index;                             // UD
  vv_gid_t   d_gid;                                   // UD  // for global destination or multicast           
};

// Address Vector - use by UD QP only (P. 484)
// This for Address management Verbs (Create, Modify, Query and Destroy)
// using this struct also in Modify QP RTR, RTS - RC, UC (P. 493) - need to fill all param and appropriate bit mask
// if using this in the alternate path parameters need to fill all the parameters (even if duplicate of primary path) and 
struct vv_add_vec_st
{                                         //               bit
                                          //               flag
  u_int8_t               phy_port_num;    //                    // port number (1,2), for UD QP port enforce only, Intel got GUID (not using this as the port for alternate path)
  vv_service_level_t     service_level;   // RC, UC   // UD     // 4 bits
  VV_BOOL_T              grh_flag;        // RC, UC   // UD
  vv_lid_t               dlid;            // RC, UC   // UD     // final destination or router LID
  vv_mtu_t               pmtu;            // RC, UC   //     x  // this MTU is for primery only (alternate must be equal to primary)
  u_int8_t               max_static_rate; // RC, UC   // UD     // 6 bits
  u_int32_t              l_ack_timeout;   // RC       //     x  // Log2 4.096usec as in vv_hca_attrib_t (YS - RTS have duplicate field - need to be the same)
  u_int32_t              retry_count;     // RC       //     x
  u_int32_t              rnr_retry_count; // RC       //     x  // if rnr-nak time == 7 will do infinit retrys (p. 365)
  u_int32_t              source_path_bit; // RC, UC   // UD     // 7 bits
  vv_add_vec_global_d_t  global_dest;     // RC, UC   //
};


// Alterate Path valid only for RC UC
// when you are using alternate path you must fill the p_key and port number and all other
// parameters in the address vector that are relevant - do not fill the phy_port_num in the add vec 
struct vv_alternate_path_st
{
  vv_p_key_t   al_path_p_key_indx; // RC, UC - must fill parmeter
  u_int8_t     al_path_port_num;   //        - must fill parmeter
  vv_add_vec_t al_path_add_vec;
};

// EEC attribute
struct vv_eec_attr_st
{
  u_int8_t                  primary_path_phy_port;
  vv_p_key_t                primary_path_p_key_index;
  vv_eec_state_t            eec_state;
  vv_psn_t                  send_psn;                  
  vv_psn_t                  receve_psn;
  u_int32_t                 read_atom_eec_resources;
  u_int32_t                 destanation_eec;
  vv_add_vec_t              add_vec;
  vv_alternate_path_t       alternate_path;
};

// Atomic operation operands
struct vv_atomic_operands_st
{
  u_int64_t  atom_first_operand;  // aligned on a 64-bit boundary. compare against or add to operand.
  u_int64_t  atom_second_operand; // aligned on a 64-bit boundary. replace the value for Compare & Swap operation.
  u_int64_t  atom_return_operand; // remote original value
};


// QP attribute flags for modify QP state (maybe will be added later into query QP also)
// MLX have differnts flags
// If changing this need to change also in the vv-mlx-common.h

#define  VV_QP_AT_NUM                    (0x00000001) // 1 
#define  VV_QP_AT_TYPE                   (0x00000002) // 2
#define  VV_QP_AT_STATE                  (0x00000004) // 3
#define  VV_QP_AT_CQ_S_H                 (0x00000008) // 4
#define  VV_QP_AT_CQ_R_H                 (0x00000010) // 5
#define  VV_QP_AT_S_MAX_OUT_WR           (0x00000020) // 6
#define  VV_QP_AT_R_MAX_OUT_WR           (0x00000040) // 7
#define  VV_QP_AT_MAX_SG_S_WR            (0x00000080) // 8
#define  VV_QP_AT_MAX_SG_R_WR            (0x00000100) // 9
#define  VV_QP_AT_S_PSN                  (0x00000200) // 10
#define  VV_QP_AT_R_PSN                  (0x00000400) // 11
#define  VV_QP_AT_ACCESS_CON_F           (0x00000800) // 12
#define  VV_QP_AT_PHY_PORT_NUM           (0x00001000) // 13
#define  VV_QP_AT_P_KEY_IX               (0x00002000) // 14
#define  VV_QP_AT_Q_KEY                  (0x00004000) // 15
#define  VV_QP_AT_RDD_H                  (0x00008000) // 16
#define  VV_QP_AT_DEST_QP                (0x00010000) // 17
#define  VV_QP_AT_DEST_RDMA_ATOM_OUT_NUM (0x00020000) // 18 remote parameter RTS
#define  VV_QP_AT_RESP_RDMA_ATOM_OUT_NUM (0x00040000) // 19 local parameter  RTR
#define  VV_QP_AT_MIN_RNR_NAK_T          (0x00080000) // 20
#define  VV_QP_AT_ADD_VEC                (0x00100000) // 21
#define  VV_QP_AT_PATH_MIG_STATE         (0x00200000) // 22
#define  VV_QP_AT_ALT_PATH               (0x00400000) // 23 maybe will need to add all the differnet alt parameters
#define  VV_QP_AT_SIG_TYPE               (0x00800000) // 24
#define  VV_QP_AT_L_ACK_T                (0x01000000) // 25
#define  VV_QP_AT_RETRY_NUM              (0x02000000) // 26
#define  VV_QP_AT_RNR_NUM                (0x04000000) // 27
#define  VV_QP_AT_SQD_NOTIFY             (0x08000000) // 28
#define  VV_QP_AT_OP_F                   (0x10000000) // 29 this is only for saving "if" in the adaptors
#define  VV_QP_AT_MAX_FLAG               (0x20000000) // 30 need to be the last one - any add value need to be before this one

// need to see what we will do with Intel - got more types
//VV_QP_AT_PD_H - Intel
//VV_QP_AT_RECEIVE_SOLI_EV - Intel
//VV_QP_AT_SEND_SIG_COMP - Intel
//VV_QP_AT_FLOW_CTR - Intel

#define  VV_QP_AT_NUM_OF_BITS_IN_MASK    (30)                 // if adding more value need to change this
#define  VV_QP_AT_NOT_USE_BITS           (0xFFFFFFFF40000000) // if adding more value need to change this

typedef u_int64_t vv_qp_at_mask_t;  // use as mask for attribute for the change QP state

#define VV_QP_AT_MASK_CLR_ALL(mask) ((mask)=0)
#define VV_QP_AT_MASK_SET_ALL(mask) ((mask)=(VV_QP_AT_MAX_FLAG-1))
#define VV_QP_AT_SET(mask,attr)     ((mask)|=(attr))
#define VV_QP_AT_CLR(mask,attr)     ((mask)&=(~(attr)))
#define VV_QP_AT_IS_SET(mask,attr)  (((mask)&(attr))!=0)



// SQE optional parameters
struct vv_qp_modify_sqe_rts_opt_st
{
  vv_qp_at_mask_t          opt_flags;
  vv_access_con_bit_mask_t opt_access_control;    // this is for access control for RDMA / Atomic operation flags
  vv_q_key_t               opt_q_key;
  u_int32_t                opt_send_max_outstand_wr;    // text (P.492) - Number of WQEs that in table 78
  u_int32_t                opt_receive_max_outstand_wr; // text (P.492) - Number of WQEs that in table 78
  vv_rnr_nak_time_delay_t  opt_min_rnr_nak_timer; // parameter that will be send from the responder to the requestor:
                                                  // Minimum RNR NAK timer to wait when no posted receive WQE (p. 493, 297)
};

// SQD optional parameters
struct vv_qp_modify_sqd_rts_opt_st
{
  vv_qp_at_mask_t         opt_flags;
  vv_add_vec_t            opt_remote_add_vec;            // connected only
  vv_alternate_path_t     opt_alternate_path;            // alternate path for RC UC  (p. 491)
  vv_path_migrate_state_t opt_path_migrate_state;
  u_int32_t               opt_local_rdma_r_atom_num;
  u_int32_t               opt_responder_rdma_r_atom_num;
  vv_q_key_t              opt_q_key;
  vv_p_key_t              opt_p_key_indx;
  u_int32_t               opt_timeout_retry_info;
  u_int32_t               opt_send_max_outstand_wr;      // text (P.492) - Number of WQEs that in table 78
  u_int32_t               opt_receive_max_outstand_wr;   // text (P.492) - Number of WQEs that in table 78
  vv_rnr_nak_time_delay_t opt_min_rnr_nak_timer;         // parameter that will be send from the responder to the requestor:
                                                         // Minimum RNR NAK timer to wait when no posted receive WQE (p. 493, 297)
  u_int8_t                opt_phy_port_num;              // RC
};

// QP attributes for the different operation
union vv_qp_attr_st
{
  struct vv_qp_create_st  // must fill all when creating QP (p. 486)
  {
    // from create QP    
    vv_qp_type_t                 qp_type;
    vv_cq_h_t                    cq_send_h;
    vv_cq_h_t                    cq_receive_h;
    u_int32_t                    send_max_outstand_wr;       // The maximum number of Work Queue Entries (WQEs) on Send Queue
    u_int32_t                    receive_max_outstand_wr;    // The maximum number of Work Queue Entries (WQEs) on Receive Queue
    u_int32_t                    max_scatgat_per_send_wr;    // can't be larger then the value in the HCA
    u_int32_t                    max_scatgat_per_receive_wr; // can't be larger then the value in the HCA
    vv_rdd_h_t                   rdd_h;                      // RD QP only
    vv_completion_notification_t signaling_type;             // for send queue, this is BOOL in Intel QP
    vv_pd_h_t                    pd_h;
    vv_completion_notification_t recv_solicited_events;      // for Intel  there its BOOL
  } create;

  //OUT  parameter
  struct vv_qp_creat_ret_st
  {
    u_int32_t                    qp_num;                     // 
    u_int32_t                    send_max_outstand_wr;       // The maximum number of Work Queue Entries (WQEs) on Send Queue
    u_int32_t                    receive_max_outstand_wr;    // The maximum number of Work Queue Entries (WQEs) on Receive Queue
    u_int32_t                    max_scatgat_per_send_wr;    // can't be larger then the value in the HCA
    u_int32_t                    max_scatgat_per_receive_wr; // can't be larger then the value in the HCA
	
#ifdef VV_FALCON	// RONY for fulcon
    u_int32_t                    uar_index;                  // UAR of the QP
    void*                        uar_base;                   // UAR base_addres
    void*                        r_wqes_buf;                 // rx wq buf area
    u_int32_t                    r_wqes_buf_sz;              // rx wq buf size
    void*                        s_wqes_buf;                 // sebd wq buf area
    u_int32_t                    s_wqes_buf_sz;              // send wq buf size
#endif // VV_FALCON
  } create_return;

  struct vv_qp_query_st
  {
    vv_qp_at_mask_t          vv_qp_attr_mask;            // vv specific attribute mask - need to be filed by the user for every attribute
    u_int32_t                qp_num;                     // 
    vv_qp_type_t             qp_type;                    // not supported by all vendors
    vv_qp_state_t            qp_state;
    vv_cq_h_t                cq_send_h;
    vv_cq_h_t                cq_receive_h;
    u_int32_t                send_max_outstand_wr;
    u_int32_t                receive_max_outstand_wr;
    u_int32_t                max_scatgat_per_send_wr;    // can't be larger then the HCA value
    u_int32_t                max_scatgat_per_receive_wr; // can't be larger then the HCA value

    // the following are not define in Reset QP state (p. 496)
    vv_psn_t                 send_psn;                   // RC, UC
    vv_psn_t                 receve_psn;                 // RC, UC
    vv_access_con_bit_mask_t access_control;             // access control for RDMA / Atomic operation flags
    u_int8_t                 phy_port_num;               // not for RD, in Intel its GUID
    vv_p_key_t               primary_p_key_indx;         // not for RD and raw datagram    
    vv_q_key_t               q_key;                      // not for RD, raw
    vv_rdd_h_t               rdd_h;                      // RD QP only
    vv_qp_t                  destanation_qp;             // RC, UC
    u_int32_t                rdma_r_atom_outstand_num;   // RC  - for remote side
    u_int32_t                responder_rdma_r_atom_num;  // RC  - for local side
    vv_rnr_nak_time_delay_t  min_rnr_nak_timer;          // parameter that will be send from the responder to the requestor:
                                                         // Minimum RNR NAK timer to wait when no posted receive WQE (p. 493, 297)
    vv_add_vec_t             primary_add_vec;            // RC, UC
    vv_path_migrate_state_t  path_migrate_state;

    vv_alternate_path_t      alternate_path;             // alternate path for RC UC  (p. 497)

    u_int32_t                max_send_inline_data;       // max inline data for the post inline send

    vv_pd_h_t                pd_h;                       // Intel implementation only!
    VV_BOOL_T                recv_solicited_events;      // from Intel
    VV_BOOL_T                send_signaled_comp;         // from Intel
    VV_BOOL_T                flow_control;               // from Intel
  } query;
  
  struct vv_qp_modify_st
  {
    vv_qp_state_t    qp_modify_into_state; // 
    vv_qp_at_mask_t  vv_qp_attr_mask;      // vv specific attribute mask - need to be filed by the user for every attribute
    vv_qp_type_t     qp_type;              // not all parameters during modify are valid for the different QP types (see below)
                                           // for now i'm not using this for now but maybe the adaptor layer can use it later to save some "if"
    union operation
    {
      struct vv_qp_modify_init_st                             // RC RD UC UD
      {                                                       //-------------
        vv_p_key_t               p_key_indx;                  // x     x  x
        u_int8_t                 phy_port_num;                // x     x  x  If there is a conflict between this and the AV port number in modify to RTR this parameter is the correct one
        vv_q_key_t               q_key;                       //    x     x  validate a remote sender's right to access a local Receive Queue. (also in WR)
        vv_access_con_bit_mask_t access_control;              // x  x (x)    this is for access control for RDMA / Atomic operation flags
                                                              //             in IBM this is the only place that specify if RDMA is supported (p. 105)
                                                              //             only RDMA Write can be done also on UC - this is the reason for the ()
      } init;

      struct vv_qp_modify_rtr_st                              // RC RD UC UD
      {                                                       //-------------
        vv_add_vec_t             remote_add_vec;              // x     x
        vv_qp_t                  destanation_qp;              // x     x
        vv_psn_t                 receive_psn;                 // x     x
        u_int32_t                responder_rdma_r_atom_num;   // x           local QP parameter
        
        // if you are using on of the optinal parameters don't forget to fill the opt bit in the mask
        vv_access_con_bit_mask_t opt_access_control;          // x  x (x)    this is for access control for RDMA / Atomic operation flags (see remark ebove)
        vv_alternate_path_t      opt_alternate_path;          // x     x
        vv_q_key_t               opt_q_key;                   //    x     x
        vv_p_key_t               opt_p_key_indx;              // x     x  x
        u_int32_t                opt_send_max_outstand_wr;    // x  x  x  x  text (P.492) - Number of WQEs that in table 78 - if resize is supported
        u_int32_t                opt_receive_max_outstand_wr; // x  x  x  x  text (P.492) - Number of WQEs that in table 78 - if resize is supported
        vv_rnr_nak_time_delay_t  opt_min_rnr_nak_timer;       // x  x        parameter that will be send from the responder to the requestor:
                                                              //             Minimum RNR NAK timer to wait when no posted receive WQE (p. 493, 297)                                                        
      } rtr;
      
	  struct vv_qp_modify_rts_st                              // RC RD UC UD
      {                                                       //-------------
        u_int32_t                local_ack_timeout;           // x           YS - the same parameter is also in the address vector - can have conflicts
        u_int32_t                retry_num;                   // x
        u_int32_t                rnr_num;                     // x           rnr-nak num == 7 will do infinit retrys (p. 365)
        vv_psn_t                 send_psn;                    // x     x  x
        u_int32_t                dest_out_rdma_r_atom_num;    // x           Number of outstanding RDMA Read / Atomic ops at destination (p. 492) - remote parameter
        VV_BOOL_T                flow_control;                //             from Intel - dose destination provide this endpoint with end-to-end flow control credits

        // if you are using on of the optinal parameters don't forget to fill the opt bit in the mask
        vv_access_con_bit_mask_t opt_access_control;          // x  x (x)    this is for access control for RDMA / Atomic operation flags (see remark ebove)
        vv_q_key_t               opt_q_key;                   //    x     x
        vv_alternate_path_t      opt_alternate_path;          // x     x
        vv_path_migrate_state_t  opt_path_migrate_state;      //             RC ?
        u_int32_t                opt_send_max_outstand_wr;    // x  x  x  x  text (P.492) - Number of WQEs that in table 78 - if resize is supported
        u_int32_t                opt_receive_max_outstand_wr; // x  x  x  x  text (P.492) - Number of WQEs that in table 78 - if resize is supported
        vv_rnr_nak_time_delay_t  opt_min_rnr_nak_timer;       // x  x        parameter that will be send from the responder to the requestor:
                                                              //             Minimum RNR NAK timer to wait when no posted receive WQE (p. 493, 297)       

        VV_BOOL_T        sqd_state;            // this is for requesting sqd asynchronous notification when it finished sending the send queue drained
                                               // when i find implementation for this I will move it or to the rts or to new rts_to_sqd
      } rts;

      // if you are using on of the optinal parameters don't forget to fill the opt bit in the mask
      vv_qp_modify_sqe_rts_opt_t  sqe_to_rts;  //
      
      vv_qp_modify_sqd_rts_opt_t  sqd_to_rts;  //

    } params;
  } modify;

  //OUT  parameter
  struct vv_qp_modify_ret_st
  {
    u_int32_t               send_max_outstand_wr;
    u_int32_t               receive_max_outstand_wr;
  } modify_return;
};

// Scatter Gather entity (implemented as array) 
// not the same as in MLX
struct vv_scatgat_st
{
  u_int64_t   v_address;  
  u_int32_t   length;    // length in bytes
  vv_l_key_t  l_key;
};

// scatter gather List
// can be use by vverbs users
struct vv_sg_list_st
{
    u_int8_t      num_of_sg;
    u_int64_t     list_nem_size;    // total length of the scatter gather list
    vv_scatgat_t *sg_list;       
};

// Physical memory buffer
// Will be hold in array not a list
// the same as in MLX !!! do not change
struct vv_phy_buf_st
{
  vv_phy_addr_t     start;
  vv_phy_addr_t     size;
};

// Physical Buffer List (P. 512)
struct vv_phy_list_st
{
    u_int8_t      number_of_buff;
    vv_phy_buf_t *phy_list;       //will be hold in array of vv_phy_buf_st not a list
};

// Work Request structure for all operation send bind and receive
struct vv_wr_st
{
  vv_wr_id_t            wr_id;                   // can be use as self pointer or to wrapper struct
  VV_BOOL_T             completion_notification; // signal or not for this WR, valid only if the QP is set for selectable signaling (p. 528)
  vv_scatgat_t*         scatgat_list;            // for raw IPv6 first 40 byte = IP header (p. 528)
  u_int32_t             num_of_data_segments;    // number of elements in the scatgat list
  void*                 user_define_p;           // for user use (skbuf...) // BUGBUG - need to remove this later - the application will build struct with wr and this pointer
  vv_wr_operation_t     wr_type;
  union operation_type
  {
    struct vv_send_wr_st
    {
      VV_BOOL_T immidiate_data_indicator;        // Send / RDMA write
      u_int32_t immidiate_data;                  // Send / RDMA write
      VV_BOOL_T solicited_event;                 // Send / RDMA write
      union qp_type
      {        
        struct vv_rc_type_st
        {
          VV_BOOL_T            fance_indicator;  // for RC, RD
          u_int64_t            r_addr;           // for RDMA and Atomic
          vv_r_key_t           r_r_key;          // for RDMA and Atomic
          VV_BOOL_T            atomic;           // for Atomic
          vv_atomic_operands_t atomic_operands;  // for Atomic
        } rc_type;

        struct vv_uc_type_st
        {
          u_int64_t            r_addr;           // for RDMA and Atomic
          vv_r_key_t           r_r_key;          // for RDMA and Atomic
          VV_BOOL_T            atomic;           // for Atomic
          vv_atomic_operands_t atomic_operands;  // for Atomic
        } uc_type;

        struct vv_rd_type_st
        {
          VV_BOOL_T  fance_indicator;            // for RC, RD
          u_int64_t  r_addr;                     // for RDMA and Atomic
          vv_r_key_t r_r_key;                    // for RDMA and Atomic
          vv_qp_t    r_qp_num;                   // datagram service
          vv_q_key_t r_q_key;                    // send datagram service (p. 403 - high order bit)
          u_int32_t  eec_number;                 // eec number not handle for RD only
        } rd_type;                   
        
        struct vv_ud_type_st
        {
          vv_add_h_t    r_add_h;                 // for UD address handle or raw P. 529 - need to check this like wc p.534 - for mlx
          vv_add_vec_t* p_add_vec;               // BUGBUG - for Intel UD send need to check they want handle , mlx need remote address handle
          vv_qp_t       r_qp_num;                // datagram service
          vv_q_key_t    r_q_key;                 // send datagram service (p. 403 - high order bit)
        } ud_type;        
      } send_qp_type;   

    } send;

    struct vv_send_raw_st
    {
      u_int32_t          etertype;               // for raw etertype only, need to check this - not finished 
      u_int8_t           static_rate;            // for raw etertype and raw IPv6 not need for Intel
      vv_qp_t            r_qp_num;
      vv_lid_t           dlid;                   // for Intel
      u_int32_t          source_path_bit;        // for Intel
      vv_service_level_t service_level;          // for Intel
    } send_raw;

    struct vv_receive_wr_st
    {
      u_int32_t not_used;    // only stab for now
    } receive;

    struct vv_bind_wr_st //(p. 522)
    {
      vv_mem_win_h_t           mem_win_h;
      vv_r_key_t               current_r_key;
      vv_r_key_t               new_r_key;
      vv_mem_reg_h_t           mem_region;
      vv_scatgat_t             address;
      vv_access_con_bit_mask_t opt_access_control;  // this is for access control for RDMA / Atomic operation flags
      //vv_intel_mem_w_bind_opt_t     bind_options;    // BUGBUG - for Intel
    } bind;

  } type;
};

// Datagram Address Info
struct vv_datag_addr_info_st
{
  vv_lid_t            slid;
  vv_service_level_t  sl;
  vv_qp_t             source_qp;
  u_int16_t           etertype;       // for raw etertype only
  u_int32_t           eec_number;     // for RD only
  u_int32_t           dlid_path_bits; // for UD and raw
  vv_qp_type_t        remote_qp_type; // remote QP type
};

// Work completion structure
// not all the fildes are valid for all transport and operation types
struct vv_wc_st
{
  vv_wr_id_t            wr_id;                    // always
  vv_wc_operation_t     operation_type;           // ?
  u_int64_t             num_bytes_transfered;     //        not with immediate data (p. 533-534), UD - need to add 40 byte
  VV_BOOL_T             immidiate_data_indicator; //        only for send or RDMA write
  u_int32_t             immidiate_data;           //
  vv_datag_addr_info_t  addres_info;              //        for UD only
  VV_BOOL_T             grh_indicator;            //        for UD only
  u_int32_t             p_key_index;              //        for GSI only
  vv_comp_status_t      completion_status;        // always
  u_int32_t             freed_resource;           // always RD only
};

// cancel
/*
struct vv_hca_h_con_st
{
    vv_hca_h_con_t  *next;
    vv_hca_h_con_t  *prev;

    vv_hca_h_t       hca_h;
    vv_hca_context_t hca_cont;
};

// cancel         
// 
struct vv_cq_h_con_st
{
    vv_cq_h_con_t  *next;
    vv_cq_h_con_t  *prev;

    vv_hca_h_t      hca_h;
    vv_cq_h_t       cq_h;
    vv_cq_context_t cq_cont;
};

// cancel         
//
struct vv_qp_h_con_st
{
    vv_qp_h_con_t  *next;
    vv_qp_h_con_t  *prev;

    vv_hca_h_t      hca_h;
    vv_qp_h_t       qp_h;
    vv_qp_context_t qp_cont;
};
*/


// for Intel use only 
//struct vv_intel_mem_w_bind_opt_st
//{
//  u_int16_t reserved1:            5;
//  u_int16_t fence:                1;
//  u_int16_t signaled_completion:  1;
//  u_int16_t reserved2:            9;
//};



// event record for the asynchronous events
struct vv_event_record_st
{
    vv_async_record_t  event_type;
	
    vv_hca_h_t         hca_app_h;   // This is the HCA handle that the application get from open hca
                                    // it is not the provider hca_h

    // BUGBUG - hca application context is NOT supported yet by the adaptor
    //vv_hca_context_t   hca_context;

    
    union type_ev_re_un
    {
        struct qp_st
        {
            vv_qp_h_t       qp_h;
            vv_qp_t         qp_num;
            vv_qp_context_t context;
        }qp;

        struct cq_st
        {
            vv_cq_h_t       cq_h;
            vv_cq_context_t context;
        }cq;
        
        struct eec_st
        {
            vv_eec_h_t       eec_h;
            vv_eec_context_t context;
        }eec;
        
        vv_gid_t          gid; // for notices: gid_in, gid_out, mc_group_create, group_delete

        u_int64_t         data;
    }type;
    /* Monski it was in the union below and is needed WITH the union below */
    u_int8_t          port_num; 
};


// for the application (vverbs users): to register async event callback
struct vv_request_event_record_st
{
    vv_request_async_record_t  req_event_type;
	
    union type_h_un
    {
        vv_hca_h_t hca_h;
        vv_qp_h_t  qp_h;
        vv_cq_h_t  cq_h;
        vv_eec_h_t eec_h;        
    }type;
};

struct vv_fmr_st
{
    vv_pd_h_t                pd_hndl;
    vv_access_con_bit_mask_t acl; 
    u_int32_t                max_pages;    //Maximum number of pages that can be mapped using this region (virtual mapping only)
    u_int8_t                 log2_page_sz; // Fixed page size for all maps on a given FMR
    u_int32_t                max_outstanding_maps; // Maximum maps before invoking unmap for this region
};

struct vv_fmr_map_st
{
    void*      start;          // Mapped memory virtual address
    u_int32_t  size;           // Size of memory mapped to this region
    u_int32_t  page_array_len; // If >0 then no memory locking is done and page table is taken from array below
    u_int64_t* page_array;     // Page size is set in EVAPI_alloc_fmr by log2_page_sz field (YSBUGBUG - need to check for every version)
};


///////////////////////////////////////////////////////////////////////////////////////////////////
// handler callbacks

// application can use only two callbacks

// Application completion notification callbacks (as in tasklet_struct)
typedef void (*vv_app_com_event_callback_t)(unsigned long data);


// Application asynchronous events callbacks
typedef void (*vv_app_async_event_callback_t)(vv_event_record_t event_record);


///////////////////////////
// OLD will be delete
// Application asynchronous events callbacks (as in tasklet_struct)
//typedef void (*vv_app_async_event_callback_t)(unsigned long data);

// Application completion notification callbacks
//typedef void (*vv_com_event_callback_t)(unsigned long data);

// Application completion notification callbacks
//typedef void (*vv_async_event_callback_t)(unsigned long data);
///////////////////////////


///////////////////////////
// Mellanox callbacks:
// do not use this in application that running over the adaptors

// Completion handler callback (for mlx)
typedef void (*vv_com_event_callback_t)(IN vv_hca_h_t  hca_h,
                                        IN vv_cq_h_t   cq_h,
                                        void          *private_data);


// Asynchronous event handler callback (for mlx)
typedef void (*vv_async_event_handler_t)(IN vv_hca_h_t         hca_h,
                                         IN vv_event_record_t *event_record_p,
                                         void                 *private_data);
///////////////////////////

# endif // __VV_VV_TYPES_H__

