/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

# ifndef __VVERBS_BASE_I_H__
# define __VVERBS_BASE_I_H__

//////////////////////////////////////////////////////////
//
// Author: Yiftah Shahar <yiftahs@voltaire.com>
//
// Description: 
// vverbs-base internal defenitions.
//
//////////////////////////////////////////////////////////

#include "linux/spinlock.h"

// Voltaire                        
#include "vv-types.h"
#include "vverbs-base.h"
#include "vv-adaptor-common.h"

  
#define VV_ADDRESS_HNDL_POOL_SUPPORT

#ifdef BARAK5_BUILD
#define VV_ADDRESS_HNDL_POOL_FIFO		// added for FIX barak 5 cq error 2
#endif

#ifdef  VV_ADDRESS_HNDL_POOL_SUPPORT

#ifndef VV_ADDRESS_HNDL_POOL_FIFO
#define VV_ADDRESS_HNDL_POOL_SIZE 0x800
#define VV_MIN_ADDRESS_HNDL_POOL_SIZE (VV_ADDRESS_HNDL_POOL_SIZE/10)
#else	// VV_ADDRESS_HNDL_POOL_FIFO
#define VV_ADDRESS_HNDL_POOL_SIZE 200  //255 THE MAX IN MLX IS 256   // for optimize the modolo size+1 == 2^n
#define VV_MIN_ADDRESS_HNDL_POOL_SIZE  5
#endif	// VV_ADDRESS_HNDL_POOL_FIFO

#define VV_ADDRESS_HNDL_POOL_LOCK_VAR       unsigned long         sflags
#define VV_ADDRESS_HNDL_POOL_LOCK(pool)     spin_lock_irqsave     (&pool->lock, sflags)
#define VV_ADDRESS_HNDL_POOL_UNLOCK(pool)   spin_unlock_irqrestore(&pool->lock, sflags)

#ifdef VV_SLEEPABLE_ASYNC_EVENTS
typedef struct vv_sleepable_async_er_item_st  vv_sleepable_async_er_item_t;
#endif

typedef struct vv_async_er_item_st  vv_async_er_item_t;
typedef struct vv_base_hca_data_st  vv_base_hca_data_t;
typedef struct vv_base_port_data_st vv_base_port_data_t;

#ifndef VV_ADDRESS_HNDL_POOL_FIFO
typedef struct vv_address_hndl_pool_st
{
  vv_add_h_t *array;
  int   cnt;
  spinlock_t lock;
}vv_address_hndl_pool_t;
#else // VV_ADDRESS_HNDL_POOL_FIFO
typedef struct vv_address_hndl_pool_st
{
  vv_add_h_t array[VV_ADDRESS_HNDL_POOL_SIZE + 1]; // 1 elamant to mark full
  int head;
  int tail;
  spinlock_t lock;
}vv_address_hndl_pool_t;
#endif	// VV_ADDRESS_HNDL_POOL_FIFO

#endif

#ifdef VV_SLEEPABLE_ASYNC_EVENTS
#define VV_MSG_MAX_DATA_SIZE sizeof(vv_directed_event_rec_t)

typedef enum
{
        vv_thread_msg_fw_event,
        vv_thread_msg_fw_directed_event,
        vv_thread_msg_stop
}vv_thread_msg_t;

typedef struct vv_directed_event_rec_st
{
	vv_event_record_t	         ev_rec;
	vv_app_async_event_callback_t    asy_ev_cb;
}vv_directed_event_rec_t;

typedef struct vv_msg_list_entry_st
{
        struct vv_msg_list_entry_st *next;
        struct vv_msg_list_entry_st *prev;
        
        struct {
                int  type;
		int  data_size;
                char data[VV_MSG_MAX_DATA_SIZE];
        }msg;

}vv_msg_list_entry_t;

typedef struct vv_thread_data_s
{
        struct semaphore                  sem;
        int                               run;
	volatile int                      stopped_event;
	int				  thread_pusher_ref_count;

        spinlock_t                        msg_list_lock;
        struct list_head                  msg_list;
	
#define VV_MSG_LIST_LOCK_VAR              unsigned long msg_list_sflags
#define VV_MSG_LIST_LOCK(__t__)           spin_lock_irqsave      (&(__t__)->msg_list_lock, msg_list_sflags)
#define VV_MSG_LIST_UNLOCK(__t__)         spin_unlock_irqrestore (&(__t__)->msg_list_lock, msg_list_sflags)
 
}vv_thread_data_t;

#endif

// Information for every opened HCA in the vverbs-base modules
struct vv_hca_info_st
{
  vv_hca_func_t*    hi_funcs;       // Functions to handle this HCA.
  vv_opened_hca_t*  hi_prov_handle; // Handle for the specific HCA as we receive from the adaptor layer
  atomic_t          hi_ref_count;
  char*             hi_uname;       // Unified name: hca0, hca1, hca2, ...
  char*             hi_sname;       // Specific name: vo0, mt21108_pci0, vieo2, ...
  struct list_head  hi_list;        // all the open HCAs.              ----------------------need to see that
  struct list_head  hi_prov_list;   // all the open HCAs, from the same provider. -----------need to see that
#ifdef VV_ADDRESS_HNDL_POOL_SUPPORT
  vv_address_hndl_pool_t *address_hndl_pool;
#endif
  struct list_head  async_ev_list;

#ifdef VV_SLEEPABLE_ASYNC_EVENTS
  struct list_head  sleepable_async_ev_list_up; 
  struct list_head  sleepable_async_ev_list_down;
  vv_thread_data_t  thread_data;
#endif  
};


// HCA provider item for the use of the vverbs-base modules
struct vv_hca_prov_st
{
  vv_hca_prov_t*    next;          // List of providers.
  vv_hca_prov_t*    prev;
  char*             hp_name;       // Provider name: Voltaire, Vieo, Intel, IBM, Mellanox, ...
  vv_hca_func_t*    hp_funcs;      // Functions to handle this HCA.
  char**            hp_devices;    // Supported specific device names: mt21108_pci0 mt21104_pci0...
  struct list_head  hp_open_hcas;  // All the Open HCA for this provider.
};


// the base hold the elements in the registered async list according priority
struct vv_async_er_item_st
{
    vv_async_er_item_t*           next;
    vv_async_er_item_t*           prev;
    u_int32_t                     prior;
    vv_app_async_event_callback_t app_cb;
    vv_request_event_record_t     req_er;
};

#ifdef VV_SLEEPABLE_ASYNC_EVENTS
/* Monski: this entry should be linked to two lists one for up events and one for down events */
struct vv_sleepable_async_er_item_st
{
    struct list_head              up_list;
    struct list_head              down_list;
    vv_app_async_event_callback_t app_cb;
    vv_request_event_record_t     req_er;
};
#endif

// hold data for hca to comper with the current query - async event generation
struct vv_base_hca_data_st
{
    vv_base_hca_data_t* next;
    vv_base_hca_data_t* prev;
    vv_hca_info_t*      hca_h_p;
    vv_hca_attrib_t     hca_attr;
    struct list_head    port_data_list;
    VV_BOOL_T           suspend;        // YSBUGBUG - the HCA suspend is not supported for now !!!
    unsigned long       jiff_to_wait;   // YSBUGBUG - the HCA suspend is not supported for now !!!
};

// hold data for port to comper with the current query - async event generation
struct vv_base_port_data_st
{
    vv_base_port_data_t* next;
    vv_base_port_data_t* prev;
    VV_BOOL_T            suspend;
    unsigned long        jiff_to_wait;
    vv_port_attrib_t     port_attr;
    vv_gid_t*            gid_tbl_p;  // table size is as in the port attrib
    vv_p_key_t*          pkey_tbl_p; // table size is as in the port attrib
};

int timermod_init(void);
void timermod_exit(void);

int vv_timerthread_init(void);
void vv_timerthread_exit(void);


# endif // __VVERBS_BASE_I_H__

