/*
 * Copyright (C) Voltaire Ltd. 2001-2003.  ALL RIGHTS RESERVED.
 *
 * This source code (the "Code") is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest in and to the Code,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * The Code is provided "as is". To the maximum extent permitted by law,
 * the Company disclaims any warranties of any kind, either expressed or
 * implied, including, without limitation, implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement.
 * The Company does not warrant that the Code will operate error free, or
 * in an uninterrupted fashion or that it is compatible with any particular
 * platform. In no event shall the Company be liable for any damages
 * whatsoever (including, without limitation, incidental, direct, indirect,
 * special and consequential damages, damages for loss of business profits,
 * business interruption, loss of business information, or other pecuniary
 * loss) arising out of the use or inability to use the Code, even if
 * advised of the possibility of such damages.
 */

# ifndef __VVERBS_H__
# define __VVERBS_H__


// Author: Yiftah Shahar <yiftahs@voltaire.com>
// Provide the Kernel Verbs consumer generic HCA support.

# include "vv-types.h"
# include "vverbs-base-i.h"
# include "vv-adaptor-common.h"
# include "vverbs-ag.h"




struct vv_hca_name {
		char name[VV_MAX_DEV_NAME_SIZE];
};


int vv_list_hcas(struct vv_hca_name *hca_list, int list_size);

extern const char* 
vv_default_hca_name();


extern vv_return_t
vv_provider_reg (IN const char      *prov_name,
                 IN vv_hca_func_t   *functions,
                 IN const char     **devices,
                 OUT vv_hca_prov_t **handlep);

extern vv_return_t
vv_provider_unreg (IN vv_hca_prov_t *handle);



// Consumer interface

// Functions implementation, is below the verbs.
void vv_hca_show_all (void);

// BUGBUG - DO NOT use the context - insert NULL
// BUGBUG - hca application context is NOT supported yet by the adaptor
extern vv_return_t vv_hca_open (IN const char* name, IN vv_hca_context_t context, OUT vv_hca_h_t* handle);
extern vv_return_t vv_hca_close (IN vv_hca_h_t handle);

extern vv_return_t vv_set_async_event_cb(IN vv_hca_h_t handle, IN vv_request_event_record_t req_er, IN vv_app_async_event_callback_t async_event_callback);

extern vv_return_t vv_dell_async_event_cb(IN vv_hca_h_t handle, IN vv_app_async_event_callback_t async_event_callback);

#ifdef VV_SLEEPABLE_ASYNC_EVENTS
extern vv_return_t vv_set_sleepable_async_event_cb(IN vv_hca_h_t handle, IN vv_request_event_record_t req_er, IN vv_app_async_event_callback_t async_event_callback);

extern vv_return_t vv_dell_sleepable_async_event_cb(IN vv_hca_h_t handle, IN vv_app_async_event_callback_t async_event_callback);
extern vv_return_t vv_async_ev_push_gid_in(IN vv_hca_h_t hca_h, IN vv_gid_t* gid);
extern vv_return_t vv_async_ev_push_gid_out(IN vv_hca_h_t hca_h, IN vv_gid_t* gid);
extern vv_return_t vv_async_ev_push_mc_group_create(IN vv_hca_h_t hca_h, IN vv_gid_t* gid);
extern vv_return_t vv_async_ev_push_mc_group_delete(IN vv_hca_h_t hca_h, IN vv_gid_t* gid);

#define VV_SL_EV_RMMOD_VARS                 \
	static atomic_t thread_runnig = {0};\
	static atomic_t rmmod_pending = {0}
	
#define VV_SL_EV_HANDLER_IN()               \
	if(atomic_read(rmmod_pending))      \
	       return;                      \
	atomic_set(thread_runnig, 1)
	
#define VV_SL_EV_HANDLER_OUT()              \
	atomic_set(thread_runnig, 0)

#define VV_SL_EV_RMMOD_IN()                 \
	atomic_set(rmmod_pending, 1);

#define VV_SL_EV_RMMOD_WAIT_ON_HANDLER_OUT()\
        while(atomic_read(thread_runnig))   \
        {                                   \
		schedule();                 \
        }

#endif

extern vv_return_t pkey2pkey_index(IN vv_hca_h_t hca, IN int port_num, IN vv_p_key_t pkey, OUT int *pkey_idx);
extern vv_return_t pkey_index2pkey(IN vv_hca_h_t hca, IN int port_num, IN int pkey_idx, OUT vv_p_key_t *pkey);
extern vv_return_t gid2gid_index(IN vv_hca_h_t hca, IN int port_num, IN const vv_gid_t *gid, OUT int *gid_idx);
extern vv_return_t gid_index2gid(IN vv_hca_h_t hca, IN int port_num, IN int gid_idx, OUT vv_gid_t *gid);

extern vv_return_t base_gid2port_num(IN vv_hca_h_t hca, IN vv_gid_t *gid, OUT int *port_num);
extern vv_return_t port_num2base_gid(IN vv_hca_h_t hca, IN int port_num, OUT vv_gid_t *gid);

extern vv_return_t port_num2base_lid(IN vv_hca_h_t hca, IN int port_num, OUT vv_lid_t *lid);
#ifdef VV_ADDRESS_HNDL_POOL_SUPPORT
extern
vv_return_t vv_address_hndl_get (vv_hca_h_t hca_p, IN vv_add_vec_t* add_vec, OUT vv_add_h_t* add_h);
extern 
vv_return_t vv_address_hndl_put(vv_hca_h_t hca_p, IN vv_add_h_t add_h);
#endif
# endif // __VVERBS_H__
