#!/bin/bash
#!/bin/bash -vx
#
# Startup script for IB-Host, Voltaire IB-Stack using Tavor/Mellanox driver
#
# Configuration variables.
#  Note: Parameters should be in the repository /usr/voltaire/config/repository.rps

if [ "A${1}" == "Ahpc" ] ; then
    echo "Voltaire HPC Pack is starting up....."
else
    echo "Voltaire ALL/BIZ pack is starting up....."
fi

SGI_RELEASE=${SGI_RELEASE:-off}
if [ -f /proc/sgi_sn ]; then
                SGI_RELEASE=on
    		echo "Voltaire IB Stack for Altix"
else
        if [ -f /etc/sgi-release ]; then
		SGI_RELEASE=on
    		echo "Voltaire IB Stack for Altix"
        fi
fi

# IBLAN: on off
IBLAN=${IBLAN:-off}
# IPOIBUD: on off
IPOIBUD=${IPOIBUD:-on}
# SMON: on off
SMON=${SMON:-off}

#load sdp & q_mng modules
SDP_ENABLE=$(rep-get modules/sdp enabled | perl -ne 'print hex')
#load kdapl modules
KDAPL_ENABLE=$(rep-get modules/kdapl enabled | perl -ne 'print hex')
#load the UCM module
UCMENABLED=$(rep-get modules/devucm enabled | perl -ne 'print hex')
#load the UGSI module
UGSIENABLED=$(rep-get modules/devugsi enabled | perl -ne 'print hex')


# Development drectories
# MODDIRS='nvigor/ib-code/utils nvigor/managment/repository nvigor/logger nvigor/ib-code/vverbs nvigor/ib-code/adaptor-tavor nvigor/ib-code/gsi nvigor/ib-code/cm nvigor/ib-code/ib-arp-table nvigor/sdp/sdp/q-mng nvigor/ib-code/blan nvigor/ib-code/ipoib-ud nvigor/sdp/sdp nvigor/sdp/sdp-sockets  nvigor/ib-code/ats nvigor/ib-code/umode-if/devucm nvigor/ib-code/umode-if/devugsi'
# MODDIRS="/usr/voltaire/modules /usr/mellanox/modules"
MODDIRS="/usr/voltaire/modules/$(uname -r)"

export PATH="/usr/voltaire/bin:/usr/voltaire/scripts:/usr/mellanox/bin:$PATH"

DMSGLVL=1

# Functions

loadmodule(){
    m=$1
    if [ `uname -r | cut -f 1,2 -d .` = "2.6" ]; then	# 2.6 module names end with a ".ko" instead of 2.4's ".o"
        m=`echo "$1" | sed s/\.o$/\.ko/g`
    fi
    shift
    # echo "loading $m"
    for d in ${MODDIRS} ; do
        if [ -e $d/$m ] ; then
            if ! insmod $d/$m "${@}" >/dev/null 2>&1 ; then
		echo "failed to load $d/$m ${@} !"
                return 1
            fi
            return 0
        fi
    done
    echo "module '$m' not found."
    return 1
}
create_dev(){
	device=$1
	major=$(cat /proc/devices | awk '$2=="'${device}'"{print $1}')
	rm -f /dev/${device}
	mknod /dev/${device} c $major 0
	chown root.root /dev/${device}
	chmod 666  /dev/${device}
}

check_for_tavor(){
    if ! lspci -n | grep -q '15b3:5a44' ; then
        echo "Voltaire IB-Host: Infiniband HCA (Tavor/Mellanox) not found in this system."
        echo "Voltaire IB-Host: Infiniband HCA (Tavor/Mellanox) not found in this system." > /dev/tty
        exit 1
    fi
}
check_for_modules_directory(){
    if [ ! -d /usr/voltaire/modules/$(uname -r) ]; then
	echo "Voltaire IB-Host: Current Kernel version '$(uname -r)' not supported."
	echo "Voltaire IB-Host: Current Kernel version '$(uname -r)' not supported." > /dev/tty
	exit 1
    fi
}

# Start work

 
dmesg -n${DMSGLVL}

# Startup the system.

check_for_modules_directory ;

if [ ! -e /usr/voltaire/config/dat.conf ] ; then
	echo ERROR: missing file /usr/voltaire/config/dat.conf
	exit 1
fi

vapi start
for ((s=0;s<10;s++)); do
        hca_id=$(vstat|grep hca_id|cut -f2 -d=)
        if [ -z $hca_id ] ; then
                sleep 1
        else
                break
        fi
done
if [ -z $hca_id ] ; then
                echo ERROR: cant find HCA id
                exit 1
fi


# Monski 1=DHCP MODE 0=STATIC MODE
DHCP_ON=$(rep-get interfaces/ipoib-ud dhcp_mode_on | perl -ne 'print hex')
# LSM - what SM is used ( minism=2, vfm=1).
LSM=$(rep-get modules/subnet-manager type )
# SAMGJ - SA multicast join: "no-join" - used with minism, "join" - join the MG, "create" - create the MG.
SAMGJ=$(rep-get modules/ipoib-ud link-join )
# ES: Verify with Aviv, that the s_partition should belong to modules and not interface ?!
S_PARTITION=$(rep-get modules/ipoib-ud s-partition | perl -ne 'print hex')
# Does the fabric SM support path query? Do we want to send it
PATH_QUERY=$(rep-get  modules/ib-arp-table sa-queries | perl -ne 'print hex')
#Static path parameters
PATH_RAW_TRAFFIC=$(rep-get  modules/ib-arp-table raw_traffic | perl -ne 'print hex')
PATH_FLOW_LBL=$(rep-get  modules/ib-arp-table flow_lbl | perl -ne 'print hex')
PATH_HOP_LMT=$(rep-get  modules/ib-arp-table hop_lmt | perl -ne 'print hex')
PATH_TRAFFIC_CLS=$(rep-get  modules/ib-arp-table traffic_cls | perl -ne 'print hex')
PATH_PKEY=$(rep-get  modules/ib-arp-table pkey | perl -ne 'print hex')
PATH_SL_MASK=$(rep-get  modules/ib-arp-table sl_mask | perl -ne 'print hex')
PATH_MTU_SEL=$(rep-get  modules/ib-arp-table mtu_sel | perl -ne 'print hex')
PATH_MTU=$(rep-get  modules/ib-arp-table mtu | perl -ne 'print hex')
PATH_RATE_SEL=$(rep-get  modules/ib-arp-table rate_sel | perl -ne 'print hex')
PATH_RATE=$(rep-get  modules/ib-arp-table rate | perl -ne 'print hex')
PATH_PKT_LIFE_TIME_SEL=$(rep-get  modules/ib-arp-table pkt_life_time_sel | perl -ne 'print hex')
PATH_PKT_LIFE_TIME=$(rep-get  modules/ib-arp-table pkt_life_time | perl -ne 'print hex')

# rmpp spec complient or backward compatible
RMPP_SPEC_COMP=$(rep-get modules/gsi rmpp-comp | perl -ne 'print hex')

#HP work around for hang during shutdown not unregister_dev when stopping the stack
NOT_UNREG_DEV=$(rep-get modules/ipoib-ud not-unreg-dev | perl -ne 'print hex')


# ATS support
#------------
# "active_mode" field defined for ats in the repository can have 3 values:
# 0 - don't load ats module at all (e.g when using minism).
# 1 - load ats module but don't use it as address resolveri(ipoib will do that)
# 2 - load ats module AS address resolver.

ATS_ON=$(rep-get modules/ats active_mode | perl -ne 'print hex')

#ipoib MTU - take the configured MTU from ipoib-ud0 only (load module only once)
MTU=$(rep-get interfaces/ipoib-ud mtu0 | perl -ne 'print hex')
if [[ -z $MTU ]] ; then
        echo "MTU not configured, using default : 1500"
        MTU=1500
fi

#ats and join multicast heart beat 
JOIN_HEART_BEAT=$(rep-get modules/ipoib-ud sm-heart-beat| perl -ne 'print hex')
JOIN_HEART_BEAT_SKEW=$(rep-get modules/ipoib-ud sm-heart-beat-skew| perl -ne 'print hex')
ATS_HEART_BEAT=$(rep-get modules/ats sm-heart-beat| perl -ne 'print hex')
ATS_HEART_BEAT_SKEW=$(rep-get modules/ats sm-heart-beat-skew| perl -ne 'print hex')

#join_arp_failure
JOIN_ARP_FAILURE=$(rep-get modules/ipoib-ud join_arp_failure | perl -ne 'print hex')

#path_retries
PATH_RETRIES=$(rep-get modules/ib-arp-table path_retries | perl -ne 'print hex')

if [[ ${ATS_ON} == 2 ]] ; then
#ATS only - for now will prevent IPOIBUD to be loaded"
#           in the future, ipoib will be up only won't register itself to ibat
   IPOIBUD=off
fi


echo "READ PARAM Join=$JOIN_HEART_BEAT ATS=$ATS_HEART_BEAT"

if (( ${SDP_ENABLE} )); then
    # Read SDP Variables
    SDP_BUF_SIZE=$(rep-get modules/sdp buf_size | perl -ne 'print hex')
    SDP_BUF_NUMBER=$(rep-get modules/sdp buf_number | perl -ne 'print hex')
    SDP_RCV_LIST_SIZE=$(rep-get modules/sdp rcv_list_size | perl -ne 'print hex')
    SDP_RDMA_TRESHOLD=$(rep-get modules/sdp rdma_treshold | perl -ne 'print hex')
    SDP_UPD_CRDT_LOW=$(rep-get modules/sdp update_credit_low | perl -ne 'print hex')
    SDP_UPD_CRDT_HIGH=$(rep-get modules/sdp update_credit_high | perl -ne 'print hex')
    SDP_SIGNALLING_TH=$(rep-get modules/sdp signalling_th | perl -ne 'print hex')
    SDP_SOCK_SEND_LIMIT=$(rep-get modules/sdp-sockets send_limit | perl -ne 'print hex')
    SDP_SOCK_FIXED_SIZE=$(rep-get modules/sdp-sockets fixed_size | perl -ne 'print hex')
    echo "SDP params: "
    echo "SDP_BUF_SIZE ${SDP_BUF_SIZE} SDP_BUF_NUMBER ${SDP_BUF_NUMBER}"
    echo "SDP_RCV_LIST_SIZE ${SDP_RCV_LIST_SIZE} SDP_RDMA_TRESHOLD ${SDP_RDMA_TRESHOLD}"
    echo "SDP_UPD_CRDT_LOW ${SDP_UPD_CRDT_LOW} SDP_UPD_CRDT_HIGH ${SDP_UPD_CRDT_HIGH}"
    echo "SDP_SIGNALLING_TH ${SDP_SIGNALLING_TH}"
    echo "SDP_SOCK_SEND_LIMIT ${SDP_SOCK_SEND_LIMIT} SDP_SOCK_FIXED_SIZE ${SDP_SOCK_FIXED_SIZE}"
    # Read Q-MNG Variables
    QMNG_CQ_SIZE=$(rep-get modules/q-mng cq_size | perl -ne 'print hex')
    QMNG_SEND_QP_SIZE=$(rep-get modules/q-mng send_qp_size | perl -ne 'print hex')
    QMNG_RECV_QP_SIZE=$(rep-get modules/q-mng recv_qp_size | perl -ne 'print hex')
    QMNG_CQ_NUMBER=$(rep-get modules/q-mng cq_number | perl -ne 'print hex')
    QMNG_QPS_PER_CQ=$(rep-get modules/q-mng qp_per_cq | perl -ne 'print hex')
    echo "q-mng: "
    echo "QMNG_CQ_SIZE ${QMNG_CQ_SIZE} QMNG_RECV_QP_SIZE ${QMNG_RECV_QP_SIZE} QMNG_SEND_QP_SIZE ${QMNG_SEND_QP_SIZE}"
    echo "QMNG_CQ_NUMBER ${QMNG_CQ_NUMBER} QMNG_QPS_PER_CQ ${QMNG_QPS_PER_CQ}"

    # Read FMR values
    FMR_UNMAP_AT_ONCE=$(rep-get modules/sdp fmr_unmap_at_once | perl -ne 'print hex')
    FMR_AVG_RECV_SIZE=$(rep-get modules/sdp avg_recv_size  | perl -ne 'print hex')
    FMR_AVG_RECV_SIZE_PERCENT=$(rep-get modules/sdp avg_recv_size_percent  | perl -ne 'print hex')
    FMR_NUM_OF_HNDLS=$(rep-get modules/sdp num_of_fmr_hndls | perl -ne 'print hex')
    FMR_WATERMARK=$(rep-get modules/sdp fmr_water_mark | perl -ne 'print hex')
    FMR_ACTIVE=$(rep-get modules/sdp active_fmr  | perl -ne 'print hex')
    
    echo "FMR params: "
    echo "FMR_AVG_RECV_SIZE ${FMR_AVG_RECV_SIZE} FMR_AVG_RECV_SIZE_PERCENT ${FMR_AVG_RECV_SIZE_PERCENT}"
    echo "FMR_NUM_OF_HNDLS ${FMR_NUM_OF_HNDLS} FMR_WATERMARK ${FMR_WATERMARK}  FMR_ACTIVE ${FMR_ACTIVE}"
    
fi

loadmodule hadump.o
loadmodule repository.o
create_dev repository
loadmodule mlog.o

VVERBS_AH_POOL_SIZE=$(rep-get modules/vverbs vverbs_ah_pool_size | perl -ne 'print hex')
loadmodule vverbs.o vverbs_ah_pool_size=${VVERBS_AH_POOL_SIZE}


loadmodule ad_tavor.o vvi_mlx_max_ah_per_pd=$((${VVERBS_AH_POOL_SIZE}+${VVERBS_AH_POOL_SIZE}/10))
if (( ${RMPP_SPEC_COMP} )) ; then
	loadmodule gsim.o rmpp_spec_compliant=1
else
	loadmodule gsim.o rmpp_spec_compliant=0
fi

usleep 1000000
loadmodule cm.o mra_on_req=3000000
echo after cm start

loadmodule ibat.o
echo enable > /proc/voltaire/ib-arp-table/control
if (( ${PATH_QUERY} == 1 )) ; then
    echo "sa_queries on " > /proc/voltaire/ib-arp-table/control
    echo "Enabeling path query initiation, in arp process"
fi

#set static path query from repository
echo setstaticpath $PATH_RAW_TRAFFIC $PATH_FLOW_LBL $PATH_HOP_LMT $PATH_TRAFFIC_CLS $PATH_PKEY $PATH_SL_MASK $PATH_MTU_SEL $PATH_MTU $PATH_RATE_SEL $PATH_RATE $PATH_PKT_LIFE_TIME_SEL $PATH_PKT_LIFE_TIME > /proc/voltaire/ib-arp-table/control

# set the number of the path retries (arp retries)
echo set_retry_cnt $PATH_RETRIES > /proc/voltaire/ib-arp-table/control
 
if (( ${SDP_ENABLE} )); then
	loadmodule q_mng.o snd_qp_size=${QMNG_SEND_QP_SIZE} rcv_qp_size=${QMNG_RECV_QP_SIZE} cq_size=${QMNG_CQ_SIZE} number_of_cq=${QMNG_CQ_NUMBER} qp_per_cq=${QMNG_QPS_PER_CQ}
fi

if (( $UCMENABLED )); then
        loadmodule devucm.o
        ibhost-mkmisc ucm
        sleep 1
        chmod 666     /dev/ucm
fi

if (( $UGSIENABLED )); then
	loadmodule devugsi.o
	ibhost-mkmisc  devugsi
        sleep 1
        chmod 666      /dev/devugsi
fi

if [[ $ATS_ON == 1 ]]; then
#Fix: IP address will be set later by ipoib-ud module..
        loadmodule ats.o ats_hostName_str=$(hostname) ats_attach_ibat=0
        echo sa_verify_period $ATS_HEART_BEAT $ATS_HEART_BEAT_SKEW > /proc/voltaire/ats/control
else
if [[ $ATS_ON == 2 ]]; then
       LIP=$(rep-get interfaces/ipoib-ud ip-addr1 )
       loadmodule ats.o ats_ipaddr_str=${LIP} ats_hostName_str=$(hostname) ats_attach_ibat=1
       echo sa_verify_period $ATS_HEART_BEAT $ATS_HEART_BEAT_SKEW > /proc/voltaire/ats/control
fi
fi


if [ $IBLAN = on ]; then
    loadmodule iblan.o
    echo 'add InfiniHost0 1' > /proc/voltaire/iblan/control
    ifconfig iblan0 ${LIP} netmask ${LNM} broadcast ${LBR}
    echo 'ibat_att iblan0' > /proc/voltaire/iblan/control
fi

#minism=2,vfm=1
if [ "A${LSM}" == "AVFM" ] ; then
    SM_OP=1
else
    SM_OP=2
fi

echo "IPoIB-UD work with ipoib_ud_sm_wm= $SM_OP"

#Gali - work around - MTU hard coded
if [ $IPOIBUD = on ]; then
    if [ "A${LSM}" == "AVFM" ] ; then
       loadmodule ipoib-ud.o ipoib_ud_dev_name="ipoib" ipoib_ud_mtu=$MTU ipoib_ud_tx_comp_notify_count=1 ipoib_ud_sm_wm=$SM_OP not_unreg_dev_wa=$NOT_UNREG_DEV
    else
       loadmodule ipoib-ud.o ipoib_ud_dev_name="ipoib" ipoib_ud_mtu=$MTU ipoib_ud_qk=2843 ipoib_ud_tx_comp_notify_count=1 ipoib_ud_sm_wm=$SM_OP not_unreg_dev_wa=$NOT_UNREG_DEV
    fi
                                                                                                                            
    
    if [ -z $hca_id ] ; then
           echo ERROR: cant find HCA id before add ipoib interface
    else
           # Support for more than one pkey
           for ((i=0;i<64;i++))
           do
           	pkey=$(rep-get interfaces/ipoib-ud pkey$i | perl -ne 'print hex')
               if [[ $pkey -ne 0 ]]; then
           		echo 'add_r '$hca_id' '$pkey' '$i'' > /proc/voltaire/ipoib-ud/control
               fi
           done
    fi

    
#    if [ "${DHCP_ON}" = 0 ]; then
#       ifconfig ipoib0 ${LIP} netmask ${LNM} broadcast ${LBR}
#    else
#	if [ -s /var/run/dhcpcd-ipoib0.pid ]; then
#		dhcpcd -n ipoib0
#	fi
#    fi


#        ifup ipoib0
    
fi

#ats and join heart beat
echo "setting heart beat: Join=$JOIN_HEART_BEAT ats=$ATS_HEART_BEAT"
echo bc_mc_group_period $JOIN_HEART_BEAT $JOIN_HEART_BEAT_SKEW > /proc/voltaire/ipoib-ud/control

#join_arp_failure
echo join_arp_failure $JOIN_ARP_FAILURE >/proc/voltaire/ipoib-ud/control

# modify /usr/voltaire/config/dat.conf
if [ -z $hca_id ] ; then
        echo ERROR: cant find HCA id before modifying /usr/voltaire/config/dat.conf
else
	sed s/InfiniHost[A-Z,a-z,0-9,_]*/$hca_id/g /usr/voltaire/config/dat.conf > /tmp/$$-dat-conf
	mv /tmp/$$-dat-conf /usr/voltaire/config/dat.conf
fi

if (( ${KDAPL_ENABLE} )); then
        LIP=$(rep-get interfaces/ipoib-ud ip-addr1 )
        loadmodule kdat_registry.o DbgLvl=0x3
        loadmodule kdapl.o DbgLvl=0x10003  
        loadmodule kdapltest_module.o

        kdapltest_major=`grep kdapltest /proc/devices | cut -d' ' -f1`
        kdapltest_minor=0
        mknod /dev/kdapltest c $kdapltest_major $kdapltest_minor
fi


if (( ${SDP_ENABLE} )); then
#    loadmodule sdp.o


   ((sdp_load = 0 )) 
    for dd in  ${MODDIRS} ; do
        if [ -e $dd/sockets-aio.o ] ; then
                  echo "activated FMR found in $dd/sockets-aio.o"
                  loadmodule sdp.o active_fmr=${FMR_ACTIVE}  avg_recv_size=${FMR_AVG_RECV_SIZE} avg_recv_size_percent=${FMR_AVG_RECV_SIZE_PERCENT} num_of_fmr_hndls=${FMR_NUM_OF_HNDLS}  fmr_water_mark=${FMR_WATERMARK}
                  
                  ((sdp_load = 1))
                  break;
        fi
        
    done

    if  [[ $sdp_load == 0 ]]  ; then
       echo "FMR is NOT activated!"
       loadmodule sdp.o;
    fi

    # Might be run over in AIO kernel:
    echo "change_send_signal_th ${SDP_SIGNALLING_TH}" > /proc/voltaire/sdp/control

    if loadmodule sockets-aio.o > /dev/null 2>&1 ; then 
	echo "SDP/AIO Sockets loaded."
	echo "change_send_signal_th 0" > /proc/voltaire/sdp/control
    elif loadmodule sockets.o > /dev/null 2>&1; then
	echo "SDP Sockets loaded."
    else
	echo "failed to load SDP Sockets."
    fi


    echo  "change_buf_size ${SDP_BUF_SIZE}" > /proc/voltaire/sdp/control
    echo  "change_buf_num ${SDP_BUF_NUMBER}" > /proc/voltaire/sdp/control
    echo  "change_post_rcv_pnd_th ${SDP_RCV_LIST_SIZE}" > /proc/voltaire/sdp/control
    echo  "change_send_bcopy_th ${SDP_RDMA_TRESHOLD}" > /proc/voltaire/sdp/control
    echo  "change_uc_watermark ${SDP_UPD_CRDT_LOW}" > /proc/voltaire/sdp/control
    echo  "change_uc_posted_buff_th ${SDP_UPD_CRDT_HIGH}" > /proc/voltaire/sdp/control
    # SDP Socket Setup
    # Fixed Size :1 - configuration controll, 0 - Application controll (default)
    echo  "sndbuf_status ${SDP_SOCK_FIXED_SIZE}" > /proc/voltaire/sdp-sockets/control
    echo  "sndbuf_size ${SDP_SOCK_SEND_LIMIT}" > /proc/voltaire/sdp-sockets/control

fi
exit 0
