#!/bin/false -- do not execute!
#

#MTL-HEADER##################################################################
#                 - Mellanox Confidential and Proprietary -
#
# Copyright (C) Jan. 2004, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
#
# Except as specifically permitted herein, no portion of the information,
# including but not limited to object code and source code, may be reproduced,
# modified, distributed, republished or otherwise exploited in any form or by
# any means for any purpose without the prior written permission of Mellanox
# Technologies Ltd. Use of software subject to the terms and conditions
# detailed in the file "LICENSE.txt".
# End of legal section.
#############################################################################
# Id ..........$Id: g_puts.tcl,v 1.6 2004/09/21 12:46:57 eitan Exp $
# Revision.....$Revision: 1.6 $
# Date.........$Date: 2004/09/21 12:46:57 $
# Author.......$Author: eitan $
#EOH#########################################################################

# We are based on Thread package.
# We track the configuration of the logger thread in g_puts_conf
# array

proc g_puts_set_file {file_name} {
   global _g_puts_file

   # close if previously opened:
   if {$_g_puts_file != "stdout"} {
      close $_g_puts_file
   }
   
   # open the requested log file:
   if {[catch {set _g_puts_file [open $file_name w]} e]} {
      puts stderr "-E- Fail to open log file:$file_name"
   }
   fconfigure $_g_puts_file -buffering line
}

proc g_puts_set_dbg_file {file_name} {
   global _g_puts_dbg_file

   # close if previously opened:
   if {$_g_puts_dbg_file != "stdout"} {
      close $_g_puts_dbg_file
   }
   
   # open the requested log file:
   if {[catch {set _g_puts_dbg_file [open $file_name w]} e]} {
      puts stderr "-E- Fail to open debug file:$file_name"
   }
   fconfigure $_g_puts_dbg_file -buffering none
}

proc g_puts {input} {
   if {[tsv::exists g_puts_conf g_puts_thread] && \
          ([tsv::get g_puts_conf g_puts_thread] != [thread::id])} {
      thread::send -async [tsv::get g_puts_conf g_puts_thread] "g_puts {$input}"
      return 0
   }

   global MATCHED  ERRORS_LINKS G_PUTS_ENABLE G_PUTS_INFORM G_PUTS_ERROR G_PUTS_DEBUG G_PUTS_WARNING G_PUTS_DEFAULT
   global _g_puts_file _g_puts_dbg_file

   if {![info exist G_PUTS_ENABLE]} {
      set G_PUTS_ENABLE 1
   }
   if {![info exist G_PUTS_ERROR]} {
      set G_PUTS_ERROR 1
   }
   if {![info exist G_PUTS_WARNING]} {
      set G_PUTS_WARNING 0
   }
   if {![info exist G_PUTS_INFORM]} {
      set G_PUTS_INFORM 1
   }
   if {![info exist G_PUTS_DEBUG]} {
      set G_PUTS_DEBUG 0
   }
   if {![info exist G_PUTS_DEFAULT]} {
      set G_PUTS_DEFAULT 0
   }

   # truncate a long message ???
   if {[string length $input] > 51200} {
      set msg "[string range $input 0 200]..."
   } else {
      set msg $input
   }

   if {$G_PUTS_ENABLE == 1} {
      set prefix [string range $input 0 2]
      
      switch -- $prefix {
         -I- {
            if {$G_PUTS_INFORM == 1} {
               puts $_g_puts_file "$msg"
            }
         }
         -W- {
            if {$G_PUTS_WARNING == 1} {
               puts $_g_puts_file "$msg"
            }
         }
         -E- {
            if {$G_PUTS_ERROR == 1} {
               puts $_g_puts_file "$msg"
            }
         }
         -D- {
            if {$G_PUTS_DEBUG == 1} {
               puts $_g_puts_dbg_file "$msg"
            }
         }
         default {
            if {$G_PUTS_DEFAULT == 1} {
               puts $_g_puts_file "$msg"
            }
         }
      }
   }

   
   if {[info exist MATCHED]} {

      set prefix [string range $input 0 8]
      
      if {$prefix == "-I- Found" || $prefix == "-I- Match"} {
         if {[info exist MATCHED]} {
            set newLine [lrange $input 2 end]
            set MATCHED "$MATCHED {$newLine}"
         }
      }
   }

   flush $_g_puts_file    
   if {[info exist ERRORS_LINKS]} {

      if {$G_PUTS_ENABLE == 1} {
         set prefix [string range $input 0 7]
         
         if {$prefix == "-E- Link"} {
            if {[info exist ERRORS_LINKS]} {
               set newLine [lrange $input 2 end]
               set modLine "[lindex $newLine 3] [lindex $newLine 4] -> [lindex $newLine 0] [lindex $newLine 1] [lrange $newLine 5 end]"
               foreach l $ERRORS_LINKS {
                  if {$l == $newLine} {
                     return;
                  }
                  if {$l == $modLine} {
                     return;
                  }
               }
               
               set ERRORS_LINKS "$ERRORS_LINKS {$newLine}"
            }
         }
      }
      
      set prefix [string range $input 0 10]
      
      if {$prefix == "-E- Already"} {
         if {[info exist ERRORS_LINKS]} {
            set newLine [lrange $input 1 end]
            foreach l $ERRORS_LINKS {
               if {$l == $newLine} {
                  return;
               }
            }
            set ERRORS_LINKS "$ERRORS_LINKS {$newLine}"
         }
      }
      
      set prefix [string range $input 0 7]
      
      if {$prefix == "-E- Guid"} {
         if {[info exist ERRORS_LINKS]} {
            set newLine [lrange $input 1 end]
            foreach l $ERRORS_LINKS {
               if {$l == $newLine} {
                  return;
               }
            }
            set ERRORS_LINKS "$ERRORS_LINKS {$newLine}"
         }
      }
   }
}

# create the logging thread if not already available
if {![tsv::exists g_puts_conf g_puts_thread]} {
   # we aere the logger thread
   tsv::set g_puts_conf g_puts_thread [thread::id]
   
   # initialize the log files.
   set _g_puts_file stdout
   set _g_puts_dbg_file stdout
}


package provide g_puts 1.0